@@ -28,20 +28,24 @@
 #include "../include/opl3.h"
 #include "../include/sb.h"
 
+EXPORT_NO_SYMBOLS;
 MODULE_DESCRIPTION("\
 Driver: Avance Logic ALS1X0\n\
-Driver: 3D Melody MF1000n\
 Card: Avance Logic ALS100\n\
 Card: Avance Logic ALS110\n\
 Card: Avance Logic ALS120\n\
+Card: Avance Logic ALS200\n\
+Card: 3D Melody MF1000\n\
 Card: Digimate 3D Sound\n\
 ISAPNP: ALS0001=@@@0001,@X@0001,@H@0001\n\
 ISAPNP: ALS0110=@@@1001,@X@1001,@H@1001\n\
 ISAPNP: ALS0120=@@@2001,@X@2001,@H@2001\n\
+ISAPNP: ALS0200=@@@0020,@X@0020,@H@0001\n\
 ISAPNP: RTL3000=@@@2001,@X@2001,@H@2001\n\
 ");
+MODULE_LICENSE("GPL");
 
-int snd_index[SND_CARDS] = SND_DEFAULT_IDX;		/* Index 1-MAX */
+int snd_index[SND_CARDS] = SND_DEFAULT_IDX;		/* Index 0-MAX */
 char *snd_id[SND_CARDS] = SND_DEFAULT_STR;		/* ID for this card */
 int snd_port[SND_CARDS] = SND_DEFAULT_PORT;		/* PnP setup */
 int snd_mpu_port[SND_CARDS] = SND_DEFAULT_PORT;		/* PnP setup */
@@ -112,6 +116,11 @@ static unsigned int snd_card_als100_pnpids[] = {
 	(ISAPNP_VENDOR('@','@','@')<<16)|ISAPNP_FUNCTION(0x2001),  /* audio */
 	(ISAPNP_VENDOR('@','X','@')<<16)|ISAPNP_FUNCTION(0x2001),  /* mpu401 */
 	(ISAPNP_VENDOR('@','H','@')<<16)|ISAPNP_FUNCTION(0x2001),  /* opl */
+	/* ALS200 */
+	(ISAPNP_VENDOR('A','L','S')<<16)|ISAPNP_DEVICE(0x0200),    /* device */
+	(ISAPNP_VENDOR('@','@','@')<<16)|ISAPNP_FUNCTION(0x0020),  /* audio */
+	(ISAPNP_VENDOR('@','X','@')<<16)|ISAPNP_FUNCTION(0x0020),  /* mpu401 */
+	(ISAPNP_VENDOR('@','H','@')<<16)|ISAPNP_FUNCTION(0x0001),  /* opl */
 	/* RTL3000 */
 	(ISAPNP_VENDOR('R','T','L')<<16)|ISAPNP_DEVICE(0x3000),    /* device */
 	(ISAPNP_VENDOR('@','@','@')<<16)|ISAPNP_FUNCTION(0x2001),  /* audio */
@@ -123,7 +132,7 @@ static unsigned int snd_card_als100_pnpids[] = {
 
 #define DRIVER_NAME	"snd-card-als100"
 
-#define snd_als100_printk(args...)	snd_printk(__FILE__": " ##args)
+#define snd_als100_printk(format, args...)	snd_printk(__FILE__": " format, ##args)
 
 
 static void snd_card_als100_use_inc(snd_card_t *card)
@@ -179,7 +188,7 @@ static int __init snd_card_als100_resources(int dev,
 					    snd_card_t *card)
 {
 	int error;
-	static int no_alternatives[] = {-1};
+	static long no_alternatives[] = {-1};
 
 	if (snd_register_ioport(card, snd_mpu_port[dev], 2,
 			DRIVER_NAME" - MPU-401", NULL) < 0)
@@ -362,10 +371,8 @@ static int __init snd_card_als100_probe(int dev, struct snd_card_als100 *acard)
 
 	rmidi = NULL;
 	if (snd_mpu_port[dev] > 0) {
-		if (snd_mpu401_uart_new(card, 0,
-				MPU401_HW_ALS100,
-				snd_mpu_port[dev],
-				snd_mpu_irq[dev],
+		if (snd_mpu401_uart_new(card, 0, MPU401_HW_ALS100,
+				snd_mpu_port[dev], snd_mpu_irq[dev],
 				&rmidi) < 0)
 			snd_als100_printk("no MPU-401device at 0x%x ?\n",
 				snd_mpu_port[dev]);
@@ -374,15 +381,15 @@ static int __init snd_card_als100_probe(int dev, struct snd_card_als100 *acard)
 	synth = NULL;
 	if (snd_fm_port[dev] > 0) {
 		if (snd_opl3_new(card, 0,
-				 snd_fm_port[dev], snd_fm_port[dev] + 2,
-				 OPL3_HW_AUTO, 0, &synth) < 0)
+				snd_fm_port[dev], snd_fm_port[dev] + 2,
+				OPL3_HW_AUTO, 0, &synth) < 0)
 			snd_als100_printk("no OPL device at 0x%x-0x%x ?\n",
 				snd_fm_port[dev], snd_fm_port[dev] + 2);
 	}
 
 	strcpy(card->abbreviation, "ALS100");
 	strcpy(card->shortname, "Avance Logic ALS100");
-	sprintf(card->longname, "%s soundcard, %s at 0x%x, irq %i, dma %i&%i",
+	sprintf(card->longname, "%s soundcard, %s at 0x%lx, irq %li, dma %li&%li",
 		card->shortname, codec->name, codec->port,
 		acard->irqptr->irq, acard->dma8ptr->dma, acard->dma16ptr->dma);
 	if (!snd_card_register(card)) {
@@ -417,11 +424,7 @@ static void __exit snd_card_als100_free(int dev)
 
 #endif
 
-#ifdef MODULE
-int __init init_module(void)
-#else
-int __init alsa_card_als100_init(void)
-#endif
+static int __init alsa_card_als100_init(void)
 {
 	int cards, dev;
 	struct snd_card_als100 *acard;
@@ -449,9 +452,7 @@ int __init alsa_card_als100_init(void)
 	return cards ? 0 : -ENODEV;
 }
 
-#ifdef MODULE
-
-void __exit cleanup_module(void)
+static void __exit alsa_card_als100_exit(void)
 {
 	int dev;
 
@@ -459,4 +460,5 @@ void __exit cleanup_module(void)
 		snd_card_als100_free(dev);
 }
 
-#endif
+module_init(alsa_card_als100_init)
+module_exit(alsa_card_als100_exit)
