@@ -26,50 +26,69 @@
 #include "../include/opl3.h"
 #include "../include/initval.h"
 
+EXPORT_NO_SYMBOLS;
 #ifdef CS4232
 MODULE_DESCRIPTION("\
 Driver: Cirrus Logic CS4232\n\
 Card: Generic CS4232\n\
 Card: Turtle Beach TBS-2000\n\
 Card: Turtle Beach Tropez Plus\n\
-Card: AudioSystem EWS64\n\
+Card: SIC CrystalWave 32\n\
+Card: TerraTec Maestro 32/96\n\
+ISAPNP: CSC0d32=CSC0000,CSC0010,PNPb006\n\
+ISAPNP: CSC1a32=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSC4232=CSC0000,CSC0002,CSC0003\n\
 ISAPNP: CSC7532=CSC0000,CSC0010,CSCb006\n\
 ISAPNP: CSC7632=CSC0000,CSC0010,PNPb006\n\
-ISAPNP: CSCa836=CSCa800,CSCa810,CSCa8a3\n\
+ISAPNP: CSCf032=CSC0000,CSC0010,CSC0003\n\
 ");
 #else
 MODULE_DESCRIPTION("\
 Driver: Cirrus Logic CS4236+\n\
-Card: Generic CS4235\n\
-Card: Generic CS4236B\n\
-Card: Generic CS4237B\n\
-Card: Generic CS4238B\n\
-Card: Generic CS4239\n\
+Card: Generic CS4235,6,7,9\n\
 Card: Genius Sound Maker 3DJ\n\
 Card: HP 6330 Onboard sound\n\
-Card: ACER AW35/Pro\n\
-Card: ACER AW37\n\
-Card: ACER AW37/Pro\n\
+Card: ACER AW3[57]/Pro\n\
 Card: EliteGroup P5TX-LA motherboard\n\
 Card: Dell Optiplex GX1\n\
+Card: Dell Workstation 400 Onboard\n\
+Card: AudioSystem EWS64\n\
 Card: Turtle Beach Malibu\n\
+Card: IBM Aptiva 2137 E24 Onboard\n\
 Card: Maxi Sound 16 PnP\n\
 Card: Gallant SC-70P\n\
+Card: NewClear 3D\n\
+ISAPNP: CSC0225=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSC0225=CSC0100,CSC0110,CSC0103\n\
 ISAPNP: CSC0437=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSC0735=CSC0000,CSC0010\n\
 ISAPNP: CSC0b35=CSC0000,CSC0010,CSC0003\n\
 ISAPNP: CSC1425=CSC0100,CSC0110\n\
 ISAPNP: CSC1525=CSC0100,CSC0110,CSC0103\n\
+ISAPNP: CSC1e37=CSC0000,CSC0010,CSC0003\n\
 ISAPNP: CSC4236=CSC0000,CSC0010,CSC0003\n\
 ISAPNP: CSC4237=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSC4336=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSC4536=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSC4637=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSC4837=CSC0000,CSC0010,CSC0003\n\
 ISAPNP: CSC6835=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSC6836=CSC0000,CSC0010,CSC0003\n\
 ISAPNP: CSC7537=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSC8025=CSC0100,CSC0110,CSC0103\n\
+ISAPNP: CSC8037=CSC0000,CSC0010,CSC0003\n\
 ISAPNP: CSC9836=CSC0000,CSC0010,CSC0003\n\
 ISAPNP: CSC9837=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSCa836=CSCa800,CSCa810,CSCa8a3\n\
+ISAPNP: CSCc835=CSC0000,CSC0010\n\
 ISAPNP: CSCd925=CSC0000,CSC0010,CSC0003\n\
 ISAPNP: CSCd937=CSC0000,CSC0010,CSC0003\n\
 ISAPNP: CSCe825=CSC0100,CSC0110\n\
+ISAPNP: CSCf235=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSCf238=CSC0000,CSC0010,CSC0003\n\
 ");
 #endif
+MODULE_LICENSE("GPL");
 
 #if defined(CS4232)
 #define IDENT "CS4232"
@@ -79,7 +98,7 @@ ISAPNP: CSCe825=CSC0100,CSC0110\n\
 #define CS4236
 #endif
 
-int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	/* Index 1-MAX */
+int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	/* Index 0-MAX */
 char *snd_id[SND_CARDS] = SND_DEFAULT_STR;	/* ID for this card */
 int snd_port[SND_CARDS] = SND_DEFAULT_PORT;	/* PnP setup */
 int snd_cport[SND_CARDS] = SND_DEFAULT_PORT;	/* PnP setup */
@@ -92,7 +111,9 @@ int snd_dma1[SND_CARDS] = SND_DEFAULT_DMA;	/* 0,1,3,5,6,7 */
 int snd_dma1_size[SND_CARDS] = SND_DEFAULT_DMA_SIZE;	/* 8,16,32,64,128 */
 int snd_dma2[SND_CARDS] = SND_DEFAULT_DMA;	/* 0,1,3,5,6,7 */
 int snd_dma2_size[SND_CARDS] = SND_DEFAULT_DMA_SIZE;	/* 8,16,32,64,128 */
+#ifdef __ISAPNP__
 int snd_isapnp[SND_CARDS] = {[0 ... (SND_CARDS - 1)] = 1};
+#endif
 MODULE_PARM(snd_index, "1-" __MODULE_STRING(SND_CARDS) "i");
 MODULE_PARM_DESC(snd_index, "Index value for " IDENT " soundcard.");
 MODULE_PARM(snd_id, "1-" __MODULE_STRING(SND_CARDS) "s");
@@ -119,8 +140,10 @@ MODULE_PARM(snd_dma2, "1-" __MODULE_STRING(SND_CARDS) "i");
 MODULE_PARM_DESC(snd_dma2, "DMA2 # for " IDENT " driver. [DMA]");
 MODULE_PARM(snd_dma2_size, "1-" __MODULE_STRING(SND_CARDS) "i");
 MODULE_PARM_DESC(snd_dma2_size, "Size of DMA2 # for " IDENT " driver. [DMA_SIZE]");
+#ifdef __ISAPNP__
 MODULE_PARM(snd_isapnp, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_isapnp, "Enable/disable ISA PnP for " IDENT " driver. [BOOL]");
+MODULE_PARM_DESC(snd_isapnp, "ISA PnP detection for " IDENT " soundcard. [BOOL]");
+#endif
 
 static struct snd_card_cs4236 {
 	snd_irq_t *irqptr;
@@ -144,6 +167,21 @@ static struct snd_card_cs4236 {
 #ifdef __ISAPNP__
 #ifdef CS4232
 static unsigned int snd_card_pnpids[] = {
+	/* Philips PCA70PS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x0d32),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('P','N','P')<<16)|ISAPNP_FUNCTION(0xb006), /* MPU */
+	/* TerraTec Maestro 32/96 (CS4232) */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x1a32),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
+	/* HP Omnibook 5500 onboard sound */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x4232),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0002), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
 	/* Turtle Beach TBS-2000 (CS4232) */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x7532),   /* DEVICE */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
@@ -154,27 +192,42 @@ static unsigned int snd_card_pnpids[] = {
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
 	(ISAPNP_VENDOR('P','N','P')<<16)|ISAPNP_FUNCTION(0xb006), /* MPU */
-	/* AudioSystem EWS64 (CS4232) */
-	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0xa836),   /* DEVICE */
-	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0xa800), /* WSS */
-	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0xa810), /* CTRL */
-	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0xa8a3), /* MPU */
+	/* SIC CrystalWave 32 (CS4232) */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0xf032),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
 	/* --- */
 	0	/* end */
 };
 #endif
 #ifdef CS4236
 static unsigned int snd_card_pnpids[] = {
+	/* Intel Marlin Spike Motherboard - CS4235 */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x0225),	  /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
+	/* Intel Marlin Spike Motherboard (#2) - CS4235 */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x0225),	  /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0100), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0110), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0103), /* MPU */
 	/* Genius Sound Maker 3DJ - CS4237B */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x0437),   /* DEVICE */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
+	/* Digital PC 5000 Onboard - CS4236B */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x0735),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	0,							  /* MPU */
 	/* some uknown CS4236B */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x0b35),   /* DEVICE */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
-	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */	
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
 	/* CS4235 on mainboard without MPU */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x1425),	  /* DEVICE */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0100), /* WSS */
@@ -185,6 +238,11 @@ static unsigned int snd_card_pnpids[] = {
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0100), /* WSS */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0110), /* CTRL */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0103), /* MPU */
+	/* Crystal Computer TidalWave128 */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x1e37),	  /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
 	/* ACER AW37 - CS4235 */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x4236),   /* DEVICE */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
@@ -195,26 +253,71 @@ static unsigned int snd_card_pnpids[] = {
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
+	/* Crystal 3D - CS4237B */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x4336),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
+	/* Typhoon Soundsystem PnP - CS4236B */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x4536),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
+	/* Unknown - CS4237B */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x4637),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
+	/* NewClear 3D - CX4237B-XQ3 */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x4837),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
 	/* Dell Optiplex GX1 - CS4236B */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x6835),   /* DEVICE */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
+	/* Dell Workstation 400 Onboard - CS4236B */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x6836),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
 	/* Turtle Beach Malibu - CS4237B */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x7537),   /* DEVICE */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
+	/* Onboard - CS4235 */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x8025),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0100), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0110), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0103), /* MPU */
+	/* IBM Aptiva 2137 E24 Onboard - CS4237B */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x8037),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
 	/* Maxi Sound 16 PnP - CS4236B */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x9836),   /* DEVICE */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
+	/* IBM IntelliStation M Pro motherboard */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0xc835),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	0,							  /* MPU */
 	/* Gallant SC-70P */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x9837),   /* DEVICE */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
+	/* AudioSystem EWS64 (CS4236) */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0xa836),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0xa800), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0xa810), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0xa8a3), /* MPU */
 	/* ACER AW37/Pro - CS4235 */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0xd925),   /* DEVICE */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
@@ -230,6 +333,16 @@ static unsigned int snd_card_pnpids[] = {
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0100), /* WSS */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0110), /* CTRL */
 	0,							  /* MPU */
+	/* CS4236 */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0xf235),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
+	/* CS4236 */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0xf238),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
 	/* --- */
 	0	/* end */
 };
@@ -327,8 +440,8 @@ static int __init snd_card_cs4236_resources(int dev,
 					    struct snd_card_cs4236 *acard,
 					    snd_card_t * card)
 {
-	static int possible_irqs[] = {-1};
-	static int possible_dmas[] = {-1};
+	static long possible_irqs[] = {-1};
+	static long possible_dmas[] = {-1};
 	int err;
 
 	if ((err = snd_register_interrupt(card, IDENT,
@@ -407,9 +520,9 @@ static int __init snd_card_cs4236_isapnp(int dev, struct snd_card_cs4236 *acard)
 		acard->mpu = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
 		if (acard->mpu == NULL) {
 			snd_printdd("isapnp failed for " IDENT " (MPU) - %i\n", idx);
-			goto __again;		/* maybe we have another config */
+			snd_mpu_port[dev] = -1;
 		} else if (acard->mpu->active) {
-			goto __again;
+			snd_mpu_port[dev] = -1;
 		}
 	} else {
 		snd_mpu_port[dev] = -1;
@@ -487,9 +600,9 @@ static void snd_card_cs4236_deactivate(struct snd_card_cs4236 *acard)
 	if (acard->wss)
 		acard->wss->deactivate(acard->wss);
 	if (acard->ctrl)
-		acard->wss->deactivate(acard->ctrl);
+		acard->ctrl->deactivate(acard->ctrl);
 	if (acard->mpu)
-		acard->wss->deactivate(acard->mpu);
+		acard->mpu->deactivate(acard->mpu);
 }
 #endif
 
@@ -561,8 +674,13 @@ static int __init snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 	acard->pcm_status_reg = codec->port + 2;
 
 #ifdef CS4236
-	if (snd_cs4236_new_mixer(pcm, 0, &mixer) < 0)
-		goto __nodev;
+	if (codec->hardware & CS4231_HW_CS4236_MASK) {
+		if (snd_cs4236_new_mixer(pcm, 0, &mixer) < 0)
+			goto __nodev;
+	} else {
+		if (snd_cs4231_new_mixer(pcm, 0, &mixer) < 0)
+			goto __nodev;
+	}
 #else
 	if (snd_cs4231_new_mixer(pcm, 0, &mixer) < 0)
 		goto __nodev;
@@ -587,13 +705,13 @@ static int __init snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 	strcpy(card->abbreviation, "CS4232");
 #endif
 	strcpy(card->shortname, pcm->name);
-	sprintf(card->longname, "%s at 0x%x, irq %i, dma %i",
+	sprintf(card->longname, "%s at 0x%lx, irq %li, dma %li",
 		pcm->name,
 		codec->port,
 		acard->irqptr->irq,
 		acard->dma1ptr->dma);
 	if (acard->dma2ptr)
-		sprintf(card->longname + strlen(card->longname), "&%i",
+		sprintf(card->longname + strlen(card->longname), "&%li",
 			acard->dma2ptr->dma);
 	if (!snd_card_register(card)) {
 		acard->card = card;
@@ -609,16 +727,7 @@ static int __init snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 	return -ENXIO;
 }
 
-#ifdef MODULE
-int __init init_module(void)
-#else
-#ifdef CS4232
-int __init alsa_card_cs4232_init(void)
-#endif
-#ifdef CS4236
-int __init alsa_card_cs4236_init(void)
-#endif
-#endif
+static int __init alsa_card_cs423x_init(void)
 {
 	int dev, cards;
 	struct snd_card_cs4236 *acard;
@@ -653,9 +762,7 @@ int __init alsa_card_cs4236_init(void)
 	return 0;
 }
 
-#ifdef MODULE
-
-void __exit cleanup_module(void)
+static void __exit alsa_card_cs423x_exit(void)
 {
 	int idx;
 	struct snd_card_cs4236 *acard;
@@ -672,4 +779,5 @@ void __exit cleanup_module(void)
 	}
 }
 
-#endif
+module_init(alsa_card_cs423x_init)
+module_exit(alsa_card_cs423x_exit)
