@@ -1,9 +1,12 @@
 /*
  *  Driver for ESS Solo-1 (ES1938, ES1946) soundcard
  *  Copyright (c) by Jaromir Koutek <miri@punknet.cz>,
- *                   Jaroslav Kysela <perex@suse.cz>
+ *                   Jaroslav Kysela <perex@suse.cz>,
+ *                   Thomas Sailer <sailer@ife.ee.ethz.ch>,
+ *                   Abramo Bagnara <abramo@alsa-project.org>,
+ *                   Markus Gruber <gruber@eikon.tum.de>
  * 
- * Rewritted from sonicvibes.c source.
+ * Rewritten from sonicvibes.c source.
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -24,18 +27,27 @@
 
 #define SND_MAIN_OBJECT_FILE
 #include "../include/driver.h"
-#include "../include/esssolo1.h"
+#include "../include/es1938.h"
 #include "../include/opl3.h"
 #include "../include/initval.h"
 
+EXPORT_NO_SYMBOLS;
+MODULE_DESCRIPTION("\
+Driver: ESS Solo-1\n\
+Card: Generic ES1938\n\
+Card: TerraTec 128i PCI\n\
+PCI: 0x125d=0x1969\n\
+");
+MODULE_LICENSE("GPL");
+
 #ifndef PCI_VENDOR_ID_ESS
-#define PCI_VENDOR_ID_ESS             0x125d
+#define PCI_VENDOR_ID_ESS		0x125d
 #endif
 #ifndef PCI_DEVICE_ID_ESS_ES1938
-#define PCI_DEVICE_ID_ESS_ES1938  0x1969
+#define PCI_DEVICE_ID_ESS_ES1938	0x1969
 #endif
 
-int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	/* Index 1-MAX */
+int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	/* Index 0-MAX */
 char *snd_id[SND_CARDS] = SND_DEFAULT_STR;	/* ID for this card */
 MODULE_PARM(snd_index, "1-" __MODULE_STRING(SND_CARDS) "i");
 MODULE_PARM_DESC(snd_index, "Index value for ESS Solo-1 soundcard.");
@@ -48,9 +60,8 @@ struct snd_solo {
 	snd_dma_t *dma1ptr;
 	snd_dma_t *dma2ptr;
 	snd_card_t *card;
-	esssolo_t *solo;
+	es1938_t *solo;
 	snd_pcm_t *pcm;
-	snd_pcm_t *pcm2;
 	snd_kmixer_t *mixer;
 #ifdef USE_MIDI
 	snd_rawmidi_t *midi_uart;
@@ -70,7 +81,7 @@ static void snd_solo_use_dec(snd_card_t * card)
 	MOD_DEC_USE_COUNT;
 }
 
-static int snd_solo_detect(snd_card_t * card, struct snd_solo *scard)
+static int __init snd_solo_detect(snd_card_t * card, struct snd_solo *scard)
 {
 	int idx;
 
@@ -79,38 +90,17 @@ static int snd_solo_detect(snd_card_t * card, struct snd_solo *scard)
 					  scard->pci)) == NULL)
 		return -ENODEV;
 	for (idx = 0; idx < 5; idx++) {
-#ifdef NEW_PCI
-		if (scard->pci->resource[idx].start == 0 ||
-		    !(scard->pci->resource[idx].flags & PCI_BASE_ADDRESS_SPACE_IO))
-#else
-		if (scard->pci->base_address[idx] == 0 ||
-		    (scard->pci->base_address[idx] & PCI_BASE_ADDRESS_SPACE) != PCI_BASE_ADDRESS_SPACE_IO)
-#endif
+		if (pci_resource_start(scard->pci, idx) == 0 ||
+		    !(pci_resource_flags(scard->pci, idx) & IORESOURCE_IO))
 			return -ENODEV;
 	}
-#ifdef NEW_PCI
-	if (snd_register_ioport(card, scard->pci->resource[1].start, 0x10, "ESS Solo-1 SB", NULL) < 0)
-#else
-	if (snd_register_ioport(card, scard->pci->base_address[1] & ~PCI_BASE_ADDRESS_SPACE, 0x10, "ESS Solo-1 SB", NULL) < 0)
-#endif
+	if (snd_register_ioport(card, pci_resource_start(scard->pci, 1), 0x10, "ESS Solo-1 SB", NULL) < 0)
 		goto __nodev;
-#ifdef NEW_PCI
-	if (snd_register_ioport(card, scard->pci->resource[2].start, 0x10, "ESS Solo-1 VC (DMA)", NULL) < 0)
-#else
-	if (snd_register_ioport(card, scard->pci->base_address[2] & ~PCI_BASE_ADDRESS_SPACE, 0x10, "ESS Solo-1 VC (DMA)", NULL) < 0)
-#endif
+	if (snd_register_ioport(card, pci_resource_start(scard->pci, 2), 0x10, "ESS Solo-1 VC (DMA)", NULL) < 0)
 		goto __nodev;
-#ifdef NEW_PCI
-	if (snd_register_ioport(card, scard->pci->resource[3].start, 4, "ESS Solo-1 MIDI", NULL) < 0)
-#else
-	if (snd_register_ioport(card, scard->pci->base_address[3] & ~PCI_BASE_ADDRESS_SPACE, 4, "ESS Solo-1 MIDI", NULL) < 0)
-#endif
+	if (snd_register_ioport(card, pci_resource_start(scard->pci, 3), 4, "ESS Solo-1 MIDI", NULL) < 0)
 		goto __nodev;
-#ifdef NEW_PCI
-	if (snd_register_ioport(card, scard->pci->resource[4].start, 4, "ESS Solo-1 Game port", NULL) < 0)
-#else
-	if (snd_register_ioport(card, scard->pci->base_address[4] & ~PCI_BASE_ADDRESS_SPACE, 4, "ESS Solo-1 Game port", NULL) < 0)
-#endif
+	if (snd_register_ioport(card, pci_resource_start(scard->pci, 4), 4, "ESS Solo-1 Game port", NULL) < 0)
 		goto __nodev;
 	return 0;
       __nodev:
@@ -122,14 +112,14 @@ static void snd_solo_interrupt2(int irq, void *dev_id, struct pt_regs *regs)
 {
 	struct snd_solo *scard = (struct snd_solo *) dev_id;
 
-	if (!scard || !scard->solo)
+	if (scard == NULL || scard->solo == NULL)
 		return;
 	snd_solo_interrupt(scard->solo);
 }
 
-#define SOLO_DMA_SIZE 32
+#define SOLO_DMA_SIZE 64
 
-static int snd_solo_resources(snd_card_t * card, struct snd_solo *scard, int dev)
+static int __init snd_solo_resources(snd_card_t * card, struct snd_solo *scard, int dev)
 {
 	int err;
 
@@ -138,7 +128,7 @@ static int snd_solo_resources(snd_card_t * card, struct snd_solo *scard, int dev
 					  NULL, &scard->irqptr)) < 0)
 		return err;
 	if ((err = snd_register_dma_channel(card, "ESS Solo-1 DMA 1", 0,
-					    SND_DMA_TYPE_PCI,
+					    SND_DMA_TYPE_PCI_16MB,
 			      SOLO_DMA_SIZE, NULL, &scard->dma1ptr)) < 0)
 		return err;
 	if ((err = snd_register_dma_channel(card, "ESS Solo-1 DMA 2", 1,
@@ -148,11 +138,10 @@ static int snd_solo_resources(snd_card_t * card, struct snd_solo *scard, int dev
 	return 0;
 }
 
-static int snd_solo_probe(int dev, struct snd_solo *scard)
+static int __init snd_solo_probe(int dev, struct snd_solo *scard)
 {
 	snd_card_t *card;
 	snd_pcm_t *pcm = NULL;
-	snd_pcm_t *pcm2 = NULL;
 	snd_kmixer_t *mixer = NULL;
 #ifdef USE_MIDI
 	snd_rawmidi_t *midi_uart = NULL;
@@ -161,10 +150,10 @@ static int snd_solo_probe(int dev, struct snd_solo *scard)
 
 	card = snd_card_new(snd_index[dev], snd_id[dev],
 			    snd_solo_use_inc, snd_solo_use_dec);
-	if (!card)
+	if (card == NULL)
 		return -ENOMEM;
 	card->type = SND_CARD_TYPE_ESS_ES1938;
-	card->static_data = scard;
+	scard->pci = NULL;
 	do {
 		if (!snd_solo_detect(card, scard))
 			break;
@@ -177,42 +166,18 @@ static int snd_solo_probe(int dev, struct snd_solo *scard)
 		snd_card_free(card);
 		return -ENODEV;
 	}
-	scard->solo = snd_solo_create(card, scard->pci, scard->dma1ptr,
-				    scard->dma2ptr, scard->irqptr, 0, 0);
-	if (!scard->solo)
+	if (snd_solo_create(card, scard->pci, scard->dma1ptr,
+			    scard->dma2ptr, scard->irqptr, 0, 0,
+			    &scard->solo) < 0)
 		goto __nodev;
-	mixer = snd_solo_mixer(scard->solo, 1, 0);
-	if (!mixer)
+	if (snd_solo_new_pcm(scard->solo, 0, &pcm) < 0)
 		goto __nodev;
-#undef USE_PCM2
-#ifdef USE_PCM2
-	pcm = snd_solo_pcm(scard->solo);
-	if (!pcm)
+	if (snd_solo_new_mixer(scard->solo, 0, pcm, &mixer) < 0)
 		goto __nodev;
-#endif
-	pcm2 = snd_solo_pcm2(scard->solo);
-	if (!pcm2)
-		goto __nodev;
-	if (snd_mixer_register(mixer, 0) < 0)
-		goto __nodev;
-#ifdef USE_PCM2
-	if (snd_pcm_register(pcm, 1) < 0) {
-		snd_mixer_unregister(mixer);
-		mixer = NULL;	/* !!! FIXME !!! */
-		goto __nodev;
-	}
-#endif
-	if (snd_pcm_register(pcm2, 0) < 0) {
-		snd_pcm_unregister(pcm);
-		pcm = NULL;	/* !!! FIXME !!! */
-		snd_mixer_unregister(mixer);
-		mixer = NULL;	/* !!! FIXME !!! */
-		goto __nodev;
-	}
 
 	strcpy(card->abbreviation, "Solo-1");
 	strcpy(card->shortname, "ESS Solo-1");
-	sprintf(card->longname, "%s rev %i, irq %i",
+	sprintf(card->longname, "%s rev %i, irq %li",
 		card->shortname,
 		scard->solo->revision,
 		scard->irqptr->irq);
@@ -221,7 +186,6 @@ static int snd_solo_probe(int dev, struct snd_solo *scard)
 		scard->card = card;
 		scard->mixer = mixer;
 		scard->pcm = pcm;
-		scard->pcm2 = pcm2;
 #ifdef USE_MIDI
 		scard->midi_uart = scard->solo->rmidi = midi_uart;
 #endif
@@ -231,73 +195,35 @@ static int snd_solo_probe(int dev, struct snd_solo *scard)
 	goto __nodev;
 
       __nodev:
-	if (synth)
-		snd_hwdep_free(synth);
-#ifdef USE_MIDI
-	if (midi_uart)
-		snd_rawmidi_free(midi_uart);
-#endif
-	if (mixer)
-		snd_mixer_free(mixer);
-	if (pcm)
-		snd_pcm_free(pcm);
-	if (pcm2)
-		snd_pcm_free(pcm2);
-	if (scard->solo) {
-		snd_solo_free(scard->solo);
-		scard->solo = NULL;
-	}
 	snd_card_free(card);
 	return -ENXIO;
 }
 
-static int snd_solo_free2(int dev)
+#ifdef MODULE
+
+static int __exit snd_solo_free2(int dev)
 {
 	struct snd_solo *scard;
-	snd_pcm_t *pcm, *pcm2;
 
 	scard = snd_solo_cards[dev];
 	snd_solo_cards[dev] = NULL;
 	if (scard) {
 		snd_card_unregister(scard->card);
-		if (scard->synth)
-			snd_hwdep_unregister(scard->synth);
-#ifdef USE_MIDI
-		if (scard->midi_uart) {
-			scard->solo->rmidi = NULL;
-			snd_rawmidi_unregister(scard->midi_uart);
-		}
-#endif
-		if (scard->mixer)
-			snd_mixer_unregister(scard->mixer);
-		if (scard->pcm) {
-			pcm = scard->pcm;
-			scard->pcm = NULL;	/* turn off interrupts */
-			snd_pcm_unregister(pcm);
-		}
-		if (scard->pcm2) {
-			pcm2 = scard->pcm2;
-			scard->pcm2 = NULL;	/* turn off interrupts */
-			snd_pcm_unregister(pcm2);
-		}
-		if (scard->solo) {
-			snd_solo_free(scard->solo);
-			scard->solo = NULL;
-		}
-		snd_card_free(scard->card);
 		snd_kfree(scard);
 	}
 	return 0;
 }
 
-int init_module(void)
+#endif
+
+static int __init alsa_card_es1938_init(void)
 {
 	int dev, cards;
 	struct snd_solo *scard;
 
 	for (dev = cards = 0; dev < SND_CARDS; dev++) {
 		scard = (struct snd_solo *) snd_kcalloc(sizeof(struct snd_solo), GFP_KERNEL);
-		if (!scard)
+		if (scard == NULL)
 			continue;
 		if (snd_solo_probe(dev, scard) < 0) {
 			snd_kfree(scard);
@@ -307,16 +233,21 @@ int init_module(void)
 		cards++;
 	}
 	if (!cards) {
+#ifdef MODULE
 		snd_printk("ESS Solo-1 soundcard #%i not found or device busy\n", dev + 1);
+#endif
 		return -ENODEV;
 	}
 	return 0;
 }
 
-void cleanup_module(void)
+static void __exit alsa_card_es1938_exit(void)
 {
 	int dev;
 
 	for (dev = 0; dev < SND_CARDS; dev++)
 		snd_solo_free2(dev);
 }
+
+module_init(alsa_card_es1938_init)
+module_exit(alsa_card_es1938_exit)
