@@ -28,11 +28,13 @@
 #include "../include/ad1848.h"
 #include "../include/initval.h"
 
+EXPORT_NO_SYMBOLS;
 MODULE_DESCRIPTION("\
 Driver: Aztech Sound Galaxy\n\
 ");
+MODULE_LICENSE("GPL");
 
-int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	/* Index 1-MAX */
+int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	/* Index 0-MAX */
 char *snd_id[SND_CARDS] = SND_DEFAULT_STR;	/* ID for this card */
 int snd_sbport[SND_CARDS] = SND_DEFAULT_PORT;	/* 0x220,0x240 */
 int snd_wssport[SND_CARDS] = SND_DEFAULT_PORT;	/* 0x530,0xe80,0xf40,0x604 */
@@ -285,8 +287,8 @@ static void snd_sgalaxy_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 static int snd_sgalaxy_resources(int dev, struct snd_sgalaxy *acard, 
 				 snd_card_t *card)
 {
-	static int possible_irqs[] = {7, 9, 10, 11, -1};
-	static int possible_dmas[] = {1, 3, 0, -1};
+	static long possible_irqs[] = {7, 9, 10, 11, -1};
+	static long possible_dmas[] = {1, 3, 0, -1};
 	int err;
 
 #if 0
@@ -461,7 +463,7 @@ static int __init snd_sgalaxy_probe(int dev, struct snd_sgalaxy *acard)
 #endif
 	strcpy(card->abbreviation, "Sound Galaxy");
 	strcpy(card->shortname, "Sound Galaxy");
-	sprintf(card->longname, "Sound Galaxy at 0x%x, irq %i, dma %i",
+	sprintf(card->longname, "Sound Galaxy at 0x%x, irq %li, dma %li",
 		snd_wssport[dev],
 		acard->irqptr->irq,
 		acard->dma1ptr->dma);
@@ -481,11 +483,7 @@ static int __init snd_sgalaxy_probe(int dev, struct snd_sgalaxy *acard)
 	return -ENXIO;
 }
 
-#ifdef MODULE
-int init_module(void)
-#else
-int __init alsa_card_sgalaxy_init(void)
-#endif
+static int __init alsa_card_sgalaxy_init(void)
 {
 	int dev, cards;
 	struct snd_sgalaxy *acard;
@@ -529,9 +527,7 @@ int __init alsa_card_sgalaxy_init(void)
 	return 0;
 }
 
-#ifdef MODULE
-
-void __exit cleanup_module(void)
+static void __exit alsa_card_sgalaxy_exit(void)
 {
 	int idx;
 	struct snd_sgalaxy *acard;
@@ -546,4 +542,5 @@ void __exit cleanup_module(void)
 	}
 }
 
-#endif
+module_init(alsa_card_sgalaxy_init)
+module_exit(alsa_card_sgalaxy_exit)
