@@ -22,9 +22,9 @@
  *
  */
 
-#include "pcm1.h"
+#include "pcm.h"
 #include "mixer.h"
-#include "midi.h"
+#include "rawmidi.h"
 
 #define SB_HW_AUTO		0
 #define SB_HW_10		1
@@ -32,7 +32,8 @@
 #define SB_HW_201		3
 #define SB_HW_PRO		4
 #define SB_HW_16		5
-#define SB_HW_ALS100		6	/* Avance Logic ALS100 chip */
+#define SB_HW_16CSP		6	/* SB16 with CSP chip */
+#define SB_HW_ALS100		7	/* Avance Logic ALS100 chip */
 
 #define SB_MODE8_HALT		0
 #define SB_MODE8_PLAYBACK	1
@@ -57,7 +58,7 @@
 typedef struct snd_stru_sbmixer sbmixer_t;
 
 struct snd_stru_sbmixer {
-	unsigned short port;
+	unsigned long port;
 
 	snd_kmixer_element_t *me_mux_mic;
 	snd_kmixer_element_t *me_mux_line;
@@ -92,7 +93,7 @@ struct snd_stru_sbmixer {
 	snd_kmixer_element_t *me_in_synth;
 	snd_kmixer_element_t *me_vol_synth;
 	snd_kmixer_element_t *me_sw3_synth_input;
-	
+
 	snd_kmixer_element_t *me_vol_pcm;
 
 	snd_kmixer_element_t *me_vol_ogain;
@@ -100,19 +101,20 @@ struct snd_stru_sbmixer {
 	snd_kmixer_element_t *me_tone;
 
 	snd_kmixer_element_t *me_out_master;
+	snd_kmixer_element_t *me_sw1_3dse;
 	snd_kmixer_element_t *me_vol_master;
 	
 	spinlock_t lock;
 };
 
 struct snd_stru_sbdsp {
-	unsigned short port;		/* base port of DSP chip */
-	unsigned short mpu_port;	/* MPU port for SB DSP 4.0+ */
-	unsigned short irq;		/* IRQ number of DSP chip */
+	unsigned long port;		/* base port of DSP chip */
+	unsigned long mpu_port;		/* MPU port for SB DSP 4.0+ */
+	unsigned long irq;		/* IRQ number of DSP chip */
 	snd_irq_t * irqptr;		/* IRQ pointer */
-	unsigned short dma8;		/* 8-bit DMA */
+	unsigned long dma8;		/* 8-bit DMA */
 	snd_dma_t * dma8ptr;		/* 8-bit DMA pointer */
-	unsigned short dma16;		/* 16-bit DMA */
+	unsigned long dma16;		/* 16-bit DMA */
 	snd_dma_t * dma16ptr;		/* 16-bit DMA pointer */
 	unsigned short version;		/* version of DSP chip */
 	unsigned short hardware;	/* see to SB_HW_XXXX */
@@ -121,7 +123,11 @@ struct snd_stru_sbdsp {
 	unsigned int mode8;		/* current mode of stream */
 	unsigned char speed8;		/* input speed */
 	unsigned char fmt8;		/* format */
-	unsigned int count8;		/* size of one block for SB 1.0 */
+	struct timer_list midi_timer;
+	unsigned int p_dma_size;
+	unsigned int p_frag_size;
+	unsigned int c_dma_size;
+	unsigned int c_frag_size;
 
 	unsigned int mode16;		/* current 16-bit mode of streams */
 	unsigned int force_mode16;	/* force 16-bit mode of streams */
@@ -130,12 +136,17 @@ struct snd_stru_sbdsp {
 
 	char name[32];
 
+#ifdef CONFIG_SND_SB16_CSP
+	void *csp_callbacks;
+	void *csp_private_data;
+	unsigned int csp_acquired;
+#endif
+
 	snd_card_t *card;
 	snd_pcm_t *pcm;
 	snd_pcm_subchn_t *playback_subchn;
-	snd_pcm1_subchn_t *playback_subchn1;
 	snd_pcm_subchn_t *capture_subchn;
-	snd_pcm1_subchn_t *capture_subchn1;
+	snd_kmixer_t *kmixer;
 
 	spinlock_t reg_lock;
 	spinlock_t open8_lock;
@@ -211,11 +222,11 @@ typedef struct snd_stru_sbdsp sbdsp_t;
 #define SB_DSP_ESS_GET_VERSION	0xe7
 #define SB_DSP_ESS_EXTENDED	0xc6
 
-#define SB_DSP_CAPTURE_SOURCE    0x0c
-#define SB_DSP_MIXS_MIC0        0x00	/* same as MIC */
-#define SB_DSP_MIXS_MIC         0x01
-#define SB_DSP_MIXS_CD          0x03
-#define SB_DSP_MIXS_LINE        0x07
+#define SB_DSP_CAPTURE_SOURCE	0x0c
+#define SB_DSP_MIXS_MIC0	0x00	/* same as MIC */
+#define SB_DSP_MIXS_MIC		0x01
+#define SB_DSP_MIXS_CD		0x03
+#define SB_DSP_MIXS_LINE	0x07
 
 /* registers for SB PRO mixer */
 #define SB_DSP_MASTER_DEV	0x22
@@ -240,8 +251,32 @@ typedef struct snd_stru_sbdsp sbdsp_t;
 #define SB_DSP4_OGAIN_DEV	0x41
 #define SB_DSP4_MIC_AGC		0x43
 
-/* additional registers for SB AWE32/64 mixer */
-#define SB_DSP4_3D		0x90
+/* additional registers for SB 16 mixer */
+#define SB_DSP4_IRQSETUP	0x80
+#define SB_DSP4_DMASETUP	0x81
+#define SB_DSP4_IRQSTATUS	0x82
+#define SB_DSP4_MPUSETUP	0x84
+
+#define SB_DSP4_3DSE		0x90
+
+/* IRQ setting bitmap */
+#define SB_IRQSETUP_IRQ9	0x01
+#define SB_IRQSETUP_IRQ5	0x02
+#define SB_IRQSETUP_IRQ7	0x04
+#define SB_IRQSETUP_IRQ10	0x08
+
+/* IRQ types */
+#define SB_IRQTYPE_8BIT		0x01
+#define SB_IRQTYPE_16BIT	0x02
+#define SB_IRQTYPE_MPUIN	0x04
+
+/* DMA setting bitmap */
+#define SB_DMASETUP_DMA0	0x01
+#define SB_DMASETUP_DMA1	0x02
+#define SB_DMASETUP_DMA3	0x08
+#define SB_DMASETUP_DMA5	0x20
+#define SB_DMASETUP_DMA6	0x40
+#define SB_DMASETUP_DMA7	0x80
 
 /*
  *
@@ -263,33 +298,46 @@ extern void snd_sb16dsp_free(void *);
 extern void snd_sb8dsp_interrupt(snd_pcm_t * pcm);
 extern void snd_sb16dsp_interrupt(snd_pcm_t * pcm, unsigned short status);
 
-extern snd_pcm_t *snd_sb8dsp_new_device(snd_card_t * card,
-				        unsigned short port,
-				        snd_irq_t * irqptr,
-				        snd_dma_t * dma8ptr,
-				        unsigned short hardware);
-extern snd_pcm_t *snd_sb16dsp_new_device(snd_card_t * card,
-				         unsigned short port,
-				         snd_irq_t * irqptr,
-				         snd_dma_t * dma8ptr,
-				         snd_dma_t * dma16ptr,
-				         unsigned short hardware);
+extern int snd_sb8dsp_new_pcm(snd_card_t * card,
+			      int device,
+			      unsigned long port,
+			      snd_irq_t * irqptr,
+			      snd_dma_t * dma8ptr,
+			      unsigned short hardware,
+			      snd_pcm_t ** rpcm);
+extern int snd_sb16dsp_new_pcm(snd_card_t * card,
+			       int device,
+			       unsigned long port,
+			       snd_irq_t * irqptr,
+			       snd_dma_t * dma8ptr,
+			       snd_dma_t * dma16ptr,
+			       unsigned short hardware,
+			       snd_pcm_t ** rpcm);
 extern int snd_sb8dsp_probe(snd_pcm_t * pcm);
 extern int snd_sb16dsp_probe(snd_pcm_t * pcm);
 extern int snd_sb16dsp_configure(snd_pcm_t * pcm);
-extern snd_kmixer_t *snd_sb8dsp_new_mixer(snd_card_t * card,
-					  sbmixer_t * sbmix,
-					  unsigned short hardware,
-					  int pcm_dev);
-extern snd_kmixer_t *snd_sb16dsp_new_mixer(snd_card_t * card,
-					   sbmixer_t * sbmix,
-					   unsigned short hardware,
-					   int pcm_dev);
+extern int snd_sb8dsp_new_mixer(sbdsp_t * codec,
+			        int device,
+				snd_pcm_t * pcm,
+				snd_kmixer_t ** rmixer);
+extern int snd_sb16dsp_new_mixer(sbdsp_t * codec,
+				 int device,
+				 snd_pcm_t * pcm,
+				 snd_kmixer_t ** rmixer);
+
+extern int snd_sb8_playback_open(void *private_data, snd_pcm_subchn_t *subchn);
+extern int snd_sb8_capture_open(void *private_data, snd_pcm_subchn_t *subchn);
+extern int snd_sb8_playback_close(void *private_data, snd_pcm_subchn_t *subchn);
+extern int snd_sb8_capture_close(void *private_data, snd_pcm_subchn_t *subchn);
+extern int snd_sb16_playback_open(void *private_data, snd_pcm_subchn_t *subchn);
+extern int snd_sb16_capture_open(void *private_data, snd_pcm_subchn_t *subchn);
+extern int snd_sb16_playback_close(void *private_data, snd_pcm_subchn_t *subchn);
+extern int snd_sb16_capture_close(void *private_data, snd_pcm_subchn_t *subchn);
 
 extern void snd_sb16dsp_proc_init(snd_pcm_t * pcm);
 extern void snd_sb16dsp_proc_done(snd_pcm_t * pcm);
 
 extern void snd_sb8dsp_midi_interrupt(snd_rawmidi_t * rmidi);
-extern snd_rawmidi_t *snd_sb8dsp_midi_new_device(snd_card_t * card, snd_pcm_t * pcm);
+extern int snd_sb8dsp_midi_new(sbdsp_t * codec, int device, snd_rawmidi_t ** rrawmidi);
 
 #endif				/* __SB_H */
