@@ -5,7 +5,7 @@
 #include "cs4231.h"
 #include "mpu401.h"
 #include "hwdep.h"
-#include "midi.h"
+#include "rawmidi.h"
 #include "wavefront.h"  /* generic OSS/ALSA/user-level wavefront header */
 
 /* MIDI interface */
@@ -18,18 +18,18 @@ typedef struct snd_stru_wavefront_midi snd_wavefront_midi_t;
 typedef struct snd_stru_wavefront_card snd_wavefront_card_t;
 typedef struct snd_stru_wavefront snd_wavefront_t;
 
-typedef enum { internal_mpu = 1, external_mpu = 2 } snd_wavefront_mpu_id;
+typedef enum { internal_mpu = 0, external_mpu = 1 } snd_wavefront_mpu_id;
 
 struct snd_stru_wavefront_midi {
-        unsigned short           base;        /* I/O port address */
+        unsigned long            base;        /* I/O port address */
 	char                     isvirtual;   /* doing virtual MIDI stuff ? */
+	char			 istimer;     /* timer is used */
         snd_wavefront_mpu_id     output_mpu;  /* most-recently-used */
         snd_wavefront_mpu_id     input_mpu;   /* most-recently-used */
-        unsigned int             internal_mode;    /* MPU401_MODE_XXX */
-        unsigned int             external_mode;    /* MPU401_MODE_XXX */
+        unsigned int             mode[2];     /* MPU401_MODE_XXX */
+	snd_rawmidi_t		 *rmidi[2];
+	struct timer_list	 timer;
         spinlock_t               open;
-        spinlock_t               input;
-        spinlock_t               output;
         spinlock_t               virtual;     /* protects isvirtual */
 };
 
@@ -38,8 +38,8 @@ struct snd_stru_wavefront_midi {
 #define	MPU_ACK		0xFE
 #define	UART_MODE_ON	0x3F
 
-extern struct snd_stru_rawmidi_direction_hw snd_wavefront_midi_output;
-extern struct snd_stru_rawmidi_direction_hw snd_wavefront_midi_input;
+extern struct snd_stru_rawmidi_channel_hw snd_wavefront_midi_output;
+extern struct snd_stru_rawmidi_channel_hw snd_wavefront_midi_input;
 
 extern void   snd_wavefront_midi_enable_virtual (snd_wavefront_card_t *);
 extern void   snd_wavefront_midi_disable_virtual (snd_wavefront_card_t *);
@@ -47,8 +47,8 @@ extern void   snd_wavefront_midi_interrupt (snd_wavefront_card_t *);
 extern int    snd_wavefront_midi_start (snd_wavefront_card_t *);
 
 struct snd_stru_wavefront {
-	unsigned short   irq;   /* "you were one, one of the few ..." */
-	unsigned short   base;  /* low i/o port address */
+	unsigned long    irq;   /* "you were one, one of the few ..." */
+	unsigned long    base;  /* low i/o port address */
 
 #define mpu_data_port    base 
 #define mpu_command_port base + 1 /* write semantics */
@@ -113,12 +113,12 @@ struct snd_stru_wavefront_card {
         snd_hwdep_t *fx_processor;
         snd_wavefront_t wavefront;
 
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 	struct isapnp_dev *wss;
 	struct isapnp_dev *ctrl;
 	struct isapnp_dev *mpu;
 	struct isapnp_dev *synth;
-#endif CONFIG_ISAPNP
+#endif /* CONFIG_ISAPNP */
 };
 
 extern void snd_wavefront_internal_interrupt (snd_wavefront_card_t *card);
@@ -154,5 +154,5 @@ extern int snd_wavefront_fx_release (snd_hwdep_t *, struct file *);
 
 #define LOGNAME "WaveFront: "
 
-#endif  __SND_WAVEFRONT_H__
+#endif  /* __SND_WAVEFRONT_H__ */
 
