@@ -19,24 +19,9 @@
  *
  */
 
-#include "driver.h"
+#include "../include/driver.h"
 
-/*
- * DMA isn't disabled now when residue is read. Looks that this causes
- * problems with some DMA controllers. Maybe old controllers may give
- * wrong values, so if you have this problem - try to change the line bellow.
- */
-
-#if LinuxVersionCode(2, 2, 0) > LINUX_VERSION_CODE
-#define isa_dma_bridge_buggy (0)
-#define isa_dma_lock(f) snd_cli(&f)
-#define isa_dma_unlock(f) snd_sti(&f)
-#else
-#define isa_dma_lock(f) f = claim_dma_lock()
-#define isa_dma_unlock(f) release_dma_lock(f)
-#endif
-
-#undef SND_DISABLE_DMA_WHEN_RESIDUE
+#ifdef CONFIG_SND_ISA
 
 /*
  * I added some delay here [Jaroslav]. Maybe this can help for some broken
@@ -50,7 +35,7 @@
  */
  
 #ifdef SND_DELAY_BY_DMA
-#define snd_dma_delay() snd_delay(3)
+#define snd_dma_delay() udelay(30)
 #else
 #define snd_dma_delay() do { ; } while(0)
 #endif
@@ -61,8 +46,7 @@ void snd_dma_program(int dma,
 {
 	unsigned long flags;
 
-	/* we don't have a DMA spinlock... */
-	isa_dma_lock(flags);
+	flags = claim_dma_lock();
 	disable_dma(dma);
 	snd_dma_delay();
 	clear_dma_ff(dma);
@@ -73,7 +57,19 @@ void snd_dma_program(int dma,
 	snd_dma_delay();
 	if (!(mode & DMA_MODE_NO_ENABLE))
 		enable_dma(dma);
-	isa_dma_unlock(flags);
+	release_dma_lock(flags);
+}
+
+void snd_dma_disable(int dma)
+{
+	unsigned long flags;
+
+	flags = claim_dma_lock();
+	disable_dma(dma);
+	snd_dma_delay();
+	clear_dma_ff(dma);
+	snd_dma_delay();
+	release_dma_lock(flags);
 }
 
 unsigned int snd_dma_residue(int dma)
@@ -81,8 +77,7 @@ unsigned int snd_dma_residue(int dma)
 	unsigned long flags;
 	unsigned int result;
 
-	/* we don't have a DMA spinlock... */
-	isa_dma_lock(flags);
+	flags = claim_dma_lock();
 	if (!isa_dma_bridge_buggy) {
 		disable_dma(dma);
 		snd_dma_delay();
@@ -92,6 +87,8 @@ unsigned int snd_dma_residue(int dma)
 	result = get_dma_residue(dma);
 	if (!isa_dma_bridge_buggy)
 		enable_dma(dma);
-	isa_dma_unlock(flags);
+	release_dma_lock(flags);
 	return result;
 }
+
+#endif /* CONFIG_SND_ISA */
