@@ -63,7 +63,7 @@ static int snd_mixer_open(unsigned short minor, int cardnum, int device,
 	if (mfile == NULL)
 		return -ENOMEM;
 	init_waitqueue_head(&mfile->change_sleep);
-	mfile->read_lock = SPIN_LOCK_UNLOCKED;
+	spin_lock_init(&mfile->read_lock);
 	mfile->mixer = mixer;
 	memset(&mfile->read_filter, 0xff, sizeof(mfile->read_filter));
 	file->private_data = mfile;
@@ -100,7 +100,7 @@ static int snd_mixer_release(unsigned short minor, int cardnum, int device,
 		spin_lock_irqsave(&mixer->ffile_lock, flags);
 		while (mfile->use) {
 			spin_unlock_irqrestore(&mixer->ffile_lock, flags);
-			current->state = TASK_INTERRUPTIBLE;
+			set_current_state(TASK_INTERRUPTIBLE);
 			schedule_timeout(1);
 			spin_lock_irqsave(&mixer->ffile_lock, flags);
 		}
@@ -281,10 +281,6 @@ int snd_mixer_element_remove(snd_kmixer_t * mixer, snd_kmixer_element_t * elemen
 	snd_kmixer_element_t *prev;
 
 	snd_debug_check(mixer == NULL || element == NULL, -ENXIO);
-	for (prev = mixer->elements; prev; prev = prev->next) {
-		snd_mixer_element_list_remove(&element->routes_next, prev);
-		snd_mixer_element_list_remove(&prev->routes_prev, element);
-	}
 	if (mixer->elements == element) {
 		mixer->elements_count--;
 		mixer->elements = element->next;
@@ -295,8 +291,14 @@ int snd_mixer_element_remove(snd_kmixer_t * mixer, snd_kmixer_element_t * elemen
 			prev->next = element->next;
 		}
 	}
-	snd_mixer_element_free(element);
+	for (prev = mixer->elements; prev; prev = prev->next) {
+		snd_mixer_element_list_remove(&element->routes_next, prev);
+		snd_mixer_element_list_remove(&element->routes_prev, prev);
+		snd_mixer_element_list_remove(&prev->routes_next, element);
+		snd_mixer_element_list_remove(&prev->routes_prev, element);
+	}
 	snd_mixer_element_notify(mixer, element, SND_MIXER_READ_ELEMENT_REMOVE);
+	snd_mixer_element_free(element);
 	return 0;
 }
 
@@ -594,8 +596,8 @@ int snd_mixer_group_remove(snd_kmixer_t * mixer, snd_kmixer_group_t * group)
 			prev->next = group->next;
 		}
 	}
-	snd_mixer_group_free(group);
 	snd_mixer_group_notify(mixer, group, SND_MIXER_READ_GROUP_REMOVE);
+	snd_mixer_group_free(group);
 	return 0;
 }
 
@@ -728,34 +730,6 @@ int snd_mixer_group_element_remove(snd_kmixer_t * mixer,
 	return -ENXIO;
 }
 
-static int snd_mixer_switch_read(snd_kmixer_t * mixer,
-				 snd_switch_t * _uswitch)
-{
-	return snd_switch_read(&mixer->switches, mixer, _uswitch);
-}
-
-static int snd_mixer_switch_write(snd_control_t * control,
-				  snd_kmixer_t * mixer,
-				  snd_switch_t * _uswitch)
-{
-	int result;
-
-	if (snd_control_busy(control))
-		return -EBUSY;
-	result = snd_switch_write(&mixer->switches, mixer, _uswitch);
-	if (result > 0) {	/* change */
-		snd_switch_t uswitch;
-
-		if (copy_from_user(&uswitch, _uswitch, sizeof(uswitch)))
-			return -EFAULT;
-		snd_control_notify_switch_value_change(control,
-						       SND_CTL_IFACE_MIXER,
-						       uswitch.name,
-						       0);
-	}
-	return result;
-}
-
 static int snd_mixer_info(snd_kmixer_t * mixer, snd_mixer_info_t * _info)
 {
 	snd_mixer_info_t info;
@@ -1123,21 +1097,12 @@ static int snd_mixer_control_ioctl(snd_card_t * card, snd_control_t * control, u
 			return 0;
 		}
 	case SND_CTL_IOCTL_MIXER_INFO:
-	case SND_CTL_IOCTL_MIXER_SWITCH_LIST:
-	case SND_CTL_IOCTL_MIXER_SWITCH_READ:
-	case SND_CTL_IOCTL_MIXER_SWITCH_WRITE:
 		mixer = snd_mixers[tmp + control->mixer_device];
 		if (mixer == NULL)
 			return -ENXIO;
 		switch (cmd) {
 		case SND_CTL_IOCTL_MIXER_INFO:
 			return snd_mixer_info(mixer, (snd_mixer_info_t *) arg);
-		case SND_CTL_IOCTL_MIXER_SWITCH_LIST:
-			return snd_switch_list(&mixer->switches, (snd_switch_list_t *) arg);
-		case SND_CTL_IOCTL_MIXER_SWITCH_READ:
-			return snd_mixer_switch_read(mixer, (snd_switch_t *) arg);
-		case SND_CTL_IOCTL_MIXER_SWITCH_WRITE:
-			return snd_mixer_switch_write(control, mixer, (snd_switch_t *) arg);
 		}
 		break;
 	}
@@ -1162,7 +1127,7 @@ static long snd_mixer_read(struct file *file, char *buffer, long count)
 		spin_lock_irqsave(&mfile->read_lock, flags);
 		if (mfile->rebuild) {
 			mfile->rebuild = 0;
-			spin_unlock_irqrestore(&mfile->read_lock, &flags);
+			spin_unlock_irqrestore(&mfile->read_lock, flags);
 			memset(&rebuild, 0, sizeof(rebuild));
 			rebuild.cmd = SND_MIXER_READ_REBUILD;
 			copy_to_user(buffer, &rebuild, sizeof(rebuild));
@@ -1234,17 +1199,12 @@ static void snd_mixer_proc_read(snd_info_buffer_t * buffer, void *private_data)
 
 static snd_minor_t snd_mixer_reg =
 {
-	"mixer",		/* comment */
-	NULL,			/* reserved */
-
-	NULL,			/* lseek */
-	snd_mixer_read,		/* read */
-	NULL,			/* write */
-	snd_mixer_open,		/* open */
-	snd_mixer_release,	/* release */
-	snd_mixer_poll,		/* poll */
-	snd_mixer_ioctl,	/* ioctl */
-	NULL			/* mmap */
+	comment:	"mixer",
+	read:		snd_mixer_read,
+	open:		snd_mixer_open,
+	release:	snd_mixer_release,
+	poll:		snd_mixer_poll,
+	ioctl:		snd_mixer_ioctl,
 };
 
 int snd_mixer_new(snd_card_t * card, char *id, int device, snd_kmixer_t ** rmixer)
@@ -1267,9 +1227,9 @@ int snd_mixer_new(snd_card_t * card, char *id, int device, snd_kmixer_t ** rmixe
 	if (id) {
 		strncpy(mixer->id, id, sizeof(mixer->id) - 1);
 	}
-	mixer->ffile_lock = SPIN_LOCK_UNLOCKED;
+	spin_lock_init(&mixer->ffile_lock);
 	init_MUTEX(&mixer->lock);
-	snd_switch_prepare(&mixer->switches);
+	snd_switch_prepare(card, &mixer->switches, mixer, SND_CTL_IFACE_MIXER, mixer->device, 0);
 	if ((err = snd_device_new(card, SND_DEV_MIXER, mixer, device, &ops, NULL)) < 0) {
 		snd_mixer_free(mixer);
 		return err;
@@ -1280,21 +1240,12 @@ int snd_mixer_new(snd_card_t * card, char *id, int device, snd_kmixer_t ** rmixe
 
 static int snd_mixer_free(snd_kmixer_t * mixer)
 {
-	snd_kmixer_element_t *element;
-	snd_kmixer_group_t *group;
-
 	snd_debug_check(mixer == NULL, -ENXIO);
-	while (mixer->elements) {
-		element = mixer->elements;
-		mixer->elements = element->next;
-		snd_mixer_element_free(element);
-	}
-	while (mixer->groups) {
-		group = mixer->groups;
-		mixer->groups = group->next;
-		snd_mixer_group_free(group);
-	}
-	snd_switch_free(&mixer->switches);
+	while (mixer->groups)
+		snd_mixer_group_remove(mixer, mixer->groups);
+	while (mixer->elements)
+		snd_mixer_element_remove(mixer, mixer->elements);
+	snd_switch_free(mixer->card, &mixer->switches);
 	if (mixer->private_data && mixer->private_free)
 		mixer->private_free(mixer->private_data);
 	snd_magic_kfree(mixer);
@@ -1308,6 +1259,7 @@ int snd_mixer_switch_add(snd_kmixer_t * mixer, snd_kswitch_t * kswitch)
 		snd_control_notify_switch_change(mixer->card,
 						 SND_CTL_READ_SWITCH_ADD,
 						 SND_CTL_IFACE_MIXER,
+						 mixer->device, 0,
 						 kswitch->name);
 	return err;
 }
@@ -1321,6 +1273,7 @@ int snd_mixer_switch_remove(snd_kmixer_t * mixer, snd_kswitch_t * kswitch)
 		snd_control_notify_switch_change(mixer->card,
 						 SND_CTL_READ_SWITCH_REMOVE,
 						 SND_CTL_IFACE_MIXER,
+						 mixer->device, 0,
 						 kswitch->name);
 	return err;
 }
@@ -1346,6 +1299,7 @@ int snd_mixer_switch_change(snd_kmixer_t * mixer, snd_kswitch_t * kswitch)
 	snd_control_notify_switch_change(mixer->card,
 					 SND_CTL_READ_SWITCH_CHANGE,
 					 SND_CTL_IFACE_MIXER,
+					 mixer->device, 0,
 					 kswitch->name);
 	return 0;
 }
@@ -1520,21 +1474,17 @@ EXPORT_SYMBOL(snd_mixer_lib_3d_effect1);
  *  INIT PART
  */
 
-#ifdef MODULE
-int __init init_module(void)
-#else
-int __init alsa_mixer_init(void)
-#endif
+static int __init alsa_mixer_init(void)
 {
 	snd_control_register_ioctl(snd_mixer_control_ioctl);
 	return 0;
 }
 
-#ifdef MODULE
-
-void __exit cleanup_module(void)
+static void __exit alsa_mixer_exit(void)
 {
 	snd_control_unregister_ioctl(snd_mixer_control_ioctl);
 }
 
-#endif
+module_init(alsa_mixer_init)
+module_exit(alsa_mixer_exit)
+MODULE_LICENSE("GPL");
