@@ -3,7 +3,7 @@
  *
  * registration of device and proc
  *
- * Copyright (C) 1998,99 Takashi Iwai <iwai@ww.uni-erlangen.de>
+ * Copyright (C) 1998,99 Takashi Iwai <tiwai@suse.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,10 +59,10 @@ static void info_read(snd_info_buffer_t *buf, void *private);
  * module interface
  */
 
-MODULE_AUTHOR("Takashi Iwai <iwai@ww.uni-erlangen.de>");
+MODULE_AUTHOR("Takashi Iwai <tiwai@suse.de>");
 MODULE_DESCRIPTION("OSS-compatible sequencer module");
 
-int init_module(void)
+static int __init alsa_seq_oss_init(void)
 {
 	int rc;
 	static snd_seq_dev_ops_t ops = {
@@ -82,19 +82,29 @@ int init_module(void)
 		return rc;
 	}
 
-	return snd_seq_device_register_driver(SND_SEQ_DEV_OSS, &ops);
+	if ((rc = snd_seq_device_register_driver(SND_SEQ_DEV_ID_OSS, &ops,
+						 sizeof(snd_seq_oss_reg_t))) < 0) {
+		snd_seq_oss_delete_client();
+		unregister_proc();
+		unregister_device();
+		return rc;
+	}
+
+	/* success */
+	snd_seq_oss_synth_init();
+	return 0;
 }
 
-
-void cleanup_module(void)
+static void __exit alsa_seq_oss_exit(void)
 {
-	snd_seq_device_unregister_driver(SND_SEQ_DEV_OSS);
+	snd_seq_device_unregister_driver(SND_SEQ_DEV_ID_OSS);
 	snd_seq_oss_delete_client();
 	unregister_proc();
 	unregister_device();
 }
 
-
+module_init(alsa_seq_oss_init)
+module_exit(alsa_seq_oss_exit)
 
 /*
  * ALSA minor device interface
@@ -140,8 +150,8 @@ static long
 dev_read(struct file *file, char *buf, long count)
 {
 	seq_oss_devinfo_t *dp;
-	if ((dp = file->private_data) == NULL)
-		return -EIO;
+	dp = file->private_data;
+	snd_debug_check(dp == NULL, -EIO);
 	return snd_seq_oss_read(dp, buf, count);
 }
 
@@ -150,8 +160,8 @@ static long
 dev_write(struct file *file, const char *buf, long count)
 {
 	seq_oss_devinfo_t *dp;
-	if ((dp = file->private_data) == NULL)
-		return -EIO;
+	dp = file->private_data;
+	snd_debug_check(dp == NULL, -EIO);
 	return snd_seq_oss_write(dp, buf, count, file);
 }
 
@@ -159,8 +169,8 @@ static int
 dev_ioctl(struct file *file, unsigned int cmd, unsigned long arg)
 {
 	seq_oss_devinfo_t *dp;
-	if ((dp = file->private_data) == NULL)
-		return -EIO;
+	dp = file->private_data;
+	snd_debug_check(dp == NULL, -EIO);
 	return snd_seq_oss_ioctl(dp, cmd, arg);
 }
 
@@ -169,8 +179,8 @@ static unsigned int
 dev_poll(struct file *file, poll_table * wait)
 {
 	seq_oss_devinfo_t *dp;
-	if ((dp = file->private_data) == NULL)
-		return 0;
+	dp = file->private_data;
+	snd_debug_check(dp == NULL, 0);
 	return snd_seq_oss_poll(dp, file, wait);
 }
 
@@ -179,20 +189,16 @@ dev_poll(struct file *file, poll_table * wait)
  */
 
 static snd_minor_t seq_oss_reg = {
-	"sequencer",
-	NULL,		/* entry */
-	NULL,		/* unregister */
-	NULL,		/* lseek */
-	dev_read,	/* read */
-	dev_write,	/* write */
-	dev_open,	/* open */
-	dev_release,	/* release */
-	dev_poll,	/* poll */
-	dev_ioctl,	/* ioctl */
-	NULL			/* mmap */
+	comment:	"sequencer",
+	read:		dev_read,
+	write:		dev_write,
+	open:		dev_open,
+	release:	dev_release,
+	poll:		dev_poll,
+	ioctl:		dev_ioctl,
 };
 
-static int
+static int __init
 register_device(void)
 {
 	int rc;
@@ -202,7 +208,16 @@ register_device(void)
 					  NULL, 0,
 					  &seq_oss_reg,
 					  SND_SEQ_OSS_DEVNAME)) < 0) {
-		snd_printk("sequencer: can't register device\n");
+		snd_printk("sequencer: can't register device seq\n");
+		up(&register_mutex);
+		return rc;
+	}
+	if ((rc = snd_register_oss_device(SND_OSS_DEVICE_TYPE_MUSIC,
+					  NULL, 0,
+					  &seq_oss_reg,
+					  SND_SEQ_OSS_DEVNAME)) < 0) {
+		snd_printk("sequencer: can't register device music\n");
+		snd_unregister_oss_device(SND_OSS_DEVICE_TYPE_SEQUENCER, NULL, 0);
 		up(&register_mutex);
 		return rc;
 	}
@@ -216,12 +231,14 @@ unregister_device(void)
 {
 	down(&register_mutex);
 	debug_printk(("seq_oss: device unregistered\n"));
+	if (snd_unregister_oss_device(SND_OSS_DEVICE_TYPE_MUSIC, NULL, 0) < 0)
+		
+		snd_printk("sequencer: error unregister device music\n");
 	if (snd_unregister_oss_device(SND_OSS_DEVICE_TYPE_SEQUENCER, NULL, 0) < 0)
-		snd_printk("sequencer: error unregister device\n");
+		snd_printk("sequencer: error unregister device seq\n");
 	up(&register_mutex);
 }
 
-
 /*
  * /proc interface
  */
@@ -243,14 +260,14 @@ info_read(snd_info_buffer_t *buf, void *private)
 
 #endif /* CONFIG_PROC_FS */
 
-static int
+static int __init
 register_proc(void)
 {
 #ifdef CONFIG_PROC_FS
 	snd_info_entry_t *entry;
 
 	entry = snd_info_create_entry(NULL, SND_SEQ_OSS_PROCNAME);
-	if (! entry)
+	if (entry == NULL)
 		return -ENOMEM;
 
 	entry->type = SND_INFO_ENTRY_TEXT;
@@ -267,7 +284,7 @@ register_proc(void)
 	return 0;
 }
 
-static void
+static void __exit
 unregister_proc(void)
 {
 #ifdef CONFIG_PROC_FS
@@ -276,3 +293,6 @@ unregister_proc(void)
 	info_entry = NULL;
 #endif
 }
+
+EXPORT_NO_SYMBOLS;
+MODULE_LICENSE("GPL");
