@@ -3,7 +3,7 @@
  *
  * OSS compatible i/o control
  *
- * Copyright (C) 1998,99 Takashi Iwai <iwai@ww.uni-erlangen.de>
+ * Copyright (C) 1998,99 Takashi Iwai <tiwai@suse.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,9 +48,8 @@ snd_seq_oss_ioctl(seq_oss_devinfo_t *dp, unsigned int cmd, unsigned long carg)
 	case SND_OSS_SNDCTL_TMR_METRONOME:
 	case SND_OSS_SNDCTL_TMR_SOURCE:
 	case SND_OSS_SNDCTL_TMR_SELECT:
-		debug_printk(("seq_oss_ioctl: timer\n"));
-		/* NOP */
-		return 0;
+	case SND_OSS_SNDCTL_SEQ_CTRLRATE:
+		return snd_seq_oss_timer_ioctl(dp->timer, cmd, arg);
 
 	case SND_OSS_SNDCTL_SEQ_PANIC:
 		debug_printk(("seq_oss_ioctl: panic\n"));
@@ -59,12 +58,12 @@ snd_seq_oss_ioctl(seq_oss_devinfo_t *dp, unsigned int cmd, unsigned long carg)
 
 	case SND_OSS_SNDCTL_SEQ_SYNC:
 		debug_printk(("seq_oss_ioctl: sync\n"));
-		if (! is_write_mode(dp->file_mode) || ! dp->writeq)
+		if (! is_write_mode(dp->file_mode) || dp->writeq == NULL)
 			return 0;
 		while (snd_seq_oss_writeq_sync(dp->writeq))
 			;
 		if (signal_pending(current))
-			return -EINTR;
+			return -ERESTARTSYS;
 		return 0;
 
 	case SND_OSS_SNDCTL_SEQ_RESET:
@@ -74,54 +73,51 @@ snd_seq_oss_ioctl(seq_oss_devinfo_t *dp, unsigned int cmd, unsigned long carg)
 
 	case SND_OSS_SNDCTL_SEQ_TESTMIDI:
 		debug_printk(("seq_oss_ioctl: test midi\n"));
-		dev = snd_ioctl_in((long*)arg);
+		if (get_user(dev, (int *)arg))
+			return -EFAULT;
 		return snd_seq_oss_midi_open(dp, dev, dp->file_mode);
 
 	case SND_OSS_SNDCTL_SEQ_GETINCOUNT:
 		debug_printk(("seq_oss_ioctl: get in count\n"));
-		if (!dp->readq || ! is_read_mode(dp->file_mode))
+		if (dp->readq == NULL || ! is_read_mode(dp->file_mode))
 			return 0;
-		return snd_ioctl_out((long*)arg, dp->readq->qlen);
+		return put_user(dp->readq->qlen, (int *)arg) ? -EFAULT : 0;
 
 	case SND_OSS_SNDCTL_SEQ_GETOUTCOUNT:
 		debug_printk(("seq_oss_ioctl: get out count\n"));
-		if (! is_write_mode(dp->file_mode) || ! dp->writeq)
+		if (! is_write_mode(dp->file_mode) || dp->writeq == NULL)
 			return 0;
-		return snd_ioctl_out((long*)arg, snd_seq_oss_writeq_get_free_size(dp->writeq));
+		return put_user(snd_seq_oss_writeq_get_free_size(dp->writeq), (int *)arg) ? -EFAULT : 0;
 
 	case SND_OSS_SNDCTL_SEQ_GETTIME:
 		debug_printk(("seq_oss_ioctl: get time\n"));
-		return snd_ioctl_out((long*)arg, snd_seq_oss_timer_get_tick(dp->timer));
-
-	case SND_OSS_SNDCTL_SEQ_CTRLRATE:
-		debug_printk(("seq_oss_ioctl: ctrl rate\n"));
-		/* if *arg == 0, just return the current rate */
-		if (snd_ioctl_in((long*)arg))
-			return -EINVAL;
-		return snd_ioctl_out((long*)arg, SND_SEQ_OSS_CTRLRATE);
+		return put_user(snd_seq_oss_timer_cur_tick(dp->timer), (int *)arg) ? -EFAULT : 0;
 
 	case SND_OSS_SNDCTL_SEQ_RESETSAMPLES:
 		debug_printk(("seq_oss_ioctl: reset samples\n"));
-		dev = snd_ioctl_in((long*)arg);
+		if (get_user(dev, (int *)arg))
+			return -EFAULT;
 		return snd_seq_oss_synth_ioctl(dp, dev, cmd, carg);
 
 	case SND_OSS_SNDCTL_SEQ_NRSYNTHS:
 		debug_printk(("seq_oss_ioctl: nr synths\n"));
-		return snd_ioctl_out((long*)arg, dp->max_synthdev);
+		return put_user(dp->max_synthdev, (int *)arg) ? -EFAULT : 0;
 
 	case SND_OSS_SNDCTL_SEQ_NRMIDIS:
 		debug_printk(("seq_oss_ioctl: nr midis\n"));
-		return snd_ioctl_out((long*)arg, dp->max_mididev);
+		return put_user(dp->max_mididev, (int *)arg) ? -EFAULT : 0;
 
 	case SND_OSS_SNDCTL_SYNTH_MEMAVL:
 		debug_printk(("seq_oss_ioctl: mem avail\n"));
-		dev = snd_ioctl_in((long*)arg);
+		if (get_user(dev, (int *)arg))
+			return -EFAULT;
 		val = snd_seq_oss_synth_ioctl(dp, dev, cmd, carg);
-		return snd_ioctl_out((long*)arg, val);
+		return put_user(val, (int *)arg) ? -EFAULT : 0;
 
 	case SND_OSS_SNDCTL_FM_4OP_ENABLE:
 		debug_printk(("seq_oss_ioctl: 4op\n"));
-		dev = snd_ioctl_in((long*)arg);
+		if (get_user(dev, (int *)arg))
+			return -EFAULT;
 		snd_seq_oss_synth_ioctl(dp, dev, cmd, carg);
 		return 0;
 
@@ -144,7 +140,7 @@ snd_seq_oss_ioctl(seq_oss_devinfo_t *dp, unsigned int cmd, unsigned long carg)
 		snd_seq_oss_fill_addr(dp, &tmpev, dp->addr.port, dp->addr.client);
 		tmpev.time.tick = 0;
 		if (! snd_seq_oss_process_event(dp, (evrec_t*)ev, &tmpev)) {
-			snd_seq_oss_dispatch(dp, &tmpev, 0);
+			snd_seq_oss_dispatch(dp, &tmpev, 0, 0);
 		}
 		return 0;
 
@@ -162,7 +158,8 @@ snd_seq_oss_ioctl(seq_oss_devinfo_t *dp, unsigned int cmd, unsigned long carg)
 		debug_printk(("seq_oss_ioctl: threshold\n"));
 		if (! is_write_mode(dp->file_mode))
 			return 0;
-		val = snd_ioctl_in((long*)arg);
+		if (get_user(val, (int *)arg))
+			return -EFAULT;
 		if (val < 1)
 			val = 1;
 		if (val >= dp->writeq->maxlen)
@@ -172,15 +169,16 @@ snd_seq_oss_ioctl(seq_oss_devinfo_t *dp, unsigned int cmd, unsigned long carg)
 
 	case SND_OSS_SNDCTL_MIDI_PRETIME:
 		debug_printk(("seq_oss_ioctl: pretime\n"));
-		if (!dp->readq || !is_read_mode(dp->file_mode))
+		if (dp->readq == NULL || !is_read_mode(dp->file_mode))
 			return 0;
-		val = snd_ioctl_in((long*)arg);
+		if (get_user(val, (int *)arg))
+			return -EFAULT;
 		if (val <= 0)
 			val = -1;
 		else
 			val = (HZ * val) / 10;
 		dp->readq->pre_event_timeout = val;
-		return snd_ioctl_out((long*)arg, val);
+		return put_user(val, (int *)arg) ? -EFAULT : 0;
 
 	default:
 		debug_printk(("seq_oss_ioctl: others\n"));
