@@ -19,16 +19,20 @@
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
-#include "driver.h"
+#include "../../include/driver.h"
 
-#include "seq_kernel.h"
+#include "../../include/seq_kernel.h"
 #include "seq_memory.h"
 #include "seq_queue.h"
 #include "seq_info.h"
+#include "seq_lock.h"
 
 /* define to keep statistics on successful allocations (for debugging) */
 #define TRACK_SUCCESS
 
+/* semaphore in struct file record */
+#define semaphore_of(fp)	((fp)->f_dentry->d_inode->i_sem)
+
 /* counter for tracking memory leaks for 'external' data */
 static int ext_alloc = 0;
 static int ext_alloc_max = 0;
@@ -44,6 +48,11 @@ static int ext_alloc_failures = 0;
 static int ext_alloc_success = 0;
 #endif
 
+inline static int snd_seq_output_ok(pool_t *pool)
+{
+	return pool->available >= pool->room;
+}
+
 
 /* release this cell, free extended data if available */
 void snd_seq_cell_free(snd_seq_event_cell_t * cell)
@@ -60,7 +69,7 @@ void snd_seq_cell_free(snd_seq_event_cell_t * cell)
 		snd_printd("oops: snd_seq_cell called with pool == NULL?\n");
 		return;
 	}
-	snd_spin_lock(pool, lock, &flags);
+	spin_lock_irqsave(&pool->lock, flags);
 	if (pool->free != NULL) {
 		/* normal situation */
 		cell->ptr_l = pool->free;	/* chain in old element */
@@ -81,9 +90,18 @@ void snd_seq_cell_free(snd_seq_event_cell_t * cell)
 		len = 0;
 	}
 	cell->event.flags = 0;	/* for sure */
-	snd_spin_unlock(pool, lock, &flags);
-	if (ptr && len > 0)
+	spin_unlock_irqrestore(&pool->lock, flags);
+	if (ptr && len >= 0)
 		snd_seq_ext_free(ptr, len);
+
+	/* if the client is sleeping, wake it up */
+	spin_lock_irqsave(&pool->output_sleep_lock, flags);
+	if (waitqueue_active(&pool->output_sleep)) {
+		/* has enough space now? */
+		if (snd_seq_output_ok(pool))
+			wake_up(&pool->output_sleep);
+	}
+	spin_unlock_irqrestore(&pool->output_sleep_lock, flags);
 }
 
 
@@ -95,10 +113,10 @@ snd_seq_event_cell_t *snd_seq_cell_alloc(pool_t *pool)
 
 	if (pool == NULL)
 		return NULL;
-	snd_spin_lock(pool, lock, &flags);
+	spin_lock_irqsave(&pool->lock, flags);
 	if (pool->ptr == NULL) {	/* not initialized */
 		pool->event_alloc_nopool++;
-		snd_spin_unlock(pool, lock, &flags);
+		spin_unlock_irqrestore(&pool->lock, flags);
 		return NULL;
 	}
 	if (pool->free != NULL) {
@@ -113,72 +131,133 @@ snd_seq_event_cell_t *snd_seq_cell_alloc(pool_t *pool)
 		cell->ptr_r = NULL;
 	} else {
 		/* no element available... */
-		snd_printd("Seq: cell_alloc failed: no cells available\n");
+		/*snd_printd("Seq: cell_alloc failed: no cells available\n");*/
 		cell = NULL;
 	}
 	if (cell == NULL)
 		pool->event_alloc_failures++;
 	else
 		pool->event_alloc_success++;
-	snd_spin_unlock(pool, lock, &flags);
+	spin_unlock_irqrestore(&pool->lock, flags);
 
 	return cell;
 }
 
 
-/* duplicate event, NULL on failure */
-extern snd_seq_event_cell_t *snd_seq_event_dup(pool_t *pool, snd_seq_event_t * event, int atomic)
+/* convert from LENGTH_VARUSR type to LENGTH_VARIABLE type */
+int snd_seq_event_convert_varlen(snd_seq_event_t *event, int atomic)
 {
-	snd_seq_event_cell_t *new_cell;
 	char *ptr;
 
-	if (event == NULL ||
-	    (event->flags & SND_SEQ_EVENT_LENGTH_MASK) == SND_SEQ_EVENT_LENGTH_VARUSR ||
-	    (event->flags & SND_SEQ_EVENT_LENGTH_MASK) == SND_SEQ_EVENT_LENGTH_VARIPC)
-		return NULL;
+	if ((event->flags & SND_SEQ_EVENT_LENGTH_MASK) != SND_SEQ_EVENT_LENGTH_VARUSR)
+		return 0;
+
+	if (event->data.ext.len < 0 || event->data.ext.len >= SND_SEQ_MAX_EVENT_LEN)
+		return -EINVAL;
+
+	/* allocate kernel buffer */
+	ptr = snd_seq_ext_malloc(event->data.ext.len, atomic);
+	if (ptr == NULL)
+		return -ENOMEM;
+
+	/* copy data to kernel space */
+	if (copy_from_user(ptr, event->data.ext.ptr, event->data.ext.len)) {
+		snd_seq_ext_free(event->data.ext.ptr, event->data.ext.len);
+		return -EFAULT;
+	}
+
+	event->flags &= ~SND_SEQ_EVENT_LENGTH_MASK;
+	event->flags |= SND_SEQ_EVENT_LENGTH_VARIABLE;
+	event->data.ext.ptr = ptr;
+
+	return 0;
+}
+
+/* duplicate event, NULL on failure */
+int snd_seq_event_dup(pool_t *pool, snd_seq_event_t *event, snd_seq_event_cell_t **cellp, int atomic)
+{
+	int err;
+	snd_seq_event_cell_t *new_cell;
+
+	*cellp = NULL;
+
+	snd_debug_check(event == NULL, -EINVAL);
 
 	new_cell = snd_seq_cell_alloc(pool);
-	if (new_cell) {
-		memcpy(&new_cell->event, event, sizeof(snd_seq_event_t));
-		if ((event->flags & SND_SEQ_EVENT_LENGTH_MASK) == SND_SEQ_EVENT_LENGTH_VARIABLE) {
-			ptr = snd_seq_ext_malloc(event->data.ext.len, atomic);
-			if (!ptr) {
-				snd_seq_cell_free(new_cell);
-				return NULL;
-			}
-			memcpy(ptr, event->data.ext.ptr, event->data.ext.len);
-			new_cell->event.data.ext.ptr = ptr;
+	if (new_cell == NULL)
+		return -EAGAIN;
+
+	memcpy(&new_cell->event, event, sizeof(snd_seq_event_t));
+	switch (event->flags & SND_SEQ_EVENT_LENGTH_MASK) {
+	case SND_SEQ_EVENT_LENGTH_VARIABLE:
+		/* increment reference counter */
+		snd_seq_ext_ref(event->data.ext.ptr);
+ 		break;
+	case SND_SEQ_EVENT_LENGTH_VARUSR:
+		/* convert type to LENGTH_VARIABLE */
+		err = snd_seq_event_convert_varlen(&new_cell->event, atomic);
+		if (err) {
+			snd_seq_cell_free(new_cell);
+			return err;
 		}
+		break;
 	}
-	return new_cell;
+	*cellp = new_cell;
+	return 0;
 }
 
 
-/* duplicate event cell, NULL on failure */
-extern snd_seq_event_cell_t *snd_seq_cell_dup(pool_t *pool, snd_seq_event_cell_t * cell, int atomic)
+/* duplicate a cell with blocking */
+int snd_seq_event_dup_blocking(pool_t *pool, snd_seq_event_t *event, snd_seq_event_cell_t **cellp, struct file *file)
 {
-	snd_seq_event_cell_t *new_cell;
-	char *ptr;
+	snd_seq_event_cell_t *cell;
+	unsigned long flags;
+	int err = 0;
 
-	if (cell == NULL ||
-	    (cell->event.flags & SND_SEQ_EVENT_LENGTH_MASK) == SND_SEQ_EVENT_LENGTH_VARUSR ||
-	    (cell->event.flags & SND_SEQ_EVENT_LENGTH_MASK) == SND_SEQ_EVENT_LENGTH_VARIPC)
-		return NULL;
-
-	new_cell = snd_seq_cell_alloc(pool);
-	if (new_cell) {
-		memcpy(&new_cell->event, &cell->event, sizeof(snd_seq_event_t));
-		if ((cell->event.flags & SND_SEQ_EVENT_LENGTH_MASK) == SND_SEQ_EVENT_LENGTH_VARIABLE) {
-			ptr = snd_seq_ext_malloc(cell->event.data.ext.len, atomic);
-			if (!ptr) {
-				snd_seq_cell_free(new_cell);
-				return NULL;
-			}
-			memcpy(ptr, new_cell->event.data.ext.ptr, cell->event.data.ext.len);
-			new_cell->event.data.ext.ptr = ptr;
-		}
+	*cellp = cell = NULL;
+	spin_lock_irqsave(&pool->output_sleep_lock, flags);
+	if (pool->available > 0) {
+		err = snd_seq_event_dup(pool, event, &cell, 0); /* non-atomic */
+		if (err != -EAGAIN)
+			goto __error_exit;
 	}
-	return new_cell;
+
+	/* sleep until pool has enough room */
+	while (cell == NULL) {
+		/* change semaphore to allow other clients
+		   to access device file */
+		if (file)
+			up(&semaphore_of(file));
+
+		snd_seq_sleep_in_lock(&pool->output_sleep, &pool->output_sleep_lock);
+
+		/* restore semaphore again */
+		if (file)
+			down(&semaphore_of(file));
+
+		/* interrupted? */
+		if (signal_pending(current)) {
+			err = -ERESTARTSYS;
+			break;
+		}
+
+		/* try again.. */
+		err = snd_seq_event_dup(pool, event, &cell, 0);
+		if (err != -EAGAIN)
+			break;
+	}
+__error_exit:
+	spin_unlock_irqrestore(&pool->output_sleep_lock, flags);
+	*cellp = cell;
+	return err;
+}
+
+
+/* poll wait */
+int snd_seq_pool_poll_wait(pool_t *pool, struct file *file, poll_table *wait)
+{
+	poll_wait(file, &pool->output_sleep, wait);
+	return snd_seq_output_ok(pool);
 }
 
 
@@ -199,19 +278,22 @@ int snd_seq_total_cells(pool_t *pool)
 	return (pool->total_elements);
 }
 
+
 /* allocate room specified number of events */
-int snd_seq_pool_init(pool_t *pool, int events)
+int snd_seq_pool_init(pool_t *pool)
 {
 	int cell;
 	snd_seq_event_cell_t *ptr, *cellptr;
 	unsigned long flags;
+	int events;
 
-	if (pool == NULL)
-		return -EINVAL;
+	snd_debug_check(pool == NULL, -EINVAL);
 	if (pool->ptr)			/* should be atomic? */
 		return 0;
+
+	events = pool->size;
 	/* alloc memory */
-	ptr = snd_calloc(sizeof(snd_seq_event_cell_t) * events);
+	ptr = snd_kcalloc(sizeof(snd_seq_event_cell_t) * events, GFP_KERNEL);
 	if (ptr) {
 
 		/* add the new cell's to the free cell list by calling the free()
@@ -222,12 +304,12 @@ int snd_seq_pool_init(pool_t *pool, int events)
 			snd_seq_cell_free(cellptr);
 		}
 		/* init statistics */
-		snd_spin_lock(pool, lock, &flags);
+		spin_lock_irqsave(&pool->lock, flags);
 		if (!pool->min_available)
 			pool->min_available = pool->available;
 		pool->total_elements = events;
 		pool->ptr = ptr;
-		snd_spin_unlock(pool, lock, &flags);
+		spin_unlock_irqrestore(&pool->lock, flags);
 	} else {
 		snd_printd("Seq: malloc for sequencer events failed\n");
 		return -ENOMEM;
@@ -242,11 +324,10 @@ int snd_seq_pool_done(pool_t *pool)
 	snd_seq_event_cell_t *ptr;	
 	unsigned int total_elements;
 
-	if (pool == NULL)
-		return -EINVAL;
-	snd_spin_lock(pool, lock, &flags);
+	snd_debug_check(pool == NULL, -EINVAL);
+	spin_lock_irqsave(&pool->lock, flags);
 	if (pool->available != pool->total_elements) {
-		snd_spin_unlock(pool, lock, &flags);
+		spin_unlock_irqrestore(&pool->lock, flags);
 		snd_printd("seq: snd_seq_pool_done() - pool isn't free!!\n");
 		return -EBUSY;
 	}
@@ -256,29 +337,35 @@ int snd_seq_pool_done(pool_t *pool)
 	pool->free = NULL;
 	pool->total_elements = 0;
 	pool->available = 0;
-	snd_spin_unlock(pool, lock, &flags);
-	if (!ptr)
+	spin_unlock_irqrestore(&pool->lock, flags);
+	if (ptr == NULL)
 		return 0;
-	snd_free(ptr, sizeof(snd_seq_event_cell_t) * total_elements);
+	snd_kfree(ptr);
 	return 0;
 }
 
 /* init new memory pool */
-pool_t *snd_seq_pool_new(void)
+pool_t *snd_seq_pool_new(int poolsize)
 {
 	pool_t *pool;
 
 	/* create pool block */
-	pool = snd_calloc(sizeof(pool_t));
-	if (!pool) {
+	pool = snd_kcalloc(sizeof(pool_t), GFP_KERNEL);
+	if (pool == NULL) {
 		snd_printd("seq: malloc failed for pool\n");
 		return NULL;
 	}
+	spin_lock_init(&pool->lock);
 	pool->ptr = NULL;
 	pool->free = NULL;
 	pool->total_elements = 0;
 	pool->available = 0;
+	init_waitqueue_head(&pool->output_sleep);
+	spin_lock_init(&pool->output_sleep_lock);
 	
+	pool->size = poolsize;
+	pool->room = (poolsize + 1) / 2;
+
 	/* init statistics */
 	pool->min_available = pool->available;
 	return pool;
@@ -296,17 +383,17 @@ int snd_seq_pool_delete(pool_t **ppool)
 		pool->available = pool->total_elements;
 		snd_seq_pool_done(pool);
 	}
-	snd_free(pool, sizeof(pool_t));
+	snd_kfree(pool);
 	return 0;
 }
 
 /* initialize sequencer memory */
-void snd_sequencer_memory_init(void)
+void __init snd_sequencer_memory_init(void)
 {
 }
 
 /* release sequencer memory */
-void snd_sequencer_memory_done(void)
+void __exit snd_sequencer_memory_done(void)
 {
 	if (ext_alloc > 0) {
 		snd_printd("seq: memory leak alert, still %d blocks of external data allocated\n", ext_alloc);
@@ -318,13 +405,15 @@ void snd_sequencer_memory_done(void)
    events etc.) for now it is just passed to the kmalloc and kfree
    calls, but in a later stadium a different allocation method could be
    used. */
-
+/* ADDITION: reference counter is allocated at the head of each block
+   followed by available space -- Iwai */
 void *snd_seq_ext_malloc(unsigned long size, int atomic)
 {
-	void *obj;
+	int *refp;
 
-	obj = kmalloc(size, atomic ? GFP_ATOMIC : GFP_KERNEL);
-	if (obj) {
+	refp = snd_kmalloc(size + sizeof(int), atomic ? GFP_ATOMIC : GFP_KERNEL);
+	if (refp) {
+		*refp = 1; /* initialize reference counter */
 		ext_alloc++;
 		ext_alloc_bytes += size;
 		if (ext_alloc > ext_alloc_max)
@@ -336,29 +425,43 @@ void *snd_seq_ext_malloc(unsigned long size, int atomic)
 #ifdef TRACK_SUCCESS
 		ext_alloc_success++;
 #endif
+		return (void*)(refp + 1);
 	} else {
 		ext_alloc_failures++;
+		return NULL;
 	}
+}
 
-	return obj;
+void snd_seq_ext_ref(void *obj)
+ {
+	if (obj) {
+		int *refp = (int*)obj - 1;
+		(*refp)++;
+	}
 }
 
 void snd_seq_ext_free(void *obj, unsigned long size)
 {
-	ext_alloc--;
-	ext_alloc_bytes -= size;
+	if (obj) {
+		int *refp = (int*)obj - 1;
+		(*refp)--;
+		if (*refp <= 0) {
+			ext_alloc--;
+			ext_alloc_bytes -= size;
 
-	if (ext_alloc < 0) {
-		snd_printk("seq: whoops, more ext data free()s than malloc()...\n");
+			if (ext_alloc < 0) {
+				snd_printk("seq: whoops, more ext data free()s than malloc()...\n");
+			}
+			snd_kfree(refp);
+		}
 	}
-	kfree(obj);
 }
 
 
 /* exported to seq_clientmgr.c */
 void snd_seq_info_pool(snd_info_buffer_t * buffer, pool_t *pool, char *space)
 {
-	if (!pool)
+	if (pool == NULL)
 		return;
 	snd_iprintf(buffer, "%sPool size          : %d\n", space, snd_seq_total_cells(pool));
 	snd_iprintf(buffer, "%sAvailable cells    : %d\n", space, snd_seq_unused_cells(pool));
@@ -372,17 +475,19 @@ void snd_seq_info_pool(snd_info_buffer_t * buffer, pool_t *pool, char *space)
 /* exported to seq_info.c */
 void snd_seq_info_memory_read(snd_info_buffer_t * buffer, void *private_data)
 {
+	/*
 	int c;
 	queue_t *q;
 
 	for (c = 0; c < SND_SEQ_MAX_QUEUES; c++) {
 		q = queueptr(c);
-		if (!q)
+		if (q == NULL)
 			continue;
 		snd_iprintf(buffer, "Event pool %i\n", c);
 		snd_seq_info_pool(buffer, q->pool, "  ");
 		snd_iprintf(buffer, "\n");
 	}
+	*/
 	snd_iprintf(buffer, "External data\n");
 	snd_iprintf(buffer, "  Blocks in use      : %d\n", ext_alloc);
 	snd_iprintf(buffer, "  Bytes in use       : %d\n", ext_alloc_bytes);
