@@ -21,7 +21,7 @@
 #ifndef __SND_SEQ_MEMORYMGR_H
 #define __SND_SEQ_MEMORYMGR_H
 
-#include "seq_kernel.h"
+#include "../../include/seq_kernel.h"
 
 typedef struct pool pool_t;
 
@@ -42,8 +42,11 @@ struct pool {
 	snd_seq_event_cell_t *ptr;	/* pointer to start of pool */
 	snd_seq_event_cell_t *free;	/* pointer to the head of the free list */
 
-	int total_elements;
-	int available;
+	int total_elements;	/* pool size actually allocated */
+	int available;		/* cells free */
+
+	int size;		/* pool size to be allocated */
+	int room;		/* watermark for sleep/wakeup */
 
 	/* statistics */
 	int min_available;
@@ -51,7 +54,12 @@ struct pool {
 	int event_alloc_failures;
 	int event_alloc_success;
 
-	snd_spin_define(lock);
+	/* Write locking */
+	wait_queue_head_t output_sleep;
+	spinlock_t output_sleep_lock;
+
+	/* Pool lock */
+	spinlock_t lock;
 };
 
 /* release this cell */
@@ -61,11 +69,11 @@ extern void snd_seq_cell_free(snd_seq_event_cell_t* cell);
 extern snd_seq_event_cell_t *snd_seq_cell_alloc(pool_t *pool);
 
 /* duplicate event, NULL on failure */
-extern snd_seq_event_cell_t *snd_seq_event_dup(pool_t *pool, snd_seq_event_t *event, int atomic);
-extern snd_seq_event_cell_t *snd_seq_event_dup_from_user(pool_t *pool, snd_seq_event_t *event);
+extern int snd_seq_event_dup(pool_t *pool, snd_seq_event_t *event, snd_seq_event_cell_t **result, int atomic);
+extern int snd_seq_event_dup_blocking(pool_t *pool, snd_seq_event_t *event, snd_seq_event_cell_t **result, struct file *file);
 
-/* duplicate event cell, NULL on failure */
-extern snd_seq_event_cell_t *snd_seq_cell_dup(pool_t *pool, snd_seq_event_cell_t *event, int atomic);
+/* convert from LENGTH_VARUSR type to LENGTH_VARIABLE type */
+extern int snd_seq_event_convert_varlen(snd_seq_event_t *event, int atomic);
 
 /* return number of unused (free) cells */
 extern int snd_seq_unused_cells(pool_t *pool);
@@ -74,13 +82,13 @@ extern int snd_seq_unused_cells(pool_t *pool);
 extern int snd_seq_total_cells(pool_t *pool);
 
 /* init pool - allocate events */
-extern int snd_seq_pool_init(pool_t *pool, int events);
+extern int snd_seq_pool_init(pool_t *pool);
 
 /* done pool - free events */
 extern int snd_seq_pool_done(pool_t *pool);
 
 /* create pool */
-extern pool_t *snd_seq_pool_new(void);
+extern pool_t *snd_seq_pool_new(int poolsize);
 
 /* remove pool */
 extern int snd_seq_pool_delete(pool_t **pool);
@@ -91,4 +99,8 @@ extern void snd_sequencer_memory_init(void);
 /* release event memory */
 extern void snd_sequencer_memory_done(void);
 
+/* polling */
+extern int snd_seq_pool_poll_wait(pool_t *pool, struct file *file, poll_table *wait);
+
+
 #endif
