@@ -48,6 +48,8 @@ static void nrpn(snd_midi_op_t *ops, void *drv, snd_midi_channel_t *chan, snd_mi
 static void sysex(snd_midi_op_t *ops, void *private, unsigned char *sysex, int len, snd_midi_channel_set_t *chset);
 static void all_sounds_off(snd_midi_op_t *ops, void *private, snd_midi_channel_t *chan);
 static void all_notes_off(snd_midi_op_t *ops, void *private, snd_midi_channel_t *chan);
+void snd_midi_reset_controllers(snd_midi_channel_t *chan);
+static void reset_all_channels(snd_midi_channel_set_t *chset);
 
 
 /*
@@ -70,6 +72,7 @@ snd_midi_process_event(snd_midi_op_t *ops,
 {
 	snd_midi_channel_t *chan;
 	void *drv;
+	int dest_channel = 0;
 
 	if (ev == NULL || chanset == NULL) {
 		snd_printd("ev or chanbase NULL (snd_midi_process_event)\n");
@@ -77,13 +80,16 @@ snd_midi_process_event(snd_midi_op_t *ops,
 	}
 	if (chanset->channels == NULL)
 		return;
-	if (ev->dest.channel >= chanset->max_channels) {
-		snd_printd("dest channel is %d, max is %d\n", ev->dest.channel,
-			chanset->max_channels);
-		return;
+
+	if (snd_seq_ev_is_channel_type(ev)) {
+		dest_channel = ev->data.note.channel;
+		if (dest_channel >= chanset->max_channels) {
+			snd_printd("dest channel is %d, max is %d\n", dest_channel, chanset->max_channels);
+			return;
+		}
 	}
 
-	chan = chanset->channels + ev->dest.channel;
+	chan = chanset->channels + dest_channel;
 	drv  = chanset->private_data;
 
 	/* EVENT_NOTE should be processed before queued */
@@ -105,11 +111,17 @@ snd_midi_process_event(snd_midi_op_t *ops,
 
 	switch (ev->type) {
 	case SND_SEQ_EVENT_NOTEON:
+		if (chan->note[ev->data.note.note] & SND_MIDI_NOTE_ON) {
+			if (ops->note_off)
+				ops->note_off(drv, ev->data.note.note, 0, chan);
+		}
 		chan->note[ev->data.note.note] = SND_MIDI_NOTE_ON;
 		if (ops->note_on)
 			ops->note_on(drv, ev->data.note.note, ev->data.note.velocity, chan);
 		break;
 	case SND_SEQ_EVENT_NOTEOFF:
+		if (! (chan->note[ev->data.note.note] & SND_MIDI_NOTE_ON))
+			break;
 		if (ops->note_off)
 			note_off(ops, drv, chan, ev->data.note.note, ev->data.note.velocity);
 		break;
@@ -191,7 +203,7 @@ snd_midi_process_event(snd_midi_op_t *ops,
 	case SND_SEQ_EVENT_TIMESIGN:
 	case SND_SEQ_EVENT_KEYSIGN:
 		goto not_yet;
-	case SND_SEQ_EVENT_HEARTBEAT:
+	case SND_SEQ_EVENT_SENSING:
 		break;
 	case SND_SEQ_EVENT_CLIENT_START:
 	case SND_SEQ_EVENT_CLIENT_EXIT:
@@ -209,7 +221,7 @@ snd_midi_process_event(snd_midi_op_t *ops,
 	case SND_SEQ_EVENT_ECHO:
 	not_yet:
 	default:
-		snd_printd("Unimplemented event %d\n", ev->type);
+		/*snd_printd("Unimplemented event %d\n", ev->type);*/
 		break;
 	}
 }
@@ -325,6 +337,10 @@ do_control(snd_midi_op_t *ops, void *drv, snd_midi_channel_set_t *chset,
 	case SND_MCTL_LSB_BANK:
 		break;
 
+	case SND_MCTL_RESET_CONTROLLERS:
+		snd_midi_reset_controllers(chan);
+		break;
+
 	case SND_MCTL_SOFT_PEDAL:
 	case SND_MCTL_LEGATO_FOOTSWITCH:
 	case SND_MCTL_HOLD2:
@@ -362,17 +378,16 @@ snd_midi_channel_set_clear(snd_midi_channel_set_t *chset)
 	for (i = 0; i < chset->max_channels; i++) {
 		snd_midi_channel_t *chan = chset->channels + i;
 		memset(chan->note, 0, sizeof(chan->note));
-		memset(chan->control, 0, sizeof(chan->control));
+
 		chan->midi_aftertouch = 0;
 		chan->midi_pressure = 0;
 		chan->midi_program = 0;
 		chan->midi_pitchbend = 0;
-		chan->gm_volume = 127;
-		chan->gm_expression = 127;
-		chan->gm_pan = 64;
+		snd_midi_reset_controllers(chan);
 		chan->gm_rpn_pitch_bend_range = 256; /* 2 semitones */
 		chan->gm_rpn_fine_tuning = 0;
 		chan->gm_rpn_coarse_tuning = 0;
+
 		if (i == 9)
 			chan->drum_channel = 1;
 		else
@@ -380,7 +395,6 @@ snd_midi_channel_set_clear(snd_midi_channel_set_t *chset)
 	}
 }
 
-
 /*
  * Process a rpn message.
  */
@@ -487,7 +501,7 @@ sysex(snd_midi_op_t *ops, void *private, unsigned char *buf, int len, snd_midi_c
 		if (chset->midi_mode != SND_MIDI_MODE_GS &&
 		    chset->midi_mode != SND_MIDI_MODE_XG) {
 			chset->midi_mode = SND_MIDI_MODE_GM;
-			/*init_midi_status(chset);*/
+			reset_all_channels(chset);
 			parsed = SND_MIDI_SYSEX_GM_ON;
 		}
 	}
@@ -502,7 +516,7 @@ sysex(snd_midi_op_t *ops, void *private, unsigned char *buf, int len, snd_midi_c
 		if (buf[5] == 0x00 && buf[6] == 0x7f && buf[7] == 0x00) {
 			/* GS reset */
 			parsed = SND_MIDI_SYSEX_GS_RESET;
-			/*init_midi_status(chset);*/
+			reset_all_channels(chset);
 		}
 
 		else if ((buf[5] & 0xf0) == 0x10 && buf[6] == 0x15) {
@@ -601,17 +615,17 @@ all_notes_off(snd_midi_op_t *ops, void *drv, snd_midi_channel_t *chan)
  */
 void snd_midi_channel_init(snd_midi_channel_t *p, int n)
 {
-	if (!p)
+	if (p == NULL)
 		return;
 
 	memset(p, 0, sizeof(snd_midi_channel_t));
 	p->private = NULL;
 	p->number = n;
 
-	p->gm_volume = 127;
-	p->gm_expression = 127;
-	p->gm_pan = 64;
-	p->gm_rpn_pitch_bend_range = 256; /* 2 octaves */
+	snd_midi_reset_controllers(p);
+	p->gm_rpn_pitch_bend_range = 256; /* 2 semitones */
+	p->gm_rpn_fine_tuning = 0;
+	p->gm_rpn_coarse_tuning = 0;
 
 	if (n == 9)
 		p->drum_channel = 1;	/* Default ch 10 as drums */
@@ -634,6 +648,28 @@ snd_midi_channel_t *snd_midi_channel_init_set(int n)
 	return chan;
 }
 
+/*
+ * reset all midi channels
+ */
+static void
+reset_all_channels(snd_midi_channel_set_t *chset)
+{
+	int ch;
+	for (ch = 0; ch < chset->max_channels; ch++) {
+		snd_midi_channel_t *chan = chset->channels + ch;
+		snd_midi_reset_controllers(chan);
+		chan->gm_rpn_pitch_bend_range = 256; /* 2 semitones */
+		chan->gm_rpn_fine_tuning = 0;
+		chan->gm_rpn_coarse_tuning = 0;
+
+		if (ch == 9)
+			chan->drum_channel = 1;
+		else
+			chan->drum_channel = 0;
+	}
+}
+
+
 /*
  * Allocate and initialise a midi channel set.
  */
@@ -642,13 +678,27 @@ snd_midi_channel_set_t *snd_midi_channel_alloc_set(int n)
 	snd_midi_channel_set_t *chset;
 
 	chset = snd_kmalloc(sizeof(*chset), GFP_KERNEL);
-	chset->channels = snd_midi_channel_init_set(n);
+	if (! chset)
+		return NULL;
 
+	chset->channels = snd_midi_channel_init_set(n);
 	chset->private_data = NULL;
 	chset->max_channels = n;
 	return chset;
 }
 
+/*
+ * Reset the midi controllers on a particular channel to default values.
+ */
+void snd_midi_reset_controllers(snd_midi_channel_t *chan)
+{
+	memset(chan->control, 0, sizeof(chan->control));
+	chan->gm_volume = 127;
+	chan->gm_expression = 127;
+	chan->gm_pan = 64;
+}
+
+
 /*
  * Free a midi channel set.
  */
@@ -661,16 +711,17 @@ void snd_midi_channel_free_set(snd_midi_channel_set_t *chset)
 	snd_kfree(chset);
 }
 
-int init_module(void)
+static int __init alsa_seq_midi_emul_init(void)
 {
 	return 0;
 }
 
-
-void cleanup_module(void)
+static void __exit alsa_seq_midi_emul_exit(void)
 {
 }
 
+module_init(alsa_seq_midi_emul_init)
+module_exit(alsa_seq_midi_emul_exit)
 
 EXPORT_SYMBOL(snd_midi_process_event);
 EXPORT_SYMBOL(snd_midi_channel_set_clear);
@@ -678,3 +729,4 @@ EXPORT_SYMBOL(snd_midi_channel_init);
 EXPORT_SYMBOL(snd_midi_channel_init_set);
 EXPORT_SYMBOL(snd_midi_channel_alloc_set);
 EXPORT_SYMBOL(snd_midi_channel_free_set);
+MODULE_LICENSE("GPL");
