@@ -22,6 +22,7 @@
 #define __SND_SEQ_PORTS_H
 
 #include "../../include/seq_kernel.h"
+#include "seq_lock.h"
 
 /* list of 'exported' ports */
 
@@ -42,16 +43,16 @@
 typedef struct subscribers_t {
 	struct subscribers_t *next;	/* ptr to next subscription entry */
 	snd_seq_addr_t addr;		/* subscription data */
+	unsigned char queue;		/* input time-stamp queue (optional) */
 	int realtime: 1;		/* realtime timestamp */
+	int convert_time: 1;		/* convert timestamp */
 } subscribers_t;
 
 typedef struct subscribers_group_t {
 	subscribers_t *list;		/* list of subscribed ports */
 	unsigned int count;		/* count of subscribers */
 	int exclusive: 1;		/* exclusive mode */
-	unsigned int use;		/* subscribers use counter */
-	spinlock_t use_lock;		/* subscribers lock */
-	wait_queue_head_t use_sleep;	/* subscribers sleeper */
+	snd_use_lock_t use_lock;
 } subscribers_group_t;
 
 typedef struct client_port_t {
@@ -59,12 +60,12 @@ typedef struct client_port_t {
 
 	int  port;			/* port number */
 	char name[64];			/* port name */	
+	char group[32];			/* group name */
 
 	/* subscribers */
-	subscribers_group_t input;	/* input group */
-	subscribers_group_t itrack;	/* track of input group (for close) */
-	subscribers_group_t output;	/* output group */
-	subscribers_group_t otrack;	/* track of output group (for exclusive) */
+	subscribers_group_t export;	/* read (export) group */
+	subscribers_group_t import;	/* write (import) group */
+
 	snd_seq_kernel_port_subscribe_t *subscribe;
 	snd_seq_kernel_port_unsubscribe_t *unsubscribe;
 	unsigned int subscribe_count;
@@ -79,12 +80,13 @@ typedef struct client_port_t {
 	
 	/* capability, inport, output, sync */
 	unsigned int capability;	/* port capability bits */
+	unsigned int cap_group;		/* port permission */
 	unsigned int type;		/* port type bits */
 
 	/* supported channels */
 	int midi_channels;
 	int synth_voices;
-	
+		
 } client_port_t;
 
 /* lock ports - to prevent create/delete operations */
@@ -96,8 +98,11 @@ extern void snd_seq_ports_unlock(client_t *client);
 /* return pointer to port structure and lock ports */
 extern client_port_t *snd_seq_port_use_ptr(client_t *client, int num);
 
+/* search for next port - ports are locked if found */
+extern client_port_t *snd_seq_port_query_nearest(client_t *client, snd_seq_port_info_t *pinfo);
+
 /* create a port, port number is returned (-1 on failure) */
-extern client_port_t *snd_seq_create_port(client_t *client);
+extern client_port_t *snd_seq_create_port(client_t *client, int port_index);
 
 /* delete a port */
 extern int snd_seq_delete_port(client_t *client, int port);
@@ -118,10 +123,10 @@ extern void snd_seq_subscribers_lock(subscribers_group_t *port);
 extern void snd_seq_subscribers_unlock(subscribers_group_t *port);
 
 /* add subscriber to subscription list */
-extern int snd_seq_port_add_subscriber(subscribers_group_t *group, snd_seq_addr_t *dest, int exclusive, int realtime);
+extern int snd_seq_port_add_subscriber(subscribers_group_t *group, snd_seq_addr_t *dest, int queue, int exclusive, int realtime, int tracked);
 
 /* remove subscriber from subscription list */ 
-extern int snd_seq_port_remove_subscriber(subscribers_group_t *group, snd_seq_addr_t *dest);
+extern int snd_seq_port_remove_subscriber(subscribers_group_t *group, snd_seq_addr_t *dest,int check_queue, int queue);
 
 /* subscribe port */
 int snd_seq_port_subscribe(client_port_t *port, snd_seq_port_subscribe_t *info);
@@ -135,4 +140,10 @@ int snd_seq_port_use(client_port_t *port, snd_seq_port_subscribe_t *info);
 /* unuse port */
 int snd_seq_port_unuse(client_port_t *port, snd_seq_port_subscribe_t *info);
 
+/* check if the address was already subscribed */
+int snd_seq_port_is_subscribed(subscribers_group_t *group, snd_seq_addr_t *addr);
+
+/* get matched subscriber */
+subscribers_t *snd_seq_port_get_subscription(subscribers_group_t *group, snd_seq_addr_t *addr);
+
 #endif
