@@ -55,4 +55,8 @@ extern snd_seq_event_cell_t *snd_seq_prioq_cell_peek(prioq_t *f);
 /* client left queue */
 extern void snd_seq_prioq_leave(prioq_t *f, int client, int timestamp);        
 
+/* Remove events */
+void snd_seq_prioq_remove_events(prioq_t * f, int client,
+	snd_seq_remove_events_t *info);
+
 #endif
