@@ -1,7 +1,7 @@
 /*
  *   ALSA sequencer Timer
- *   Copyright (c) 1998 by Frank van de Pol <frank@vande-pol.demon.nl>
- *                         Jaroslav Kysela <perex@suse.cz>
+ *   Copyright (c) 1998-1999 by Frank van de Pol <frank@vande-pol.demon.nl>
+ *                              Jaroslav Kysela <perex@suse.cz>
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -28,25 +28,37 @@
 extern int snd_seq_default_timer;
 extern int snd_seq_default_timer_resolution;
 
-static void inline snd_seq_timer_set_tick_time(timer_t *tmr)
+static void snd_seq_timer_set_tick_time(seq_timer_t *tmr)
 {
-	tmr->tick_time = (tmr->tempo * 1000) / tmr->ppq;
+	if (tmr->tempo < 1000000)
+		tmr->tick_time = (tmr->tempo * 1000) / tmr->ppq;
+	else {
+		/* might overflow.. */
+		unsigned int s;
+		s = tmr->tempo % tmr->ppq;
+		s = (s * 1000) / tmr->ppq;
+		tmr->tick_time = (tmr->tempo / tmr->ppq) * 1000;
+		tmr->tick_time += s;
+	}
+	if (tmr->tick_time <= 0)
+		tmr->tick_time = 1;
 }
 
 /* create new timer (constructor) */
-timer_t *snd_seq_timer_new(void)
+seq_timer_t *snd_seq_timer_new(void)
 {
-	timer_t *tmr;
+	seq_timer_t *tmr;
 	
-	tmr = snd_kcalloc(sizeof(timer_t), GFP_KERNEL);
+	tmr = snd_kcalloc(sizeof(seq_timer_t), GFP_KERNEL);
 	if (tmr == NULL) {
 		snd_printd("malloc failed for snd_seq_timer_new() \n");
 		return NULL;
 	}
-	tmr->lock = SPIN_LOCK_UNLOCKED;
+	spin_lock_init(&tmr->lock);
 
 	/* reset setup to defaults */
 	snd_seq_timer_defaults(tmr);
+	
 	/* reset time */
 	snd_seq_timer_reset(tmr);
 	
@@ -54,9 +66,9 @@ timer_t *snd_seq_timer_new(void)
 }
 
 /* delete timer (destructor) */
-void snd_seq_timer_delete(timer_t **tmr)
+void snd_seq_timer_delete(seq_timer_t **tmr)
 {
-	timer_t *t = *tmr;
+	seq_timer_t *t = *tmr;
 	*tmr = NULL;
 
 	if (t == NULL) {
@@ -72,7 +84,7 @@ void snd_seq_timer_delete(timer_t **tmr)
 	snd_kfree(t);
 }
 
-void snd_seq_timer_defaults(timer_t * tmr)
+void snd_seq_timer_defaults(seq_timer_t * tmr)
 {
 	/* setup defaults */
 	tmr->ppq = 96;		/* 96 PPQ */
@@ -86,7 +98,7 @@ void snd_seq_timer_defaults(timer_t * tmr)
 	tmr->timer_no = snd_seq_default_timer;
 }
 
-void snd_seq_timer_reset(timer_t * tmr)
+void snd_seq_timer_reset(seq_timer_t * tmr)
 {
 	unsigned long flags;
 
@@ -103,7 +115,9 @@ void snd_seq_timer_reset(timer_t * tmr)
 
 	tmr->cur_tick_time = 0;
 
+#ifdef use_seqsync
 	tmr->sync_tmp = 0;
+#endif
 	spin_unlock_irqrestore(&tmr->lock, flags);
 }
 
@@ -116,9 +130,9 @@ static void snd_seq_timer_interrupt(snd_timer_instance_t *timeri,
 	unsigned long flags;
 	snd_seq_real_time_t period;
 	queue_t *q = (queue_t *)data;
-	timer_t *tmr;
+	seq_timer_t *tmr;
 
-	q = queueptr(q->queue);
+	/*q = queueptr(q->queue);*/
 	if (q == NULL)
 		return;
 	tmr = q->timer;
@@ -150,44 +164,28 @@ static void snd_seq_timer_interrupt(snd_timer_instance_t *timeri,
 
 	spin_unlock_irqrestore(&tmr->lock, flags);
 
-#if 0
-	/* sync timer has expired *//* FIXME: hack to send midi sync info */
-	if ((tmr->cur_tick > tmr->sync_tmp) && (snd_seq_unused_cells() > 10)) {
-		snd_seq_event_t ev;
-
-		ev.type = SND_SEQ_EVENT_CLOCK;
-		ev.flags = SND_SEQ_TIME_STAMP_TICK | SND_SEQ_TIME_MODE_ABS;
-		ev.time.tick = 0;
-		ev.source.queue = 0;
-		ev.source.client = 0;
-		ev.source.port = 0;
-		ev.dest.queue = 1;
-		ev.dest.client = 255;
-		ev.dest.port = 255;
-		if (!snd_seq_kernel_client_dispatch(0, &ev)) {
-			tmr->sync_tmp += tmr->ppq / 24;
-	}
-#endif
 	/* check queues and dispatch events */
-	snd_seq_check_queue(q, 1);
+	snd_seq_check_queue(q, 1, 0);
 }
 
 
-int snd_seq_timer_set_tempo(timer_t * tmr, int tempo)
+int snd_seq_timer_set_tempo(seq_timer_t * tmr, int tempo)
 {
 	unsigned long flags;
 
 	if (tmr && tempo > 0) {
 		spin_lock_irqsave(&tmr->lock, flags);
-		/* store location of tempo change */
-		tmr->tempo_tick = tmr->cur_tick;
-		tmr->tempo_time.tv_sec = tmr->cur_time.tv_sec;
-		tmr->tempo_time.tv_nsec = tmr->cur_time.tv_nsec;
+		if (tempo != tmr->tempo) {
+			/* store location of tempo change */
+			tmr->tempo_tick = tmr->cur_tick;
+			tmr->tempo_time.tv_sec = tmr->cur_time.tv_sec;
+			tmr->tempo_time.tv_nsec = tmr->cur_time.tv_nsec;
 
-		tmr->tempo = tempo;
-		snd_seq_timer_set_tick_time(tmr);
-		tmr->cur_tick += tmr->cur_tick_time / tmr->tick_time;
-		tmr->cur_tick_time %= tmr->tick_time;
+			tmr->tempo = tempo;
+			snd_seq_timer_set_tick_time(tmr);
+			tmr->cur_tick += tmr->cur_tick_time / tmr->tick_time;
+			tmr->cur_tick_time %= tmr->tick_time;
+		}
 		spin_unlock_irqrestore(&tmr->lock, flags);
 		return 0;
 	}
@@ -195,12 +193,12 @@ int snd_seq_timer_set_tempo(timer_t * tmr, int tempo)
 }
 
 
-int snd_seq_timer_set_ppq(timer_t * tmr, int ppq)
+int snd_seq_timer_set_ppq(seq_timer_t * tmr, int ppq)
 {
 	unsigned long flags;
 
-	if (tmr && ppq > 0) {
-		if (tmr->running && (ppq != tmr->ppq)) {
+	if (tmr && ppq > 0 && ppq != tmr->ppq) {
+		if (tmr->running) {
 			/* refuse to change ppq on running timers */
 			/* because it will upset the song position (ticks) */
 			snd_printd("seq: cannot change ppq of a running timer\n");
@@ -219,26 +217,68 @@ int snd_seq_timer_set_ppq(timer_t * tmr, int ppq)
 }
 
 
+extern int snd_seq_timer_set_position_tick(seq_timer_t *tmr, snd_seq_tick_time_t position)
+{
+	unsigned long flags;
+
+	if (tmr) {
+		spin_lock_irqsave(&tmr->lock, flags);
+		
+		/* set position */
+		tmr->cur_tick = position;
+
+		/* store location of position change */
+		tmr->tempo_tick = tmr->cur_tick;
+		tmr->tempo_time = tmr->cur_time;
+		
+		spin_unlock_irqrestore(&tmr->lock, flags);
+		return 0;
+	}
+	return -EINVAL;
+}
+
+
+extern int snd_seq_timer_set_position_time(seq_timer_t *tmr, snd_seq_real_time_t position)
+{
+	unsigned long flags;
+
+	if (tmr) {
+		spin_lock_irqsave(&tmr->lock, flags);
+				
+		/* set position */
+		snd_seq_sanity_real_time(&position);
+		tmr->cur_time = position;
+
+		/* store location of position change */
+		tmr->tempo_tick = tmr->cur_tick;
+		tmr->tempo_time = tmr->cur_time;
+		
+		spin_unlock_irqrestore(&tmr->lock, flags);
+		return 0;
+	}
+	return -EINVAL;
+}
+
+
 int snd_seq_timer_open(queue_t *q)
 {
 	snd_timer_instance_t *t;
-	timer_t *tmr;
+	seq_timer_t *tmr;
 	unsigned int tmp, tmp1;
 	char str[32];
 
 	tmr = q->timer;
-	if (!tmr)
-		return -EINVAL;
+	snd_debug_check(tmr == NULL, -EINVAL);
 	if (tmr->timeri)
 		return -EBUSY;
 	sprintf(str, "sequencer queue %i", q->queue);
 	switch (tmr->type) {
 	case SND_SEQ_TIMER_MASTER:	/* standard timer */
 		t = snd_timer_open(str, tmr->timer_no, SND_TIMER_STYPE_SEQUENCER, q->queue);
-		if (!t) {
+		if (t == NULL) {
 			if (tmr->timer_no != (SND_TIMER_TYPE_GLOBAL | SND_TIMER_GLOBAL_SYSTEM))
 				t = snd_timer_open(str, SND_TIMER_TYPE_GLOBAL | SND_TIMER_GLOBAL_SYSTEM, SND_TIMER_STYPE_SEQUENCER, q->queue);
-			if (!t) {
+			if (t == NULL) {
 				snd_printk("sequencer fatal error: cannot create timer\n");
 				return -ENODEV;
 			}
@@ -246,7 +286,7 @@ int snd_seq_timer_open(queue_t *q)
 		break;
 	case SND_SEQ_TIMER_SLAVE:
 		t = snd_timer_open_slave(str, tmr->slave_type, tmr->timer_no);
-		if (!t)
+		if (t == NULL)
 			return -ENODEV;
 		break;
 	default:
@@ -272,11 +312,10 @@ int snd_seq_timer_open(queue_t *q)
 
 int snd_seq_timer_close(queue_t *q)
 {
-	timer_t *tmr;
+	seq_timer_t *tmr;
 	
 	tmr = q->timer;
-	if (!tmr)
-		return -EINVAL;
+	snd_debug_check(tmr == NULL, -EINVAL);
 	if (tmr->timeri) {
 		snd_timer_stop(tmr->timeri);
 		snd_timer_close(tmr->timeri);
@@ -285,7 +324,7 @@ int snd_seq_timer_close(queue_t *q)
 	return 0;
 }
 
-void snd_seq_timer_stop(timer_t * tmr)
+void snd_seq_timer_stop(seq_timer_t * tmr)
 {
 	if (!tmr->running)
 		return;
@@ -293,7 +332,7 @@ void snd_seq_timer_stop(timer_t * tmr)
 	snd_timer_stop(tmr->timeri);
 }
 
-void snd_seq_timer_start(timer_t * tmr)
+void snd_seq_timer_start(seq_timer_t * tmr)
 {
 	if (tmr->running)
 		snd_seq_timer_stop(tmr);
@@ -303,7 +342,7 @@ void snd_seq_timer_start(timer_t * tmr)
 	do_gettimeofday(&tmr->last_update);
 }
 
-void snd_seq_timer_continue(timer_t * tmr)
+void snd_seq_timer_continue(seq_timer_t * tmr)
 {
 	if (tmr->running)
 		return;
@@ -324,27 +363,33 @@ int snd_seq_timer_midi_close(queue_t *q)
 
 
 /* return current 'real' time. use timeofday() to get better granularity. */
-snd_seq_real_time_t snd_seq_timer_get_cur_time(timer_t *tmr)
+snd_seq_real_time_t snd_seq_timer_get_cur_time(seq_timer_t *tmr)
 {
-	snd_seq_real_time_t cur_time = tmr->cur_time;
-	struct timeval tm;
+	snd_seq_real_time_t cur_time;
 	
+	cur_time = tmr->cur_time;
+#if 0 /* FIXME: is this necessary? */
 	if (tmr->running) { 
+		struct timeval tm;
+		int usec;
 		do_gettimeofday(&tm);
-		cur_time.tv_nsec += 1000 * (tm.tv_usec - tmr->last_update.tv_usec);
-		while (cur_time.tv_nsec >= 1000000000) {
-			/* roll-over */
-			cur_time.tv_nsec -= 1000000000;
-        	        cur_time.tv_sec ++;
-        	}
+		usec = (int)(tm.tv_usec - tmr->last_update.tv_usec);
+		if (usec < 0) {
+			cur_time.tv_nsec += (1000000 + usec) * 1000;
+			cur_time.tv_sec += tm.tv_sec - tmr->last_update.tv_sec - 1;
+		} else {
+			cur_time.tv_nsec += usec * 1000;
+			cur_time.tv_sec += tm.tv_sec - tmr->last_update.tv_sec;
+		}
+		snd_seq_sanity_real_time(&cur_time);
         }
-                
+#endif
 	return cur_time;	
 }
 
 /* TODO: use interpolation on tick queue (will only be usefull for very
  high PPQ values) */
-snd_seq_tick_time_t snd_seq_timer_get_cur_tick(timer_t *tmr)
+snd_seq_tick_time_t snd_seq_timer_get_cur_tick(seq_timer_t *tmr)
 {
 	return tmr->cur_tick;
 }
@@ -355,7 +400,7 @@ void snd_seq_info_timer_read(snd_info_buffer_t * buffer, void *private_data)
 {
 	int idx;
 	queue_t *q;
-	timer_t *tmr;
+	seq_timer_t *tmr;
 	snd_timer_instance_t *ti;
 	unsigned long resolution;
 	
