@@ -196,7 +196,7 @@ static void snd_gf1_pcm_interrupt_wave(snd_gus_card_t * gus, snd_gus_voice_t *pv
 	gus = pcmp->gus;
 	runtime = pcmp->subchn->runtime;
 
-	spin_lock_irq(&gus->reg_lock);
+	spin_lock(&gus->reg_lock);
 	snd_gf1_select_voice(gus, pvoice->number);
 	voice_ctrl = snd_gf1_read8(gus, SND_GF1_VB_ADDRESS_CONTROL) & ~0x8b;
 	ramp_ctrl = (snd_gf1_read8(gus, SND_GF1_VB_VOLUME_CONTROL) & ~0xa4) | 0x03;
@@ -239,7 +239,7 @@ static void snd_gf1_pcm_interrupt_wave(snd_gus_card_t * gus, snd_gus_voice_t *pv
 			voice_ctrl &= ~0x20;
 		}
 	}
-	spin_unlock_irq(&gus->reg_lock);
+	spin_unlock(&gus->reg_lock);
 
 	snd_pcm_transfer_done(pcmp->subchn);
 #if 0
@@ -263,10 +263,10 @@ static void snd_gf1_pcm_interrupt_volume(snd_gus_card_t * gus, snd_gus_voice_t *
 	gus_pcm_private_t *pcmp = snd_magic_cast(gus_pcm_private_t, pvoice->private_data, );
 
 	/* stop ramp, but leave rollover bit untouched */
-	spin_lock_irq(&gus->reg_lock);
+	spin_lock(&gus->reg_lock);
 	snd_gf1_select_voice(gus, pvoice->number);
 	snd_gf1_ctrl_stop(gus, SND_GF1_VB_VOLUME_CONTROL);
-	spin_unlock_irq(&gus->reg_lock);
+	spin_unlock(&gus->reg_lock);
 	if (pcmp == NULL)
 		return;
 	/* are we active? */
@@ -277,11 +277,11 @@ static void snd_gf1_pcm_interrupt_volume(snd_gus_card_t * gus, snd_gus_voice_t *
 	if (pcmp->subchn == NULL)
 		return;
 	vol = !cvoice ? gus->gf1.pcm_volume_level_left : gus->gf1.pcm_volume_level_right;
-	spin_lock_irq(&gus->reg_lock);
+	spin_lock(&gus->reg_lock);
 	snd_gf1_select_voice(gus, pvoice->number);
 	snd_gf1_write16(gus, SND_GF1_VW_VOLUME, vol);
 	pcmp->final_volume = 1;
-	spin_unlock_irq(&gus->reg_lock);
+	spin_unlock(&gus->reg_lock);
 }
 
 static void snd_gf1_pcm_volume_change(snd_gus_card_t * gus)
@@ -359,8 +359,9 @@ static void snd_gf1_pcm_separate_8bit(unsigned char *buf, int dma_size, int coun
 }
 				       
 static int snd_gf1_pcm_playback_memcpy(snd_pcm_subchn_t *subchn,
+				       int voice,
 				       int pos,
-				       const void *src,
+				       void *src,
 				       int count)
 {
 	snd_pcm_runtime_t *runtime = subchn->runtime;
@@ -594,7 +595,7 @@ static snd_pcm_hardware_t snd_gf1_pcm_playback =
 	SND_PCM_CHNINFO_BLOCK,
 	SND_PCM_FMT_S8 | SND_PCM_FMT_U8 |
 	SND_PCM_FMT_S16_LE | SND_PCM_FMT_U16_LE,	/* hardware formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_44100,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_44100,
 	5510,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
@@ -607,7 +608,6 @@ static snd_pcm_hardware_t snd_gf1_pcm_playback =
 	snd_gf1_pcm_playback_ioctl,
 	snd_gf1_pcm_playback_prepare,
 	snd_gf1_pcm_playback_trigger,
-	snd_pcm_playback_write,
 	snd_gf1_pcm_playback_pointer
 };
 
@@ -617,7 +617,7 @@ static snd_pcm_hardware_t snd_gf1_pcm_capture =
 	SND_PCM_CHNINFO_BLOCK | SND_PCM_CHNINFO_INTERLEAVE |
 	SND_PCM_CHNINFO_MMAP_VALID,
 	SND_PCM_FMT_S8 | SND_PCM_FMT_U8,	/* hardware formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_44100,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_44100,
 	5510,			/* min. rate */
 	44100,			/* max. rate */
 	1,			/* min. voices */
@@ -630,7 +630,6 @@ static snd_pcm_hardware_t snd_gf1_pcm_capture =
 	snd_gf1_pcm_capture_ioctl,
 	snd_gf1_pcm_capture_prepare,
 	snd_gf1_pcm_capture_trigger,
-	snd_pcm_capture_read,
 	snd_gf1_pcm_capture_pointer
 };
 
@@ -659,7 +658,7 @@ static int snd_gf1_pcm_playback_open(void *private_data,
 	if (pcmp == NULL)
 		return -ENOMEM;
 	pcmp->gus = gus;
-	pcmp->lock = SPIN_LOCK_UNLOCKED;
+	spin_lock_init(&pcmp->lock);
 	init_waitqueue_head(&pcmp->sleep);
 	atomic_set(&pcmp->dma_count, 0);
 
