@@ -214,9 +214,61 @@ static void snd_cs4236_capture_format(cs4231_t * codec, unsigned char cdfr)
 	spin_unlock_irqrestore(&codec->reg_lock, flags);
 }
 
+#ifdef CONFIG_PM
+
+static void snd_cs4236_suspend(cs4231_t * codec)
+{
+	int reg;
+	unsigned long flags;
+	
+	spin_lock_irqsave(&codec->reg_lock, flags);
+	for (reg = 0; reg < 32; reg++)
+		codec->image[reg] = snd_cs4231_in(codec, reg);
+	for (reg = 0; reg < 18; reg++)
+		codec->eimage[reg] = snd_cs4236_ext_in(codec, CS4236_I23VAL(reg));
+	for (reg = 2; reg < 9; reg++)
+		codec->cimage[reg] = snd_cs4236_ctrl_in(codec, reg);
+	spin_unlock_irqrestore(&codec->reg_lock, flags);
+}
+
+static void snd_cs4236_resume(cs4231_t * codec)
+{
+	int reg;
+	unsigned long flags;
+	
+	snd_cs4231_mce_up(codec);
+	spin_lock_irqsave(&codec->reg_lock, flags);
+	for (reg = 0; reg < 32; reg++) {
+		switch (reg) {
+		case CS4236_EXT_REG:
+		case CS4231_VERSION:
+		case 27:	/* why? CS4235 - master left */
+		case 29:	/* why? CS4235 - master right */
+			break;
+		default:
+			snd_cs4231_out(codec, reg, codec->image[reg]);
+			break;
+		}
+	}
+	for (reg = 0; reg < 18; reg++)
+		snd_cs4236_ext_out(codec, CS4236_I23VAL(reg), codec->eimage[reg]);
+	for (reg = 2; reg < 9; reg++) {
+		switch (reg) {
+		case 7:
+			break;
+		default:
+			snd_cs4236_ctrl_out(codec, reg, codec->cimage[reg]);
+		}
+	}
+	spin_unlock_irqrestore(&codec->reg_lock, flags);
+	snd_cs4231_mce_down(codec);
+}
+
+#endif /* CONFIG_PM */
+
 int snd_cs4236_new_pcm(snd_card_t * card, int device,
-		       unsigned short port,
-		       unsigned short cport,	/* control port */
+		       unsigned long port,
+		       unsigned long cport,	/* control port */
 		       snd_irq_t * irqptr,
 		       snd_dma_t * dmaptr1,
 		       snd_dma_t * dmaptr2,
@@ -234,8 +286,17 @@ int snd_cs4236_new_pcm(snd_card_t * card, int device,
 		return err;
 	codec = snd_magic_cast(cs4231_t, pcm->private_data, -ENXIO);
 	if (!(codec->hardware & CS4231_HW_CS4236_MASK)) {
-		snd_device_free(card, pcm);
-		return -ENODEV;
+		if (codec->hardware == CS4231_HW_CS4232A) {
+			/* exception - down version from original cs4236
+			   see snd_cs4231_probe() in cs4231.c */
+			strcpy(pcm->id, "CS4236");
+			codec->cport = cport;
+			*rpcm = pcm;
+			return 0;
+		} else {
+			snd_device_free(card, pcm);
+			return -ENODEV;
+		}
 	}
 	strcpy(pcm->id, "CS4236");
 	codec->cport = cport;
@@ -251,7 +312,7 @@ int snd_cs4236_new_pcm(snd_card_t * card, int device,
 		ver2 = snd_cs4236_ext_in(codec, CS4236_VERSION);
 		snd_printdd("CS4236: [0x%x] C1 (version) = 0x%x, ext = 0x%x\n", cport, ver1, ver2);
 		if (ver1 != ver2) {
-			snd_printk("CS4236+ chip detected, but control port 0x%x is not valid\n", cport);
+			snd_printk("CS4236+ chip detected, but control port 0x%lx is not valid\n", cport);
 			snd_device_free(card, pcm);
 			return -ENODEV;
 		}
@@ -273,6 +334,10 @@ int snd_cs4236_new_pcm(snd_card_t * card, int device,
 	  codec->set_capture_rate = snd_cs4236_xrate;
 	codec->set_playback_format = snd_cs4236_playback_format;
 	codec->set_capture_format = snd_cs4236_capture_format;
+#ifdef CONFIG_PM
+	codec->suspend = snd_cs4236_suspend;
+	codec->resume = snd_cs4236_resume;
+#endif
 	snd_cs4236_ext_out(codec, CS4236_DAC_MUTE, 0xe0);	/* IFSE enable, digital master mute */
 	*rpcm = pcm;
 	return 0;
@@ -875,7 +940,7 @@ static int snd_cs4236_iec958_set(snd_kmixer_t * mixer, snd_kswitch_t * kswitch,
 
 		time = HZ / 5;
 		while (time > 0) {
-			current->state = TASK_INTERRUPTIBLE;
+			set_current_state(TASK_INTERRUPTIBLE);
 			time = schedule_timeout(time);
 		}
 
@@ -889,7 +954,7 @@ static int snd_cs4236_iec958_set(snd_kmixer_t * mixer, snd_kswitch_t * kswitch,
 
 		time = HZ / 5;
 		while (time > 0) {
-			current->state = TASK_INTERRUPTIBLE;
+			set_current_state(TASK_INTERRUPTIBLE);
 			time = schedule_timeout(time);
 		}
 
@@ -916,12 +981,9 @@ static int snd_cs4236_iec958_set(snd_kmixer_t * mixer, snd_kswitch_t * kswitch,
 
 static snd_kswitch_t snd_cs4236_iec958 =
 {
-	SND_MIXER_SW_IEC958_OUTPUT,
-	(snd_get_switch_t *)snd_cs4236_iec958_get,
-	(snd_set_switch_t *)snd_cs4236_iec958_set,
-	0,
-	NULL,
-	NULL
+	name:	SND_MIXER_SW_IEC958_OUTPUT,
+	get:	(snd_get_switch_t *)snd_cs4236_iec958_get,
+	set:	(snd_set_switch_t *)snd_cs4236_iec958_set,
 };
 
 static int snd_cs4236_mixer_group_ctrl1(snd_kmixer_group_t * group,
@@ -2247,11 +2309,15 @@ EXPORT_SYMBOL(snd_cs4236_new_mixer);
  *  INIT part
  */
 
-int init_module(void)
+static int __init alsa_cs4236_init(void)
 {
 	return 0;
 }
 
-void cleanup_module(void)
+static void __exit alsa_cs4236_exit(void)
 {
 }
+
+module_init(alsa_cs4236_init)
+module_exit(alsa_cs4236_exit)
+MODULE_LICENSE("GPL");
