@@ -1,7 +1,7 @@
 
 /*
     opti93x.c - lowlevel code for OPTi 82c93x chips
-    Copyright (C) 1999 by Massimo Piccioni <dafastidio@libero.it>
+    Copyright (C) 1999-2000 by Massimo Piccioni <dafastidio@libero.it>
 
     Inspired from CS4231 code by Jaroslav Kysela.
 
@@ -105,7 +105,7 @@ static unsigned char snd_opti93x_in(opti93x_t *codec, unsigned char reg)
 }
 
 static void snd_opti93x_out(opti93x_t *codec, unsigned char reg,
-							unsigned char value)
+			    unsigned char value)
 {
 	snd_opti93x_busy_wait(codec);
 	outb(codec->mce_bit | (reg & 0x1f), OPTi93X_PORT(codec, INDEX));
@@ -113,13 +113,13 @@ static void snd_opti93x_out(opti93x_t *codec, unsigned char reg,
 }
 
 static void snd_opti93x_out_image(opti93x_t *codec, unsigned char reg,
-							unsigned char value)
+				  unsigned char value)
 {
 	snd_opti93x_out(codec, reg, codec->image[reg] = value);
 }
 
 static void snd_opti93x_out_mask(opti93x_t *codec, unsigned char reg,
-					unsigned char mask, unsigned char value)
+				 unsigned char mask, unsigned char value)
 {
 	snd_opti93x_out_image(codec, reg,
 		(codec->image[reg] & ~mask) | (value & mask));
@@ -188,7 +188,7 @@ static void snd_opti93x_mute(opti93x_t *codec, int mute)
 
 
 static unsigned int snd_opti93x_get_count(unsigned char format,
-							unsigned int size)
+					  unsigned int size)
 {
 	switch (format & 0xe0) {
 	case OPTi93X_LINEAR_16_LIT:
@@ -227,8 +227,8 @@ static unsigned int snd_opti93x_xrate(opti93x_t *codec, unsigned int rate)
 
 
 static int snd_opti93x_playback_ioctl(void *private_data,
-					snd_pcm_subchn_t *subchn,
-					unsigned int cmd, unsigned long *arg)
+				      snd_pcm_subchn_t *subchn,
+				      unsigned int cmd, unsigned long *arg)
 {
 	opti93x_t *codec = snd_magic_cast(opti93x_t, private_data, -ENXIO);
 	int error;
@@ -243,8 +243,8 @@ static int snd_opti93x_playback_ioctl(void *private_data,
 }
 
 static int snd_opti93x_capture_ioctl(void *private_data,
-					snd_pcm_subchn_t *subchn,
-					unsigned int cmd, unsigned long *arg)
+				     snd_pcm_subchn_t *subchn,
+				     unsigned int cmd, unsigned long *arg)
 {
 	return snd_opti93x_playback_ioctl(private_data, subchn, cmd, arg);
 }
@@ -358,7 +358,7 @@ static void snd_opti93x_close(opti93x_t *codec, unsigned int mode)
 }
 
 static int snd_opti93x_trigger(opti93x_t *codec, unsigned char what,
-							int channel, int cmd)
+			       int channel, int cmd)
 {
 	int error = 0;
 	unsigned long flags;
@@ -433,7 +433,7 @@ static int snd_opti93x_playback_trigger(void *private_data,
 }
 
 static int snd_opti93x_capture_trigger(void *private_data,
-					snd_pcm_subchn_t * subchn, int cmd)
+				       snd_pcm_subchn_t * subchn, int cmd)
 {
 	opti93x_t *codec = snd_magic_cast(opti93x_t, private_data, -ENXIO);
 	return snd_opti93x_trigger(codec,
@@ -441,7 +441,7 @@ static int snd_opti93x_capture_trigger(void *private_data,
 }
 
 static int snd_opti93x_playback_prepare(void *private_data,
-				snd_pcm_subchn_t * subchn)
+					snd_pcm_subchn_t * subchn)
 {
 	opti93x_t *codec = snd_magic_cast(opti93x_t, private_data, -ENXIO);
 	snd_pcm_runtime_t *runtime = subchn->runtime;
@@ -475,7 +475,7 @@ static int snd_opti93x_playback_prepare(void *private_data,
 }
 
 static int snd_opti93x_capture_prepare(void *private_data,
-				snd_pcm_subchn_t *subchn)
+				       snd_pcm_subchn_t *subchn)
 {
 	opti93x_t *codec = snd_magic_cast(opti93x_t, private_data, -ENXIO);
 	snd_pcm_runtime_t *runtime = subchn->runtime;
@@ -509,7 +509,7 @@ static int snd_opti93x_capture_prepare(void *private_data,
 }
 
 static unsigned int snd_opti93x_playback_pointer(void *private_data,
-						snd_pcm_subchn_t *subchn)
+						 snd_pcm_subchn_t *subchn)
 {
 	opti93x_t *codec = snd_magic_cast(opti93x_t, private_data, -ENXIO);
 
@@ -520,7 +520,7 @@ static unsigned int snd_opti93x_playback_pointer(void *private_data,
 }
 
 static unsigned int snd_opti93x_capture_pointer(void *private_data,
-					       snd_pcm_subchn_t *subchn)
+						snd_pcm_subchn_t *subchn)
 {
 	opti93x_t *codec = snd_magic_cast(opti93x_t, private_data, -ENXIO);
 	
@@ -545,7 +545,6 @@ static void snd_opti93x_overrange(opti93x_t *codec)
 
 void snd_opti93x_interrupt(snd_pcm_t *pcm, unsigned char status)
 {
-	unsigned long flags;
 	opti93x_t *codec;
 
 	codec = snd_magic_cast(opti93x_t, pcm->private_data, );
@@ -553,16 +552,16 @@ void snd_opti93x_interrupt(snd_pcm_t *pcm, unsigned char status)
 		return;
 
 	status = snd_opti9xx_read(codec->chip, OPTi9XX_MC_REG(11));
-	if (status & OPTi93X_IRQ_PLAYBACK)
+	if ((status & OPTi93X_IRQ_PLAYBACK) && codec->playback_subchn)
 		snd_pcm_transfer_done(codec->playback_subchn);
-	if (status & OPTi93X_IRQ_CAPTURE) {
+	if ((status & OPTi93X_IRQ_CAPTURE) && codec->capture_subchn) {
 		snd_opti93x_overrange(codec);
 		snd_pcm_transfer_done(codec->capture_subchn);
 	}
 
-	spin_lock_irqsave(&codec->lock, flags);
+	spin_lock(&codec->lock);
 	outb(0x00, OPTi93X_PORT(codec, STATUS));
-	spin_unlock_irqrestore(&codec->lock, flags);
+	spin_unlock(&codec->lock);
 }
 
 
@@ -587,7 +586,6 @@ static snd_pcm_hardware_t snd_opti93x_playback = {
 	snd_opti93x_playback_ioctl,
 	snd_opti93x_playback_prepare,
 	snd_opti93x_playback_trigger,
-	snd_pcm_playback_write,
 	snd_opti93x_playback_pointer,
 };
 
@@ -612,12 +610,11 @@ static snd_pcm_hardware_t snd_opti93x_capture = {
 	snd_opti93x_capture_ioctl,
 	snd_opti93x_capture_prepare,
 	snd_opti93x_capture_trigger,
-	snd_pcm_capture_read,
 	snd_opti93x_capture_pointer,
 };
 
 static int snd_opti93x_playback_open(void *private_data,
-						snd_pcm_subchn_t *subchn)
+				     snd_pcm_subchn_t *subchn)
 {
 	int error;
 	opti93x_t *codec = snd_magic_cast(opti93x_t, private_data, -ENXIO);
@@ -627,7 +624,7 @@ static int snd_opti93x_playback_open(void *private_data,
 			sizeof(*runtime->hw), GFP_KERNEL)))
 		return -ENOMEM;
 	memcpy(runtime->hw, &snd_opti93x_playback, sizeof(*runtime->hw));
-	runtime->hw_free = _snd_kfree;
+	runtime->hw_free = (snd_kfree_type)_snd_kfree;
 
 	snd_pcm_set_sync(subchn);
 	snd_pcm_set_mixer(subchn, codec->mixer->device, codec->me_playback);
@@ -641,7 +638,7 @@ static int snd_opti93x_playback_open(void *private_data,
 }
 
 static int snd_opti93x_capture_open(void *private_data,
-						snd_pcm_subchn_t *subchn)
+				    snd_pcm_subchn_t *subchn)
 {
 	int error;
 	opti93x_t *codec = snd_magic_cast(opti93x_t, private_data, -ENXIO);
@@ -651,7 +648,7 @@ static int snd_opti93x_capture_open(void *private_data,
 			sizeof(*runtime->hw), GFP_KERNEL)))
 		return -ENOMEM;
 	memcpy(runtime->hw, &snd_opti93x_capture, sizeof(*runtime->hw));
-	runtime->hw_free = _snd_kfree;
+	runtime->hw_free = (snd_kfree_type)_snd_kfree;
 
 	snd_pcm_set_sync(subchn);
 	snd_pcm_set_mixer(subchn, codec->mixer->device, codec->me_capture);
@@ -665,7 +662,7 @@ static int snd_opti93x_capture_open(void *private_data,
 }
 
 static int snd_opti93x_playback_close(void *private_data,
-						snd_pcm_subchn_t *subchn)
+				      snd_pcm_subchn_t *subchn)
 {
 	opti93x_t *codec = snd_magic_cast(opti93x_t, private_data, -ENXIO);
 
@@ -676,7 +673,7 @@ static int snd_opti93x_playback_close(void *private_data,
 }
 
 static int snd_opti93x_capture_close(void *private_data,
-						snd_pcm_subchn_t *subchn)
+				     snd_pcm_subchn_t *subchn)
 {
 	opti93x_t *codec = snd_magic_cast(opti93x_t, private_data, -ENXIO);
 
@@ -724,7 +721,8 @@ static void snd_opti93x_free(void *private_data)
 }
 
 int snd_opti93x_new_pcm(snd_card_t *card, int device, opti9xx_t *chip,
-		snd_dma_t *dma1ptr, snd_dma_t *dma2ptr, snd_pcm_t **rpcm)
+			snd_dma_t *dma1ptr, snd_dma_t *dma2ptr,
+			snd_pcm_t **rpcm)
 {
 	int error;
 	snd_pcm_t *pcm;
@@ -750,7 +748,7 @@ int snd_opti93x_new_pcm(snd_card_t *card, int device, opti9xx_t *chip,
 	codec->dma1ptr = dma1ptr;
 	codec->dma2ptr = dma2ptr;
 
-	codec->lock = SPIN_LOCK_UNLOCKED;
+	spin_lock_init(&codec->lock);
 	codec->hardware = chip->hardware;
 	codec->chip = chip;
 
@@ -799,7 +797,7 @@ int snd_opti93x_new_pcm(snd_card_t *card, int device, opti9xx_t *chip,
 
 
 static int snd_opti93x_mixer_get_recsrc(opti93x_t *codec,
-						snd_kmixer_element_t *element)
+					snd_kmixer_element_t *element)
 {
 	if (element == codec->me_mux_line)
 		return OPTi93X_MIXOUT_LINE;
@@ -830,10 +828,11 @@ static snd_kmixer_element_t *snd_opti93x_mixer_get_recsrc_element(
 }
 
 int snd_opti93x_mixer_mux(snd_kmixer_element_t *element, int w_flag,
-						snd_kmixer_element_t **elements)
+			  snd_kmixer_element_t **elements)
 {
 	int change = 0, left, right, nleft, nright;
-	opti93x_t *codec = snd_magic_cast(opti93x_t, element->private_data, -ENXIO);
+	opti93x_t *codec = snd_magic_cast(opti93x_t, element->private_data,
+		-ENXIO);
 	unsigned long flags;
 
 	spin_lock_irqsave(&codec->lock, flags);
@@ -867,13 +866,14 @@ int snd_opti93x_mixer_mux(snd_kmixer_element_t *element, int w_flag,
 }
 
 int snd_opti93x_mixer_stereo_volume(snd_kmixer_element_t *element,
-						int w_flag, int *voices,
-						int max, int invert, int shift,
-						unsigned char left_reg,
-						unsigned char right_reg)
+				    int w_flag, int *voices,
+				    int max, int invert, int shift,
+				    unsigned char left_reg,
+				    unsigned char right_reg)
 {
-	opti93x_t *codec = snd_magic_cast(opti93x_t, element->private_data, -ENXIO);
 	int change = 0, left, right;
+	opti93x_t *codec = snd_magic_cast(opti93x_t, element->private_data,
+		-ENXIO);
 	unsigned long flags;
 
 	spin_lock_irqsave(&codec->lock, flags);
@@ -883,7 +883,7 @@ int snd_opti93x_mixer_stereo_volume(snd_kmixer_element_t *element,
 
 	if (!w_flag) {
 		voices[0] = invert ? (max - left) : left;
-		voices[0] = invert ? (max - right) : right;
+		voices[1] = invert ? (max - right) : right;
 	}
 	else {
 		change = invert ?
@@ -902,14 +902,15 @@ int snd_opti93x_mixer_stereo_volume(snd_kmixer_element_t *element,
 }
 
 int snd_opti93x_mixer_stereo_switch(snd_kmixer_element_t *element,
-							int w_flag,
-							unsigned int *bitmap,
-							int bit, int invert,
-							unsigned char left_reg,
-							unsigned char right_reg)
+				    int w_flag,
+				    unsigned int *bitmap,
+				    int bit, int invert,
+				    unsigned char left_reg,
+				    unsigned char right_reg)
 {
-	opti93x_t *codec = snd_magic_cast(opti93x_t, element->private_data, -ENXIO);
 	int change = 0, left, right;
+	opti93x_t *codec = snd_magic_cast(opti93x_t, element->private_data,
+		-ENXIO);
 	unsigned long flags;
 
 	spin_lock_irqsave(&codec->lock, flags);
@@ -945,7 +946,7 @@ int snd_opti93x_mixer_stereo_switch(snd_kmixer_element_t *element,
 }
 
 int snd_opti93x_mixer_mic_switch(snd_kmixer_element_t *element, int w_flag,
-							unsigned int *bitmap)
+				 unsigned int *bitmap)
 {
 	return snd_opti93x_mixer_stereo_switch(element,
 		w_flag, bitmap,
@@ -955,9 +956,10 @@ int snd_opti93x_mixer_mic_switch(snd_kmixer_element_t *element, int w_flag,
 }
 
 int snd_opti93x_mixer_mic_volume(snd_kmixer_element_t *element, int w_flag,
-								int *voices)
+				 int *voices)
 {
-	opti93x_t *codec = snd_magic_cast(opti93x_t, element->private_data, -ENXIO);
+	opti93x_t *codec = snd_magic_cast(opti93x_t, element->private_data,
+		-ENXIO);
 	
 	return snd_opti93x_mixer_stereo_volume(element,
 		w_flag, voices,
@@ -967,9 +969,10 @@ int snd_opti93x_mixer_mic_volume(snd_kmixer_element_t *element, int w_flag,
 }
 
 int snd_opti93x_mixer_aux_switch(snd_kmixer_element_t *element, int w_flag,
-							unsigned int *bitmap)
+				 unsigned int *bitmap)
 {
-	opti93x_t *codec = snd_magic_cast(opti93x_t, element->private_data, -ENXIO);
+	opti93x_t *codec = snd_magic_cast(opti93x_t, element->private_data,
+		-ENXIO);
 	
 	return snd_opti93x_mixer_stereo_switch(element,
 		w_flag, bitmap,
@@ -981,9 +984,10 @@ int snd_opti93x_mixer_aux_switch(snd_kmixer_element_t *element, int w_flag,
 }
 
 int snd_opti93x_mixer_aux_volume(snd_kmixer_element_t *element, int w_flag,
-								int *voices)
+				 int *voices)
 {
-	opti93x_t *codec = snd_magic_cast(opti93x_t, element->private_data, -ENXIO);
+	opti93x_t *codec = snd_magic_cast(opti93x_t, element->private_data,
+		-ENXIO);
 	
 	return snd_opti93x_mixer_stereo_volume(element,
 		w_flag, voices,
@@ -995,7 +999,7 @@ int snd_opti93x_mixer_aux_volume(snd_kmixer_element_t *element, int w_flag,
 }
 
 int snd_opti93x_mixer_cd_switch(snd_kmixer_element_t *element, int w_flag,
-							unsigned int *bitmap)
+				unsigned int *bitmap)
 {
 	return snd_opti93x_mixer_stereo_switch(element,
 		w_flag, bitmap,
@@ -1005,7 +1009,7 @@ int snd_opti93x_mixer_cd_switch(snd_kmixer_element_t *element, int w_flag,
 }
 
 int snd_opti93x_mixer_cd_volume(snd_kmixer_element_t *element, int w_flag,
-								int *voices)
+				int *voices)
 {
 	return snd_opti93x_mixer_stereo_volume(element,
 		w_flag, voices,
@@ -1015,7 +1019,7 @@ int snd_opti93x_mixer_cd_volume(snd_kmixer_element_t *element, int w_flag,
 }
 
 int snd_opti931_mixer_fm_switch(snd_kmixer_element_t *element, int w_flag,
-							unsigned int *bitmap)
+				unsigned int *bitmap)
 {
 	return snd_opti93x_mixer_stereo_switch(element,
 		w_flag, bitmap,
@@ -1025,7 +1029,7 @@ int snd_opti931_mixer_fm_switch(snd_kmixer_element_t *element, int w_flag,
 }
 
 int snd_opti931_mixer_fm_volume(snd_kmixer_element_t *element, int w_flag,
-								int *voices)
+				int *voices)
 {
 	return snd_opti93x_mixer_stereo_volume(element,
 		w_flag, voices,
@@ -1035,7 +1039,7 @@ int snd_opti931_mixer_fm_volume(snd_kmixer_element_t *element, int w_flag,
 }
 
 int snd_opti93x_mixer_line_switch(snd_kmixer_element_t *element, int w_flag,
-							unsigned int *bitmap)
+				  unsigned int *bitmap)
 {
 	return snd_opti93x_mixer_stereo_switch(element,
 		w_flag, bitmap,
@@ -1045,7 +1049,7 @@ int snd_opti93x_mixer_line_switch(snd_kmixer_element_t *element, int w_flag,
 }
 
 int snd_opti93x_mixer_line_volume(snd_kmixer_element_t *element, int w_flag,
-								int *voices)
+				  int *voices)
 {
 	return snd_opti93x_mixer_stereo_volume(element,
 		w_flag, voices,
@@ -1055,7 +1059,7 @@ int snd_opti93x_mixer_line_volume(snd_kmixer_element_t *element, int w_flag,
 }
 
 int snd_opti93x_mixer_igain_volume(snd_kmixer_element_t *element, int w_flag,
-								int *voices)
+				   int *voices)
 {
 	return snd_opti93x_mixer_stereo_volume(element,
 		w_flag, voices,
@@ -1065,7 +1069,7 @@ int snd_opti93x_mixer_igain_volume(snd_kmixer_element_t *element, int w_flag,
 }
 
 int snd_opti93x_mixer_mgain_switch(snd_kmixer_element_t *element, int w_flag,
-							unsigned int *bitmap)
+				   unsigned int *bitmap)
 {
 	return snd_opti93x_mixer_stereo_switch(element,
 		w_flag, bitmap,
@@ -1075,7 +1079,7 @@ int snd_opti93x_mixer_mgain_switch(snd_kmixer_element_t *element, int w_flag,
 }
 
 int snd_opti93x_mixer_dac_switch(snd_kmixer_element_t *element, int w_flag,
-							unsigned int *bitmap)
+				 unsigned int *bitmap)
 {
 	return snd_opti93x_mixer_stereo_switch(element,
 		w_flag, bitmap,
@@ -1084,9 +1088,11 @@ int snd_opti93x_mixer_dac_switch(snd_kmixer_element_t *element, int w_flag,
 		OPTi93X_DAC_RIGHT);
 }
 
-int snd_opti93x_mixer_dac_volume(snd_kmixer_element_t *element, int w_flag, int *voices)
+int snd_opti93x_mixer_dac_volume(snd_kmixer_element_t *element, int w_flag,
+				 int *voices)
 {
-	opti93x_t *codec = snd_magic_cast(opti93x_t, element->private_data, -ENXIO);
+	opti93x_t *codec = snd_magic_cast(opti93x_t, element->private_data,
+		-ENXIO);
 	
 	return snd_opti93x_mixer_stereo_volume(element,
 		w_flag, voices,
@@ -1096,7 +1102,7 @@ int snd_opti93x_mixer_dac_volume(snd_kmixer_element_t *element, int w_flag, int
 }
 
 int snd_opti93x_mixer_out_switch(snd_kmixer_element_t *element, int w_flag,
-							unsigned int *bitmap)
+				 unsigned int *bitmap)
 {
 	return snd_opti93x_mixer_stereo_switch(element,
 		w_flag, bitmap,
@@ -1105,9 +1111,11 @@ int snd_opti93x_mixer_out_switch(snd_kmixer_element_t *element, int w_flag,
 		OPTi93X_OUT_RIGHT);
 }
 
-int snd_opti93x_mixer_out_volume(snd_kmixer_element_t *element, int w_flag, int *voices)
+int snd_opti93x_mixer_out_volume(snd_kmixer_element_t *element, int w_flag,
+				 int *voices)
 {
-	opti93x_t *codec = snd_magic_cast(opti93x_t, element->private_data, -ENXIO);
+	opti93x_t *codec = snd_magic_cast(opti93x_t, element->private_data,
+		-ENXIO);
 	
 	return snd_opti93x_mixer_stereo_volume(element,
 		w_flag, voices,
@@ -1117,19 +1125,20 @@ int snd_opti93x_mixer_out_volume(snd_kmixer_element_t *element, int w_flag, int
 }
 
 int snd_opti93x_mixer_group_ctrl(snd_kmixer_group_t *group,
-					snd_kmixer_file_t *file,
-					int w_flag,
-					snd_mixer_group_t *ugroup,
-					snd_mixer_volume1_control_t *volume1,
-					snd_kmixer_element_t *volume1_element,
-					int max,
-					snd_mixer_sw1_control_t *sw1,
-					snd_kmixer_element_t *sw1_element,
-					snd_kmixer_element_t *mux_in)
+				 snd_kmixer_file_t *file,
+				 int w_flag,
+				 snd_mixer_group_t *ugroup,
+				 snd_mixer_volume1_control_t *volume1,
+				 snd_kmixer_element_t *volume1_element,
+				 int max,
+				 snd_mixer_sw1_control_t *sw1,
+				 snd_kmixer_element_t *sw1_element,
+				 snd_kmixer_element_t *mux_in)
 {
-	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data, -ENXIO);
 	unsigned int bitmap;
 	int change = 0;
+	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data,
+		-ENXIO);
 	snd_kmixer_element_t *elements[2];
 	int voices[2];
 
@@ -1212,11 +1221,12 @@ int snd_opti93x_mixer_group_ctrl(snd_kmixer_group_t *group,
 }
 
 static int snd_opti93x_mixer_group_mic(snd_kmixer_group_t *group,
-						snd_kmixer_file_t *file,
-						int w_flag,
-						snd_mixer_group_t *ugroup)
+				       snd_kmixer_file_t *file,
+				       int w_flag,
+				       snd_mixer_group_t *ugroup)
 {
-	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data, -ENXIO);
+	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data,
+		-ENXIO);
 
 	return snd_opti93x_mixer_group_ctrl(group, file, w_flag, ugroup,
 		snd_opti93x_mixer_mic_volume,
@@ -1228,11 +1238,12 @@ static int snd_opti93x_mixer_group_mic(snd_kmixer_group_t *group,
 }
 
 static int snd_opti93x_mixer_group_line(snd_kmixer_group_t *group,
-						snd_kmixer_file_t *file,
-						int w_flag,
-						snd_mixer_group_t *ugroup)
+					snd_kmixer_file_t *file,
+					int w_flag,
+					snd_mixer_group_t *ugroup)
 {
-	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data, -ENXIO);
+	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data,
+		-ENXIO);
 
 	return snd_opti93x_mixer_group_ctrl(group, file, w_flag, ugroup,
 		snd_opti93x_mixer_line_volume,
@@ -1244,11 +1255,12 @@ static int snd_opti93x_mixer_group_line(snd_kmixer_group_t *group,
 }
 
 static int snd_opti93x_mixer_group_aux(snd_kmixer_group_t *group,
-						snd_kmixer_file_t *file,
-						int w_flag,
-						snd_mixer_group_t *ugroup)
+				       snd_kmixer_file_t *file,
+				       int w_flag,
+				       snd_mixer_group_t *ugroup)
 {
-	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data, -ENXIO);
+	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data,
+		-ENXIO);
 
 	return snd_opti93x_mixer_group_ctrl(group, file, w_flag, ugroup,
 		snd_opti93x_mixer_aux_volume,
@@ -1260,11 +1272,12 @@ static int snd_opti93x_mixer_group_aux(snd_kmixer_group_t *group,
 }
 
 static int snd_opti93x_mixer_group_cd(snd_kmixer_group_t *group,
-						snd_kmixer_file_t *file,
-						int w_flag,
-						snd_mixer_group_t *ugroup)
+				      snd_kmixer_file_t *file,
+				      int w_flag,
+				      snd_mixer_group_t *ugroup)
 {
-	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data, -ENXIO);
+	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data,
+		-ENXIO);
 
 	return snd_opti93x_mixer_group_ctrl(group, file, w_flag, ugroup,
 		snd_opti93x_mixer_cd_volume,
@@ -1276,11 +1289,12 @@ static int snd_opti93x_mixer_group_cd(snd_kmixer_group_t *group,
 }
 
 static int snd_opti931_mixer_group_fm(snd_kmixer_group_t *group,
-						snd_kmixer_file_t *file,
-						int w_flag,
-						snd_mixer_group_t *ugroup)
+				      snd_kmixer_file_t *file,
+				      int w_flag,
+				      snd_mixer_group_t *ugroup)
 {
-	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data, -ENXIO);
+	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data,
+		-ENXIO);
 
 	return snd_opti93x_mixer_group_ctrl(group, file, w_flag, ugroup,
 		snd_opti931_mixer_fm_volume,
@@ -1292,11 +1306,12 @@ static int snd_opti931_mixer_group_fm(snd_kmixer_group_t *group,
 }
 
 static int snd_opti93x_mixer_group_igain(snd_kmixer_group_t *group,
-						snd_kmixer_file_t *file,
-						int w_flag,
-						snd_mixer_group_t *ugroup)
+					 snd_kmixer_file_t *file,
+					 int w_flag,
+					 snd_mixer_group_t *ugroup)
 {
-	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data, -ENXIO);
+	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data,
+		-ENXIO);
 
 	return snd_opti93x_mixer_group_ctrl(group, file, w_flag, ugroup,
 		snd_opti93x_mixer_igain_volume,
@@ -1308,11 +1323,12 @@ static int snd_opti93x_mixer_group_igain(snd_kmixer_group_t *group,
 }
 
 static int snd_opti93x_mixer_group_dac(snd_kmixer_group_t *group,
-						snd_kmixer_file_t *file,
-						int w_flag,
-						snd_mixer_group_t *ugroup)
+				       snd_kmixer_file_t *file,
+				       int w_flag,
+				       snd_mixer_group_t *ugroup)
 {
-	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data, -ENXIO);
+	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data,
+		-ENXIO);
 
 	return snd_opti93x_mixer_group_ctrl(group, file, w_flag, ugroup,
 		snd_opti93x_mixer_dac_volume,
@@ -1324,11 +1340,12 @@ static int snd_opti93x_mixer_group_dac(snd_kmixer_group_t *group,
 }
 
 static int snd_opti93x_mixer_group_out(snd_kmixer_group_t *group,
-						snd_kmixer_file_t *file,
-						int w_flag,
-						snd_mixer_group_t *ugroup)
+				       snd_kmixer_file_t *file,
+				       int w_flag,
+				       snd_mixer_group_t *ugroup)
 {
-	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data, -ENXIO);
+	opti93x_t *codec = snd_magic_cast(opti93x_t, group->private_data,
+		-ENXIO);
 
 	return snd_opti93x_mixer_group_ctrl(group, file, w_flag, ugroup,
 		snd_opti93x_mixer_out_volume,
@@ -1339,7 +1356,7 @@ static int snd_opti93x_mixer_group_out(snd_kmixer_group_t *group,
 		NULL);
 }
 
-int snd_opti93x_new_mixer(snd_pcm_t * pcm, int device, snd_kmixer_t **rmixer)
+int snd_opti93x_new_mixer(snd_pcm_t *pcm, int device, snd_kmixer_t **rmixer)
 {
 	int error;
 	opti93x_t *codec;
@@ -1668,15 +1685,15 @@ EXPORT_SYMBOL(snd_opti93x_interrupt);
 EXPORT_SYMBOL(snd_opti93x_new_pcm);
 EXPORT_SYMBOL(snd_opti93x_new_mixer);
 
-#ifdef MODULE
-
-int __init init_module(void)
+static int __init alsa_opti93x_init(void)
 {
 	return 0;
 }
 
-void __exit cleanup_module(void)
+static void __exit alsa_opti93x_exit(void)
 {
 }
 
-#endif
+module_init(alsa_opti93x_init)
+module_exit(alsa_opti93x_exit)
+MODULE_LICENSE("GPL");
