@@ -1,5 +1,6 @@
 /*
  *  Copyright (c) by Jaroslav Kysela <perex@suse.cz>
+ *                   Cirrus Logic, Inc.
  *  Routines for control of Cirrus Logic CS461x chips
  *
  *  BUGS:
@@ -34,6 +35,10 @@
  *  constants
  */
 
+#if 0
+#define ACCEPT_VALID		/* REQUIRED ONLY FOR OSS EMULATION */
+#endif
+
 #define CS461X_BA0_SIZE		0x2000
 #define CS461X_BA1_DATA0_SIZE	0x3000
 #define CS461X_BA1_DATA1_SIZE	0x3800
@@ -46,22 +51,22 @@
 
 static void snd_cs461x_poke(cs461x_t *codec, unsigned long reg, unsigned int val)
 {
-	codec->ba1.idx[(reg >> 16) & 3][(reg >> 2) & 0x3fff] = val;
+	writel(val, &codec->ba1.idx[(reg >> 16) & 3][(reg >> 2) & 0x3fff]);
 }
 
 static unsigned int snd_cs461x_peek(cs461x_t *codec, unsigned long reg)
 {
-	return codec->ba1.idx[(reg >> 16) & 3][(reg >> 2) & 0x3fff];
+	return readl(&codec->ba1.idx[(reg >> 16) & 3][(reg >> 2) & 0x3fff]);
 }
 
 static void snd_cs461x_pokeBA0(cs461x_t *codec, unsigned long reg, unsigned int val)
 {
-	codec->ba0[reg >> 2] = val;
+	writel(val, &codec->ba0[reg >> 2]);
 }
 
 static unsigned int snd_cs461x_peekBA0(cs461x_t *codec, unsigned long reg)
 {
-	return codec->ba0[reg >> 2];
+	return readl(&codec->ba0[reg >> 2]);
 }
 
 static void snd_cs461x_codec_write(void *private_data,
@@ -94,7 +99,7 @@ static void snd_cs461x_codec_write(void *private_data,
 	snd_cs461x_pokeBA0(codec, BA0_ACCDA, val);
 	snd_cs461x_pokeBA0(codec, BA0_ACCTL, ACCTL_DCV | ACCTL_VFRM |
 				             ACCTL_ESYN | ACCTL_RSTN);
-	for (count = 0; count < 1000; count++) {
+	for (count = 0; count < 4000; count++) {
 		/*
 		 *  First, we want to wait for a short time.
 		 */
@@ -104,13 +109,9 @@ static void snd_cs461x_codec_write(void *private_data,
 		 *  ACCTL = 460h, DCV should be reset by now and 460h = 07h
 		 */
 		if (!(snd_cs461x_peekBA0(codec, BA0_ACCTL) & ACCTL_DCV))
-			break;
+			return;
 	}
-	/*
-	 *  Make sure the write completed.
-	 */
-	if (snd_cs461x_peekBA0(codec, BA0_ACCTL) & ACCTL_DCV)
-		snd_printk("cs461x: AC'97 write problem, reg = 0x%x, val = 0x%x\n", reg, val);
+	snd_printk("cs461x: AC'97 write problem, reg = 0x%x, val = 0x%x\n", reg, val);
 }
 
 static unsigned short snd_cs461x_codec_read(void *private_data,
@@ -153,7 +154,7 @@ static unsigned short snd_cs461x_codec_read(void *private_data,
 	/*
 	 *  Wait for the read to occur.
 	 */
-	for (count = 0; count < 500; count++) {
+	for (count = 0; count < 1000; count++) {
 		/*
 		 *  First, we want to wait for a short time.
 	 	 */
@@ -163,17 +164,13 @@ static unsigned short snd_cs461x_codec_read(void *private_data,
 		 *  ACCTL = 460h, DCV should be reset by now and 460h = 17h
 		 */
 		if (!(snd_cs461x_peekBA0(codec, BA0_ACCTL) & ACCTL_DCV))
-			break;
+			goto ok1;
 	}
 
-	/*
-	 *  Make sure the read completed.
-	 */
-	if (snd_cs461x_peekBA0(codec, BA0_ACCTL) & ACCTL_DCV) {
-		snd_printk("cs461x: AC'97 read problem (ACCTL_DCV), reg = 0x%x\n", reg);
-		return 0xffff;
-	}
+	snd_printk("cs461x: AC'97 read problem (ACCTL_DCV), reg = 0x%x\n", reg);
+	return 0xffff;
 
+ok1:
 	/*
 	 *  Wait for the valid status bit to go active.
 	 */
@@ -184,18 +181,14 @@ static unsigned short snd_cs461x_codec_read(void *private_data,
 		 *  VSTS - Valid Status
 		 */
 		if (snd_cs461x_peekBA0(codec, BA0_ACSTS) & ACSTS_VSTS)
-			break;
+			goto ok2;
 		udelay(10);
 	}
 	
-	/*
-	 *  Make sure we got valid status.
-	 */
-	if (!(snd_cs461x_peekBA0(codec, BA0_ACSTS) & ACSTS_VSTS)) {
-		snd_printk("cs461x: AC'97 read problem (ACSTS_VSTS), reg = 0x%x\n", reg);
-		return 0xffff;
-	}
+	snd_printk("cs461x: AC'97 read problem (ACSTS_VSTS), reg = 0x%x\n", reg);
+	return 0xffff;
 
+ok2:
 	/*
 	 *  Read the data returned from the AC97 register.
 	 *  ACSDA = Status Data Register = 474h
@@ -217,17 +210,19 @@ int snd_cs461x_download(cs461x_t *codec,
                         unsigned long offset,
                         unsigned long len)
 {
-	unsigned long counter;
 	unsigned int *dst;
 
 	if ((offset & 3) || (len & 3))
 		return -EINVAL;
 	dst = codec->ba1.idx[(offset >> 16) & 3];
-	offset &= 0xffff;
-	len >>= 2;
-	/* I'm not sure, if the big endian architecture needs the byteswap here */
-	for (counter = 0; counter < len; counter++)
-		dst[offset + counter] = src[counter];
+	dst += ((offset & 0xFFFF) << 2) / sizeof(unsigned int);
+	len /= sizeof(unsigned int);
+
+	/* writel already converts 32-bit value to right endianess */
+	while (len-- > 0) {
+		writel(*src++, dst);
+		dst++;
+	}
 	return 0;
 }
 
@@ -304,7 +299,8 @@ static void snd_cs461x_clear_serial_FIFOs(cs461x_t *codec)
 	 *  See if the devices are powered down.  If so, we must power them up first
 	 *  or they will not respond.
 	 */
-	if (!((tmp = snd_cs461x_peekBA0(codec, BA0_CLKCR1)) & CLKCR1_SWCE)) {
+	tmp = snd_cs461x_peekBA0(codec, BA0_CLKCR1);
+	if (!(tmp & CLKCR1_SWCE)) {
 		snd_cs461x_pokeBA0(codec, BA0_CLKCR1, tmp | CLKCR1_SWCE);
 		powerdown = 1;
 	}
@@ -360,6 +356,8 @@ static void snd_cs461x_powerup_dac(cs461x_t *codec)
 	 *  register.
 	 */
 	tmp = snd_cs461x_codec_read(codec, AC97_POWERDOWN);
+	if (tmp & 2)	/* already */
+		return;
 	snd_cs461x_codec_write(codec, AC97_POWERDOWN, tmp & 0xfdff);
 
 	/*
@@ -397,6 +395,8 @@ static void snd_cs461x_powerup_adc(cs461x_t *codec)
 	 *  register.
 	 */
 	tmp = snd_cs461x_codec_read(codec, AC97_POWERDOWN);
+	if (tmp & 1)	/* already */
+		return;
 	snd_cs461x_codec_write(codec, AC97_POWERDOWN, tmp & 0xfeff);
 
 	/*
@@ -837,7 +837,7 @@ static int snd_cs461x_capture_prepare(void *private_data,
 	codec->cffc = 0;
 	if (runtime->format.voices < 2)
 		codec->cff <<= 1;
-	if (!snd_pcm_format_width(runtime->format.format) == 16)
+	if (snd_pcm_format_width(runtime->format.format) != 16)
 		codec->cff <<= 1;
 	return 0;
 }
@@ -852,7 +852,7 @@ static unsigned int snd_cs461x_playback_pointer(void *private_data,
 	result = snd_cs461x_peek(codec, BA1_PBA) - virt_to_bus(codec->pbuf);
 	if (result > 2048)
 		result -= 2048;
-	if (!snd_pcm_format_width(runtime->format.format) == 16)
+	if (snd_pcm_format_width(runtime->format.format) != 16)
 		result >>= 1;
 	return result;
 }
@@ -867,7 +867,7 @@ static unsigned int snd_cs461x_capture_pointer(void *private_data,
 	result = snd_cs461x_peek(codec, BA1_CBA) - virt_to_bus(codec->cbuf);
 	if (result > 2048)
 		result -= 2048;
-	if (!snd_pcm_format_width(runtime->format.format) == 16)
+	if (snd_pcm_format_width(runtime->format.format) != 16)
 		result >>= 1;
 	return result;
 }
@@ -905,19 +905,27 @@ void snd_cs461x_interrupt(cs461x_t *codec)
 	}
 	if ((status & HISR_MIDI) && codec->rmidi) {
 		unsigned char c;
+		
+		spin_lock(&codec->reg_lock);
 		while ((snd_cs461x_peekBA0(codec, BA0_MIDSR) & MIDSR_RBE) == 0) {
 			c = snd_cs461x_peekBA0(codec, BA0_MIDRP);
 			if ((codec->midcr & MIDCR_RIE) == 0)
 				continue;
+			spin_unlock(&codec->reg_lock);
 			snd_rawmidi_receive(codec->rmidi, &c, 1);
+			spin_lock(&codec->reg_lock);
 		}
 		while ((snd_cs461x_peekBA0(codec, BA0_MIDSR) & MIDSR_TBF) == 0) {
 			if ((codec->midcr & MIDCR_TIE) == 0)
 				break;
-			if (snd_rawmidi_transmit(codec->rmidi, &c, 1) != 1)
+			if (snd_rawmidi_transmit(codec->rmidi, &c, 1) != 1) {
+				codec->midcr &= ~MIDCR_TIE;
+				snd_cs461x_pokeBA0(codec, BA0_MIDCR, codec->midcr);
 				break;
+			}
 			snd_cs461x_pokeBA0(codec, BA0_MIDWP, c);
 		}
+		spin_unlock(&codec->reg_lock);
 	}
 	/*
 	 *  EOI to the PCI part....reenables interrupts
@@ -928,10 +936,13 @@ void snd_cs461x_interrupt(cs461x_t *codec)
 static snd_pcm_hardware_t snd_cs461x_playback =
 {
 	SND_PCM_CHNINFO_MMAP |
+#ifdef ACCEPT_VALID
+	SND_PCM_CHNINFO_MMAP_VALID |	/* NOT TRUE!!! OSS REQUIRES IT */
+#endif
 	SND_PCM_CHNINFO_BLOCK | SND_PCM_CHNINFO_INTERLEAVE |
 	SND_PCM_CHNINFO_BLOCK_TRANSFER,		/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* hardware formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000,
 	5500,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
@@ -944,17 +955,19 @@ static snd_pcm_hardware_t snd_cs461x_playback =
 	snd_cs461x_playback_ioctl,
 	snd_cs461x_playback_prepare,
 	snd_cs461x_playback_trigger,
-	snd_pcm_playback_write,
 	snd_cs461x_playback_pointer
 };
 
 static snd_pcm_hardware_t snd_cs461x_capture =
 {
 	SND_PCM_CHNINFO_MMAP |
+#ifdef ACCEPT_VALID
+	SND_PCM_CHNINFO_MMAP_VALID |	/* NOT TRUE!!! OSS REQUIRES IT */
+#endif
 	SND_PCM_CHNINFO_BLOCK | SND_PCM_CHNINFO_INTERLEAVE |
 	SND_PCM_CHNINFO_BLOCK_TRANSFER,		/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* hardware formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000_48000,
 	5500,			/* min. rate */
 	48000,			/* max. rate */
 	1,			/* min. voices */
@@ -967,7 +980,6 @@ static snd_pcm_hardware_t snd_cs461x_capture =
 	snd_cs461x_capture_ioctl,
 	snd_cs461x_capture_prepare,
 	snd_cs461x_capture_trigger,
-	snd_pcm_capture_read,
 	snd_cs461x_capture_pointer
 };
 
@@ -979,7 +991,7 @@ static int snd_cs461x_playback_open(void *private_data,
 
 	if ((err = snd_pcm_dma_alloc(subchn, codec->dma1ptr, "CS461x - DAC")) < 0)
 		return err;
-	codec->pbuf = snd_malloc_pages(PAGE_SIZE, &pg, 0);
+	codec->pbuf = snd_malloc_pages(PAGE_SIZE, &pg, GFP_KERNEL);
 	if (codec->pbuf == NULL) {
 		snd_pcm_dma_free(subchn);
 		return -ENOMEM;
@@ -987,6 +999,9 @@ static int snd_cs461x_playback_open(void *private_data,
 	codec->playback_subchn = subchn;
 	subchn->runtime->hw = &snd_cs461x_playback;
 	snd_pcm_set_mixer(subchn, codec->mixer->device, codec->ac97->me_playback);
+	codec->active_ctrl(codec, 1);
+	codec->amplifier_ctrl(codec, 1);
+
 	return 0;
 }
 
@@ -998,7 +1013,7 @@ static int snd_cs461x_capture_open(void *private_data,
 
 	if ((err = snd_pcm_dma_alloc(subchn, codec->dma2ptr, "CS461x - ADC")) < 0)
 		return err;
-	codec->cbuf = snd_malloc_pages(PAGE_SIZE, &pg, 0);
+	codec->cbuf = snd_malloc_pages(PAGE_SIZE, &pg, GFP_KERNEL);
 	if (codec->cbuf == NULL) {
 		snd_pcm_dma_free(subchn);
 		return -ENOMEM;
@@ -1006,6 +1021,9 @@ static int snd_cs461x_capture_open(void *private_data,
 	codec->capture_subchn = subchn;
 	subchn->runtime->hw = &snd_cs461x_capture;
 	snd_pcm_set_mixer(subchn, codec->mixer->device, codec->ac97->me_capture);
+	codec->active_ctrl(codec, 1);
+	codec->amplifier_ctrl(codec, 1);
+
 	return 0;
 }
 
@@ -1018,6 +1036,9 @@ static int snd_cs461x_playback_close(void *private_data,
 	if (codec->pbuf)
 		snd_free_pages(codec->pbuf, PAGE_SIZE);
 	snd_pcm_dma_free(subchn);
+	codec->active_ctrl(codec, -1);
+	codec->amplifier_ctrl(codec, -1);
+
 	return 0;
 }
 
@@ -1030,6 +1051,9 @@ static int snd_cs461x_capture_close(void *private_data,
 	if (codec->cbuf)
 		snd_free_pages(codec->cbuf, PAGE_SIZE);
 	snd_pcm_dma_free(subchn);
+	codec->active_ctrl(codec, -1);
+	codec->amplifier_ctrl(codec, -1);
+
 	return 0;
 }
 
@@ -1069,6 +1093,60 @@ int snd_cs461x_pcm(cs461x_t * codec, int device, snd_pcm_t ** rpcm)
  *  Mixer routines
  */
 
+static void snd_cs461x_mixer_write(void *private_data, unsigned short reg, unsigned short val)
+{
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, );
+	int val2 = 0;
+	codec->active_ctrl(codec, 1);
+	if (reg == AC97_CD)
+		val2 = snd_cs461x_codec_read(codec, AC97_CD);
+
+	snd_cs461x_codec_write(private_data, reg, val);
+	
+	/*
+	 *	Adjust power if the mixer is selected/deselected according
+	 *	to the CD.
+	 *
+	 *	IF the CD is a valid input source (mixer or direct) AND
+	 *		the CD is not muted THEN power is needed
+	 *
+	 *	We do two things. When record select changes the input to
+	 *	add/remove the CD we adjust the power count if the CD is
+	 *	unmuted.
+	 *
+	 *	When the CD mute changes we adjust the power level if the
+	 *	CD was a valid input.
+	 *
+	 *      We also check for CD volume != 0, as the CD mute isn't
+	 *      normally tweaked from userspace.
+	 */
+	 
+	/* CD mute change ? */
+	
+	if (reg == AC97_CD) {
+		/* Mute bit change ? */
+		if ((val2^val)&0x8000 || ((val2 == 0x1f1f || val == 0x1f1f) && val2 != val)) {
+			/* Mute on */
+			if(val&0x8000 || val == 0x1f1f)
+				codec->amplifier_ctrl(codec, -1);
+			else /* Mute off power on */
+				codec->amplifier_ctrl(codec, 1);
+		}
+	}
+
+	codec->active_ctrl(codec, -1);
+}
+
+static unsigned short snd_cs461x_mixer_read(void *private_data, unsigned short reg)
+{
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, -ENXIO);
+	unsigned short val;
+	codec->active_ctrl(codec, 1);
+	val = snd_cs461x_codec_read(private_data, reg);
+	codec->active_ctrl(codec, -1);
+	return val;
+}
+
 static void snd_cs461x_mixer_free_ac97(void *private_data)
 {
 	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, );
@@ -1083,8 +1161,8 @@ int snd_cs461x_mixer(cs461x_t * codec, int device, snd_pcm_t * pcm, snd_kmixer_t
 
 	*rmixer = NULL;
 	memset(&ac97, 0, sizeof(ac97));
-	ac97.write = snd_cs461x_codec_write;
-	ac97.read = snd_cs461x_codec_read;
+	ac97.write = snd_cs461x_mixer_write;
+	ac97.read = snd_cs461x_mixer_read;
 	ac97.private_data = codec;
 	ac97.private_free = snd_cs461x_mixer_free_ac97;
 	if ((err = snd_ac97_mixer(codec->card, device, &ac97, 1, &pcm->device, &mixer)) < 0)
@@ -1110,6 +1188,7 @@ static int snd_cs461x_midi_input_open(snd_rawmidi_t * rmidi)
 	unsigned long flags;
 	cs461x_t *codec = snd_magic_cast(cs461x_t, rmidi->private_data, -ENXIO);
 
+	codec->active_ctrl(codec, 1);
 	spin_lock_irqsave(&codec->reg_lock, flags);
 	codec->uartm |= CS461X_MODE_INPUT;
 	codec->midcr |= MIDCR_RXE;
@@ -1136,6 +1215,8 @@ static int snd_cs461x_midi_input_close(snd_rawmidi_t * rmidi)
 	}
 	codec->uartm &= ~CS461X_MODE_INPUT;
 	spin_unlock_irqrestore(&codec->reg_lock, flags);
+
+	codec->active_ctrl(codec, -1);
 	return 0;
 }
 
@@ -1144,6 +1225,8 @@ static int snd_cs461x_midi_output_open(snd_rawmidi_t * rmidi)
 	unsigned long flags;
 	cs461x_t *codec = snd_magic_cast(cs461x_t, rmidi->private_data, -ENXIO);
 
+	codec->active_ctrl(codec, 1);
+
 	spin_lock_irqsave(&codec->reg_lock, flags);
 	codec->uartm |= CS461X_MODE_OUTPUT;
 	codec->midcr |= MIDCR_TXE;
@@ -1170,6 +1253,8 @@ static int snd_cs461x_midi_output_close(snd_rawmidi_t * rmidi)
 	}
 	codec->uartm &= ~CS461X_MODE_OUTPUT;
 	spin_unlock_irqrestore(&codec->reg_lock, flags);
+
+	codec->active_ctrl(codec, -1);
 	return 0;
 }
 
@@ -1179,11 +1264,17 @@ static void snd_cs461x_midi_input_trigger(snd_rawmidi_t * rmidi, int up)
 	cs461x_t *codec = snd_magic_cast(cs461x_t, rmidi->private_data, );
 
 	spin_lock_irqsave(&codec->reg_lock, flags);
-	if (up)
-		codec->midcr |= MIDCR_RIE;
-	else
-		codec->midcr &= ~MIDCR_RIE;
-	snd_cs461x_pokeBA0(codec, BA0_MIDCR, codec->midcr);
+	if (up) {
+		if ((codec->midcr & MIDCR_RIE) == 0) {
+			codec->midcr |= MIDCR_RIE;
+			snd_cs461x_pokeBA0(codec, BA0_MIDCR, codec->midcr);
+		}
+	} else {
+		if (codec->midcr & MIDCR_RIE) {
+			codec->midcr &= ~MIDCR_RIE;
+			snd_cs461x_pokeBA0(codec, BA0_MIDCR, codec->midcr);
+		}
+	}
 	spin_unlock_irqrestore(&codec->reg_lock, flags);
 }
 
@@ -1191,49 +1282,44 @@ static void snd_cs461x_midi_output_trigger(snd_rawmidi_t * rmidi, int up)
 {
 	unsigned long flags;
 	cs461x_t *codec = snd_magic_cast(cs461x_t, rmidi->private_data, );
+	unsigned char byte;
 
 	spin_lock_irqsave(&codec->reg_lock, flags);
 	if (up) {
-		codec->midcr |= MIDCR_TIE;
-		/* fill UART FIFO buffer at first, and turn Tx interrupts only if necessary */
-		while ((codec->midcr & MIDCR_TIE) &&
-		       (snd_cs461x_peekBA0(codec, BA0_MIDSR) & MIDSR_TBF) == 0) {
-			unsigned char byte;
-			if (snd_rawmidi_transmit(rmidi, &byte, 1) != 1) {
-				codec->midcr &= ~MIDCR_TIE;
-			} else {
-				snd_cs461x_pokeBA0(codec, BA0_MIDWP, byte);
+		if ((codec->midcr & MIDCR_TIE) == 0) {
+			codec->midcr |= MIDCR_TIE;
+			/* fill UART FIFO buffer at first, and turn Tx interrupts only if necessary */
+			while ((codec->midcr & MIDCR_TIE) &&
+			       (snd_cs461x_peekBA0(codec, BA0_MIDSR) & MIDSR_TBF) == 0) {
+				if (snd_rawmidi_transmit(rmidi, &byte, 1) != 1) {
+					codec->midcr &= ~MIDCR_TIE;
+				} else {
+					snd_cs461x_pokeBA0(codec, BA0_MIDWP, byte);
+				}
 			}
+			snd_cs461x_pokeBA0(codec, BA0_MIDCR, codec->midcr);
 		}
 	} else {
-		codec->midcr &= ~MIDCR_TIE;
+		if (codec->midcr & MIDCR_TIE) {
+			codec->midcr &= ~MIDCR_TIE;
+			snd_cs461x_pokeBA0(codec, BA0_MIDCR, codec->midcr);
+		}
 	}
-	snd_cs461x_pokeBA0(codec, BA0_MIDCR, codec->midcr);
 	spin_unlock_irqrestore(&codec->reg_lock, flags);
 }
 
 static struct snd_stru_rawmidi_channel_hw snd_cs461x_midi_output =
 {
-	0,			/* flags */
-	NULL,			/* private_data */
-	NULL,			/* private_free */
-	snd_cs461x_midi_output_open,	/* open */
-	snd_cs461x_midi_output_close,	/* close */
-	snd_cs461x_midi_output_trigger,/* trigger */
-	{NULL},			/* io.write */
-	NULL,			/* abort */
+	open:           snd_cs461x_midi_output_open,
+	close:          snd_cs461x_midi_output_close,
+	trigger:        snd_cs461x_midi_output_trigger,
 };
 
 static struct snd_stru_rawmidi_channel_hw snd_cs461x_midi_input =
 {
-	0,			/* flags */
-	NULL,			/* private_data */
-	NULL,			/* private_free */
-	snd_cs461x_midi_input_open,	/* open */
-	snd_cs461x_midi_input_close,	/* close */
-	snd_cs461x_midi_input_trigger,	/* trigger */
-	{NULL},			/* io.read */
-	NULL,			/* abort */
+	open:           snd_cs461x_midi_input_open,
+	close:          snd_cs461x_midi_input_close,
+	trigger:        snd_cs461x_midi_input_trigger,
 };
 
 int snd_cs461x_midi(cs461x_t * codec, int device, snd_rawmidi_t **rrawmidi)
@@ -1242,13 +1328,9 @@ int snd_cs461x_midi(cs461x_t * codec, int device, snd_rawmidi_t **rrawmidi)
 	int err;
 
 	*rrawmidi = NULL;
-	if ((err = snd_rawmidi_new(codec->card, "ES1370/1", device, &rmidi)) < 0)
+	if ((err = snd_rawmidi_new(codec->card, "CS461X", device, &rmidi)) < 0)
 		return err;
-#ifdef CHIP1370
-	strcpy(rmidi->name, "ES1370");
-#else
-	strcpy(rmidi->name, "ES1371");
-#endif
+	strcpy(rmidi->name, "CS461X");
 	memcpy(&rmidi->chn[SND_RAWMIDI_CHANNEL_OUTPUT].hw, &snd_cs461x_midi_output, sizeof(snd_cs461x_midi_output));
 	memcpy(&rmidi->chn[SND_RAWMIDI_CHANNEL_INPUT].hw, &snd_cs461x_midi_input, sizeof(snd_cs461x_midi_input));
 	rmidi->info_flags |= SND_RAWMIDI_INFO_OUTPUT | SND_RAWMIDI_INFO_INPUT | SND_RAWMIDI_INFO_DUPLEX;
@@ -1402,72 +1484,13 @@ static int snd_cs461x_proc_done(cs461x_t * codec)
 }
 
 /*
- *  initialization routines
+ *  initialize chip
  */
 
-int snd_cs461x_create(snd_card_t * card,
-		      struct pci_dev * pci,
-		      snd_dma_t * dma1ptr,
-		      snd_dma_t * dma2ptr,
-		      snd_irq_t * irqptr,
-		      cs461x_t ** rcodec)
+static int snd_cs461x_chip_init(cs461x_t *codec, int busywait)
 {
-	cs461x_t *codec;
-	unsigned short cmdw;
-	unsigned char cmdb;
 	unsigned int tmp;
-	int idx, err;
-	static snd_device_ops_t ops = {
-		(snd_dev_free_t *)snd_cs461x_free,
-		NULL,
-		NULL
-	};
-	
-	*rcodec = NULL;
-	codec = snd_magic_kcalloc(cs461x_t, 0, GFP_KERNEL);
-	if (codec == NULL)
-		return -ENOMEM;
-	codec->reg_lock = SPIN_LOCK_UNLOCKED;
-	codec->card = card;
-	codec->pci = pci;
-	codec->dma1ptr = dma1ptr;
-	codec->dma2ptr = dma2ptr;
-	codec->irqptr = irqptr;
-#ifdef NEW_PCI
-	codec->ba0_addr = pci->resource[0].start;
-	codec->ba1_addr = pci->resource[1].start;
-#else
-	codec->ba0_addr = pci->base_address[0] & ~PCI_BASE_ADDRESS_SPACE;
-	codec->ba1_addr = pci->base_address[1] & ~PCI_BASE_ADDRESS_SPACE;
-#endif
-	if (codec->ba0_addr == 0 || codec->ba0_addr == ~0 ||
-	    codec->ba1_addr == 0 || codec->ba1_addr == ~0) {
-	    	snd_printk("cs461x: wrong address(es) - ba0 = 0x%lx, ba1 = 0x%lx\n", codec->ba0_addr, codec->ba1_addr);
-	    	snd_cs461x_free(codec);
-	    	return -ENOMEM;
-	}
-	codec->ba0 = ioremap(codec->ba0_addr, CS461X_BA0_SIZE);
-	codec->ba1.name.data0 = ioremap(codec->ba1_addr + BA1_SP_DMEM0, CS461X_BA1_DATA0_SIZE);
-	codec->ba1.name.data1 = ioremap(codec->ba1_addr + BA1_SP_DMEM1, CS461X_BA1_DATA1_SIZE);
-	codec->ba1.name.pmem = ioremap(codec->ba1_addr + BA1_SP_PMEM, CS461X_BA1_PRG_SIZE);
-	codec->ba1.name.reg = ioremap(codec->ba1_addr + BA1_SP_REG, CS461X_BA1_REG_SIZE);
-	if (codec->ba0 == NULL || codec->ba1.name.data0 == NULL ||
-	    codec->ba1.name.data1 == NULL || codec->ba1.name.pmem == NULL ||
-	    codec->ba1.name.reg == NULL) {
-		snd_cs461x_free(codec);
-		return -ENOMEM;
-	}
-	pci_set_master(pci);
-	pci_read_config_word(pci, PCI_COMMAND, &cmdw);
-	if ((cmdw & PCI_COMMAND_MEMORY) != PCI_COMMAND_MEMORY) {
-		cmdw |= PCI_COMMAND_MEMORY;
-		pci_write_config_word(pci, PCI_COMMAND, cmdw);
-	}
-	pci_set_master(pci);
-	pci_read_config_byte(pci, PCI_LATENCY_TIMER, &cmdb);
-	if (cmdb < 32)
-		cmdb = 32;
-	pci_write_config_byte(pci, PCI_LATENCY_TIMER, cmdb);
+	int timeout;
 
 	/* 
 	 *  First, blast the clock control register to zero so that the PLL starts
@@ -1491,7 +1514,7 @@ int snd_cs461x_create(snd_card_t * card,
          *  for a reset.
          */
         snd_cs461x_pokeBA0(codec, BA0_ACCTL, 0);
-        udelay(50);
+        udelay(500);
         snd_cs461x_pokeBA0(codec, BA0_ACCTL, ACCTL_RSTN);
 
 	/*
@@ -1506,7 +1529,7 @@ int snd_cs461x_create(snd_card_t * card,
 	 *  generating bit clock (so we don't try to start the PLL without an
 	 *  input clock).
 	 */
-	mdelay(1);
+	mdelay(10);
 
 	/*
 	 *  Set the serial port timing configuration, so that
@@ -1531,7 +1554,7 @@ int snd_cs461x_create(snd_card_t * card,
 	/*
          *  Wait until the PLL has stabilized.
 	 */
-	mdelay(1);
+	mdelay(100);
 
 	/*
 	 *  Turn on clocking of the core so that we can setup the serial ports.
@@ -1557,33 +1580,31 @@ int snd_cs461x_create(snd_card_t * card,
 	snd_cs461x_pokeBA0(codec, BA0_SERC2, SERC2_SI1F_AC97 | SERC1_SO1EN);
 	snd_cs461x_pokeBA0(codec, BA0_SERMC1, SERMC1_PTC_AC97 | SERMC1_MSPE);
 
+	mdelay(5);		/* Shouldnt be needed ?? */
+	
 	/*
 	 * Wait for the codec ready signal from the AC97 codec.
 	 */
-	for (idx = 0; idx < 1000; idx++) {
-		/*
-		 *  First, lets wait a short while to let things settle out a bit,
-		 *  and to prevent retrying the read too quickly.
-		 */
-		udelay(50);
-
+	timeout = 150;
+	while (timeout-- > 0) {
 		/*
 		 *  Read the AC97 status register to see if we've seen a CODEC READY
 		 *  signal from the AC97 codec.
 		 */
 		if (snd_cs461x_peekBA0(codec, BA0_ACSTS) & ACSTS_CRDY)
-			break;
-	}
-
-	/*
-	 *  Make sure CODEC is READY.
-	 */
-	if (!(snd_cs461x_peekBA0(codec, BA0_ACSTS) & ACSTS_CRDY)) {
-		snd_printk("cs461x: create - never read codec ready from AC'97\n");
-		snd_cs461x_free(codec);
-		return -EIO;
+			goto ok1;
+		if (busywait)
+			mdelay(10);
+		else {
+			set_current_state(TASK_UNINTERRUPTIBLE);
+			schedule_timeout((HZ+99)/100);
+		}
 	}
 
+	snd_printk("cs461x: create - never read codec ready from AC'97\n");
+	snd_printk("cs461x: it is not probably bug, try to use CS4236 driver\n");
+	return -EIO;
+ok1:
 	/*
 	 *  Assert the vaid frame signal so that we can start sending commands
 	 *  to the AC97 codec.
@@ -1594,30 +1615,25 @@ int snd_cs461x_create(snd_card_t * card,
 	 *  Wait until we've sampled input slots 3 and 4 as valid, meaning that
 	 *  the codec is pumping ADC data across the AC-link.
 	 */
-	for (idx = 0; idx < 1000; idx++) {
-		/*
-		 *  First, lets wait a short while to let things settle out a bit,
-		 *  and to prevent retrying the read too quickly.
-		 */
-		udelay(500);
+	timeout = 150;
+	while (timeout-- > 0) {
 		/*
 		 *  Read the input slot valid register and see if input slots 3 and
 		 *  4 are valid yet.
 		 */
 		if ((snd_cs461x_peekBA0(codec, BA0_ACISV) & (ACISV_ISV3 | ACISV_ISV4)) == (ACISV_ISV3 | ACISV_ISV4))
-			break;
-	}
-
-	/*
-	 *  Make sure input slots 3 and 4 are valid.  If not, then return
-	 *  an error.
-	 */
-	if ((snd_cs461x_peekBA0(codec, BA0_ACISV) & (ACISV_ISV3 | ACISV_ISV4)) != (ACISV_ISV3 | ACISV_ISV4)) {
-		snd_printk("cs461x: create - never read ISV3 & ISV4 from AC'97\n");
-		snd_cs461x_free(codec);
-		return -EIO;
+			goto ok2;
+		if (busywait)
+			mdelay(10);
+		else {
+			set_current_state(TASK_UNINTERRUPTIBLE);
+			schedule_timeout((HZ+99)/100);
+		}
 	}
 
+	snd_printk("cs461x: create - never read ISV3 & ISV4 from AC'97\n");
+	return -EIO;
+ok2:
 	/*
 	 *  Now, assert valid frame and the slot 3 and 4 valid bits.  This will
 	 *  commense the transfer of digital audio data to the AC97 codec.
@@ -1647,7 +1663,6 @@ int snd_cs461x_create(snd_card_t * card,
 	 */
 	if (snd_cs461x_download_image(codec) < 0) {
 		snd_printk("cs461x: image download error\n");
-		snd_cs461x_free(codec);
 		return -EIO;
 	}
 
@@ -1665,6 +1680,8 @@ int snd_cs461x_create(snd_card_t * card,
 	codec->cctl = tmp & 0x0000ffff;
 	snd_cs461x_poke(codec, BA1_CCTL, tmp & 0xffff0000);
 
+	mdelay(5);		/* Do we need this ?? */
+	
 	snd_cs461x_powerup_adc(codec);
 	snd_cs461x_powerup_dac(codec);
 
@@ -1686,26 +1703,17 @@ int snd_cs461x_create(snd_card_t * card,
 	tmp &= ~0x0000003f;
 	tmp |=  0x00000001;
 	snd_cs461x_poke(codec, BA1_CIE, tmp);	/* capture interrupt enable */	
-
-	snd_cs461x_proc_init(card, codec);
-
-	if ((err = snd_device_new(card, SND_DEV_LOWLEVEL, codec, 0, &ops, NULL)) < 0) {
-		snd_cs461x_free(codec);
-		return err;
-	}
-
-	*rcodec = codec;
 	return 0;
 }
 
-int snd_cs461x_free(cs461x_t * codec)
+
+/*
+ * stop the h/w
+ */
+static void snd_cs461x_hw_stop(cs461x_t *codec)
 {
 	unsigned int tmp;
 
-	snd_debug_check(codec == NULL, -EINVAL);
-
-	snd_cs461x_proc_done(codec);
-
 	tmp = snd_cs461x_peek(codec, BA1_PFIE);
 	tmp &= ~0x0000f03f;
 	tmp |=  0x00000010;
@@ -1752,6 +1760,386 @@ int snd_cs461x_free(cs461x_t * codec)
 	 */
 	tmp = snd_cs461x_peekBA0(codec, BA0_CLKCR1) & ~CLKCR1_SWCE;
 	snd_cs461x_pokeBA0(codec, BA0_CLKCR1, tmp);
+}
+
+
+/*
+ *	AMP control - null AMP
+ */
+ 
+static void amp_none(cs461x_t *codec, int change)
+{	
+}
+
+/*
+ *	Crystal EAPD mode
+ */
+ 
+static void amp_voyetra(cs461x_t *codec, int change)
+{
+	/* Manage the EAPD bit on the Crystal 4297 
+	   and the Analog AD1885 */
+	   
+	int old = codec->amplifier;
+	
+	codec->amplifier += change;
+	if (codec->amplifier && !old) {
+		/* Turn the EAPD amp on */
+		snd_cs461x_codec_write(codec,  AC97_POWERDOWN, 
+				       snd_cs461x_codec_read(codec, AC97_POWERDOWN) |
+				       0x8000);
+	} else if (old && !codec->amplifier) {
+		/* Turn the EAPD amp off */
+		snd_cs461x_codec_write(codec,  AC97_POWERDOWN, 
+				       snd_cs461x_codec_read(codec, AC97_POWERDOWN) &
+				       ~0x8000);
+	}
+}
+
+
+/*
+ *	Game Theatre XP card - EGPIO[2] is used to enable the external amp.
+ */
+ 
+static void amp_hercules(cs461x_t *chip, int change)
+{
+	int old = chip->amplifier;
+
+	chip->amplifier += change;
+	if (chip->amplifier && !old) {
+		snd_cs461x_pokeBA0(chip, BA0_EGPIODR, 
+				   EGPIODR_GPOE2);     /* enable EGPIO2 output */
+		snd_cs461x_pokeBA0(chip, BA0_EGPIOPTR, 
+				   EGPIOPTR_GPPT2);   /* open-drain on output */
+	} else if (old && !chip->amplifier) {
+		snd_cs461x_pokeBA0(chip, BA0_EGPIODR, 0); /* disable */
+		snd_cs461x_pokeBA0(chip, BA0_EGPIOPTR, 0); /* disable */
+	}
+}
+
+
+#if 0
+/*
+ *	Untested
+ */
+ 
+static void amp_voyetra_4294(cs461x_t *codec, int change)
+{
+	codec->amplifier+=change;
+
+	if(codec->amplifier) {
+		/* Switch the GPIO pins 7 and 8 to open drain */
+		snd_cs461x_codec_write(codec, 0x4C,
+				       snd_cs461x_codec_read(codec, 0x4C) & 0xFE7F);
+		snd_cs461x_codec_write(codec, 0x4E,
+				       snd_cs461x_codec_read(codec, 0x4E) | 0x0180);
+		/* Now wake the AMP (this might be backwards) */
+		snd_cs461x_codec_write(codec, 0x54,
+				       snd_cs461x_codec_read(codec, 0x54) & ~0x0180);
+	} else {
+		snd_cs461x_codec_write(codec, 0x54,
+				       snd_cs461x_codec_read(codec, 0x54) | 0x0180);
+	}
+}
+#endif
+
+
+/*
+ * piix4 pci ids
+ */
+#ifndef PCI_VENDOR_ID_INTEL
+#define PCI_VENDOR_ID_INTEL 0x8086
+#endif /* PCI_VENDOR_ID_INTEL */
+
+#ifndef PCI_DEVICE_ID_INTEL_82371AB_3
+#define PCI_DEVICE_ID_INTEL_82371AB_3 0x7113
+#endif /* PCI_DEVICE_ID_INTEL_82371AB_3 */
+
+/*
+ *	Handle the CLKRUN on a thinkpad. We must disable CLKRUN support
+ *	whenever we need to beat on the chip.
+ *
+ *	The original idea and code for this hack comes from David Kaiser at
+ *	Linuxcare. Perhaps one day Crystal will document their chips well
+ *	enough to make them useful.
+ */
+ 
+static void clkrun_hack(cs461x_t *codec, int change)
+{
+	u16 control;
+	int old;
+	
+	if (codec->acpi_dev == NULL)
+		return;
+
+	old = codec->amplifier;
+	codec->amplifier += change;
+	
+	/* Read ACPI port */	
+	control = inw(codec->acpi_port + 0x10);
+
+	/* Flip CLKRUN off while running */
+	if (! codec->amplifier && old)
+		outw(control | 0x2000, codec->acpi_port + 0x10);
+	else if (codec->amplifier && ! old)
+		outw(control & ~0x2000, codec->acpi_port + 0x10);
+}
+
+	
+/*
+ * detect intel piix4
+ */
+static void clkrun_init(cs461x_t *codec)
+{
+	u8 pp;
+
+	codec->acpi_dev = pci_find_device(PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82371AB_3, NULL);
+	if (codec->acpi_dev == NULL)
+		return;		/* Not a thinkpad thats for sure */
+
+	/* Find the control port */		
+	pci_read_config_byte(codec->acpi_dev, 0x41, &pp);
+	codec->acpi_port = pp << 8;
+}
+
+
+/*
+ * Card subid table
+ */
+ 
+struct cs_card_type
+{
+	u16 vendor;
+	u16 id;
+	char *name;
+	void (*init)(cs461x_t *);
+	void (*amp)(cs461x_t *, int);
+	void (*active)(cs461x_t *, int);
+};
+
+static struct cs_card_type __initdata cards[]={
+	{0x1489, 0x7001, "Genius Soundmaker 128 value", NULL, amp_none, NULL},
+	{0x5053, 0x3357, "Voyetra", NULL, amp_voyetra, NULL},
+	{0x1071, 0x6003, "Mitac MI6020/21", NULL, amp_voyetra, NULL},
+	{0x14AF, 0x0050, "Hercules Game Theatre XP", NULL, amp_hercules, NULL},
+	{0x1681, 0x0050, "Hercules Game Theatre XP", NULL, amp_hercules, NULL},
+	{0x1681, 0x0051, "Hercules Game Theatre XP", NULL, amp_hercules, NULL},
+	{0x1681, 0x0052, "Hercules Game Theatre XP", NULL, amp_hercules, NULL},
+	{0x1681, 0x0053, "Hercules Game Theatre XP", NULL, amp_hercules, NULL},
+	{0x1681, 0x0054, "Hercules Game Theatre XP", NULL, amp_hercules, NULL},
+	/* Not sure if the 570 needs the clkrun hack */
+	{PCI_VENDOR_ID_IBM, 0x0132, "Thinkpad 570", clkrun_init, NULL, clkrun_hack},
+	{PCI_VENDOR_ID_IBM, 0x0153, "Thinkpad 600X/A20/T20", clkrun_init, NULL, clkrun_hack},
+	{PCI_VENDOR_ID_IBM, 0x1010, "Thinkpad 600E (unsupported)", NULL, NULL, NULL},
+	{0, 0, "Card without SSID set", NULL, NULL, NULL },
+	{0, 0, NULL, NULL, NULL, NULL}
+};
+
+
+/*
+ * APM support
+ */
+#ifdef CONFIG_PM
+static int snd_cs461x_pm_callback(struct pm_dev *dev, pm_request_t rqst, void *data)
+{
+	cs461x_t *codec = snd_magic_cast(cs461x_t, dev->data, 0);
+	int i, amp_saved;
+
+	switch (rqst) {
+	case PM_SUSPEND:
+		if (codec->playback_subchn)
+			snd_cs461x_playback_trigger(codec, codec->playback_subchn, SND_PCM_TRIGGER_STOP);
+		
+		if (codec->capture_subchn)
+			snd_cs461x_capture_trigger(codec, codec->capture_subchn, SND_PCM_TRIGGER_STOP);
+
+		snd_cs461x_hw_stop(codec);
+		break;
+
+	case PM_RESUME:
+		amp_saved = codec->amplifier;
+		codec->amplifier = 0;
+		codec->active_ctrl(codec, 1); /* force to on */
+
+		snd_cs461x_chip_init(codec, 1);
+
+		snd_cs461x_codec_write(codec, AC97_MASTER, 0x8000);
+		for (i = 0; i < 100; i++) {
+			if (snd_cs461x_codec_read(codec, AC97_MASTER) == 0x8000)
+				break;
+			mdelay(10); /* Zzzz.. */
+		}
+
+		/* restore ac97 registers */
+		snd_ac97_resume(codec->ac97);
+#if 0
+		for (i = 2; i <= 0x20; i += 2) {
+			if (i == AC97_MASTER_TONE)
+				continue;
+			snd_cs461x_codec_write(codec, i, codec->ac97->regs[i]);
+		}
+#endif
+		if (amp_saved)
+			codec->amplifier_ctrl(codec, 1); /* try to turn on */
+		if (codec->playback_subchn) {
+			snd_cs461x_playback_prepare(codec, codec->playback_subchn);
+			snd_cs461x_playback_trigger(codec, codec->playback_subchn, SND_PCM_TRIGGER_GO);
+		}
+		if (codec->capture_subchn) {
+			snd_cs461x_capture_prepare(codec, codec->capture_subchn);
+			snd_cs461x_capture_trigger(codec, codec->capture_subchn, SND_PCM_TRIGGER_GO);
+		}
+		if (! amp_saved) {
+			codec->amplifier = 1;
+			codec->active_ctrl(codec, -1);
+		}
+		codec->amplifier = amp_saved;
+		break;
+	}
+	return 0;
+}
+#endif /* CONFIG_PM */
+
+
+/*
+ * create a cs461x card instance
+ */
+int snd_cs461x_create(snd_card_t * card,
+		      struct pci_dev * pci,
+		      snd_dma_t * dma1ptr,
+		      snd_dma_t * dma2ptr,
+		      snd_irq_t * irqptr,
+		      int external_amp, int thinkpad,
+		      cs461x_t ** rcodec)
+{
+	cs461x_t *codec;
+	unsigned short cmdw;
+	unsigned char cmdb;
+	int err;
+	struct cs_card_type *cp;
+	u16 ss_card, ss_vendor;
+	static snd_device_ops_t ops = {
+		(snd_dev_free_t *)snd_cs461x_free,
+		NULL,
+		NULL
+	};
+	
+	*rcodec = NULL;
+	codec = snd_magic_kcalloc(cs461x_t, 0, GFP_KERNEL);
+	if (codec == NULL)
+		return -ENOMEM;
+	spin_lock_init(&codec->reg_lock);
+	codec->card = card;
+	codec->pci = pci;
+	codec->dma1ptr = dma1ptr;
+	codec->dma2ptr = dma2ptr;
+	codec->irqptr = irqptr;
+	codec->ba0_addr = pci_resource_start(pci, 0);
+	codec->ba1_addr = pci_resource_start(pci, 1);
+	if (codec->ba0_addr == 0 || codec->ba0_addr == ~0 ||
+	    codec->ba1_addr == 0 || codec->ba1_addr == ~0) {
+	    	snd_printk("cs461x: wrong address(es) - ba0 = 0x%lx, ba1 = 0x%lx\n", codec->ba0_addr, codec->ba1_addr);
+	    	snd_cs461x_free(codec);
+	    	return -ENOMEM;
+	}
+
+	/* set up amp and clkrun hack */
+	pci_read_config_word(pci, PCI_SUBSYSTEM_VENDOR_ID, &ss_vendor);
+	pci_read_config_word(pci, PCI_SUBSYSTEM_ID, &ss_card);
+
+	for (cp = &cards[0]; cp->name; cp++) {
+		if (cp->vendor == ss_vendor && cp->id == ss_card) {
+			snd_printk("cs461x: hack for %s enabled\n", cp->name);
+			if (cp->init)
+				cp->init(codec);
+			codec->amplifier_ctrl = cp->amp;
+			codec->active_ctrl = cp->active;
+			break;
+		}
+	}
+
+	if (external_amp) {
+		snd_printk("cs461x: Crystal EAPD support forced on.\n");
+		codec->amplifier_ctrl = amp_voyetra;
+	}
+
+	if (thinkpad) {
+		snd_printk("cs461x: Activating CLKRUN hack for Thinkpad.\n");
+		codec->active_ctrl = clkrun_hack;
+		clkrun_init(codec);
+	}
+	
+	if (codec->amplifier_ctrl == NULL)
+		codec->amplifier_ctrl = amp_none;
+	if (codec->active_ctrl == NULL)
+		codec->active_ctrl = amp_none;
+
+	codec->active_ctrl(codec, 1);
+
+	codec->ba0 = ioremap(codec->ba0_addr, CS461X_BA0_SIZE);
+	codec->ba1.name.data0 = ioremap(codec->ba1_addr + BA1_SP_DMEM0, CS461X_BA1_DATA0_SIZE);
+	codec->ba1.name.data1 = ioremap(codec->ba1_addr + BA1_SP_DMEM1, CS461X_BA1_DATA1_SIZE);
+	codec->ba1.name.pmem = ioremap(codec->ba1_addr + BA1_SP_PMEM, CS461X_BA1_PRG_SIZE);
+	codec->ba1.name.reg = ioremap(codec->ba1_addr + BA1_SP_REG, CS461X_BA1_REG_SIZE);
+	if (codec->ba0 == NULL || codec->ba1.name.data0 == NULL ||
+	    codec->ba1.name.data1 == NULL || codec->ba1.name.pmem == NULL ||
+	    codec->ba1.name.reg == NULL) {
+		snd_cs461x_free(codec);
+		return -ENOMEM;
+	}
+
+	pci_set_master(pci);
+	pci_read_config_word(pci, PCI_COMMAND, &cmdw);
+	if ((cmdw & PCI_COMMAND_MEMORY) != PCI_COMMAND_MEMORY) {
+		cmdw |= PCI_COMMAND_MEMORY;
+		pci_write_config_word(pci, PCI_COMMAND, cmdw);
+	}
+	pci_read_config_byte(pci, PCI_LATENCY_TIMER, &cmdb);
+	if (cmdb < 32)
+		cmdb = 32;
+	pci_write_config_byte(pci, PCI_LATENCY_TIMER, cmdb);
+
+	err = snd_cs461x_chip_init(codec, 0);
+	if (err < 0) {
+		snd_cs461x_free(codec);
+		return err;
+	}
+
+	snd_cs461x_proc_init(card, codec);
+
+#ifdef CONFIG_PM
+	codec->pm_dev = pm_register(PM_PCI_DEV, 0, snd_cs461x_pm_callback);
+	if (codec->pm_dev)
+		codec->pm_dev->data = codec;
+#endif
+
+	if ((err = snd_device_new(card, SND_DEV_LOWLEVEL, codec, 0, &ops, NULL)) < 0) {
+		snd_cs461x_free(codec);
+		return err;
+	}
+
+	codec->active_ctrl(codec, -1);
+
+	*rcodec = codec;
+	return 0;
+}
+
+int snd_cs461x_free(cs461x_t * codec)
+{
+	snd_debug_check(codec == NULL, -EINVAL);
+
+	if (codec->active_ctrl)
+		codec->active_ctrl(codec, 1);
+
+#ifdef CONFIG_PM
+	if (codec->pm_dev)
+		pm_unregister(codec->pm_dev);
+#endif
+
+	snd_cs461x_proc_done(codec);
+
+	if (codec->ba0)
+		snd_cs461x_hw_stop(codec);
 
 	if (codec->ba0)
 		iounmap(codec->ba0);
@@ -1764,6 +2152,11 @@ int snd_cs461x_free(cs461x_t * codec)
 	if (codec->ba1.name.reg)
 		iounmap(codec->ba1.name.reg);
 
+	if (codec->active_ctrl)
+		codec->active_ctrl(codec, -codec->amplifier); /* force to off */
+	if (codec->amplifier_ctrl)
+		codec->amplifier_ctrl(codec, -codec->amplifier); /* force to off */
+
 	snd_magic_kfree(codec);
 	return 0;
 }
@@ -1778,15 +2171,16 @@ EXPORT_SYMBOL(snd_cs461x_midi);
  *  INIT part
  */
 
-#ifdef MODULE
-
-int __init init_module(void)
+static int __init alsa_cs461x_init(void)
 {
 	return 0;
 }
 
-void __exit cleanup_module(void)
+static void __exit alsa_cs461x_exit(void)
 {
 }
 
-#endif
+module_init(alsa_cs461x_init)
+module_exit(alsa_cs461x_exit)
+
+MODULE_LICENSE("GPL");
