@@ -99,13 +99,13 @@ static void snd_sb8_compute_rate(snd_pcm_subchn_t * subchn)
 			}
 		}
 		runtime->format.rate = (256000000 / (65536 - (codec->speed8 << 8))) / runtime->format.voices;
-#if 0
-		snd_printd(__FUNCTION__ ": rate = %u, pchn->real_rate = %u\n", rate, pchn->real_rate);
-#endif
 		break;
 	default:
 		snd_printd("unknown hardware for sb8 compute rate!!!\n");
 	}
+#if 0
+	snd_printd(__FUNCTION__ ": rate = %u, pchn->real_rate = %u\n", rate, runtime->format.rate);
+#endif
 }
 
 static int snd_sb8_playback_ioctl(void *private_data,
@@ -149,7 +149,7 @@ static int snd_sb8_playback_prepare(void *private_data,
 		snd_sb8_ack_8bit(codec);
 		snd_sb8dsp_command(codec, SB_DSP_SPEAKER_ON);
 		spin_unlock_irqrestore(&codec->reg_lock, flags);
-		snd_dma_program(codec->dma8, &runtime->dma_area->buf[runtime->buf_position % codec->p_dma_size], codec->p_dma_size, DMA_MODE_WRITE);
+		snd_dma_program(codec->dma8, runtime->dma_area->buf, codec->p_dma_size, DMA_MODE_WRITE);
 		break;
 	case SB_HW_20:
 	case SB_HW_201:
@@ -216,6 +216,7 @@ static int snd_sb8_playback_trigger(void *private_data,
 			snd_sb8dsp_reset(codec);
 			codec->mode8 = SB_MODE8_HALT;
 		} else {
+			spin_unlock_irqrestore(&codec->reg_lock, flags);
 			return -EINVAL;
 		}
 		break;
@@ -231,6 +232,7 @@ static int snd_sb8_playback_trigger(void *private_data,
 			} else if (cmd == SND_PCM_TRIGGER_STOP) {
 				snd_sb8dsp_reset(codec);
 			} else {
+				spin_unlock_irqrestore(&codec->reg_lock, flags);
 				return -EINVAL;
 			}
 		} else {
@@ -239,6 +241,7 @@ static int snd_sb8_playback_trigger(void *private_data,
 			} else if (cmd == SND_PCM_TRIGGER_STOP) {
 				snd_sb8dsp_command(codec, SB_DSP_DMA8_OFF);
 			} else {
+				spin_unlock_irqrestore(&codec->reg_lock, flags);
 				return -EINVAL;
 			}
 		}
@@ -267,7 +270,7 @@ static int snd_sb8_capture_prepare(void *private_data,
 		snd_sb8dsp_command(codec, codec->speed8);
 		snd_sb8_ack_8bit(codec);
 		spin_unlock_irqrestore(&codec->reg_lock, flags);
-		snd_dma_program(codec->dma8, &runtime->dma_area->buf[runtime->buf_position % codec->c_dma_size], codec->c_dma_size, DMA_MODE_READ);
+		snd_dma_program(codec->dma8, runtime->dma_area->buf, codec->c_dma_size, DMA_MODE_READ);
 		break;
 	case SB_HW_20:
 	case SB_HW_201:
@@ -287,7 +290,7 @@ static int snd_sb8_capture_prepare(void *private_data,
 			} else {
 				snd_sb8mixer_write(&codec->mixer, 0x0e, snd_sb8mixer_read(&codec->mixer, 0x0e) & ~0x02);
 			}
-			spin_unlock_irqrestore(&codec->mixer.lock, &flags1);
+			spin_unlock_irqrestore(&codec->mixer.lock, flags1);
 		}
 		if (codec->hardware == SB_HW_PRO) {
 			snd_sb8dsp_command(codec, runtime->format.voices > 1 ? SB_DSP_STEREO_8BIT : SB_DSP_MONO_8BIT);
@@ -336,6 +339,7 @@ static int snd_sb8_capture_trigger(void *private_data,
 			snd_sb8dsp_reset(codec);
 			codec->mode8 = SB_MODE8_HALT;
 		} else {
+			spin_unlock_irqrestore(&codec->reg_lock, flags);
 			return -EINVAL;
 		}
 		break;
@@ -351,6 +355,7 @@ static int snd_sb8_capture_trigger(void *private_data,
 			} else if (cmd == SND_PCM_TRIGGER_STOP) {
 				snd_sb8dsp_reset(codec);
 			} else {
+				spin_unlock_irqrestore(&codec->reg_lock, flags);
 				return -EINVAL;
 			}
 		} else {
@@ -359,6 +364,7 @@ static int snd_sb8_capture_trigger(void *private_data,
 			} else if (cmd == SND_PCM_TRIGGER_STOP) {
 				snd_sb8dsp_command(codec, SB_DSP_DMA8_OFF);
 			} else {
+				spin_unlock_irqrestore(&codec->reg_lock, flags);
 				return -EINVAL;
 			}
 		}
@@ -384,7 +390,7 @@ void snd_sb8dsp_interrupt(snd_pcm_t * pcm)
 		if ((codec->hardware == SB_HW_10 ||
 		     codec->hardware == SB_HW_20) &&
 		    *subchn->runtime->status == SND_PCM_STATUS_RUNNING) {
-		    	snd_sb8_playback_prepare(runtime->private_data, subchn);
+			snd_dma_program(codec->dma8, &runtime->dma_area->buf[runtime->buf_position % codec->p_dma_size], codec->p_dma_size, DMA_MODE_WRITE);
 		    	snd_sb8_playback_trigger(runtime->private_data, subchn, SND_PCM_TRIGGER_GO);
 		}
 		break;
@@ -395,7 +401,7 @@ void snd_sb8dsp_interrupt(snd_pcm_t * pcm)
 		if ((codec->hardware == SB_HW_10 ||
 		     codec->hardware == SB_HW_20) &&
 		    *subchn->runtime->status == SND_PCM_STATUS_RUNNING) {
-		    	snd_sb8_capture_prepare(runtime->private_data, subchn);
+			snd_dma_program(codec->dma8, &runtime->dma_area->buf[runtime->buf_position % codec->c_dma_size], codec->c_dma_size, DMA_MODE_READ);
 		    	snd_sb8_capture_trigger(runtime->private_data, subchn, SND_PCM_TRIGGER_GO);
 		}
 		break;
@@ -432,7 +438,7 @@ static snd_pcm_hardware_t snd_sb8_playback =
 	SND_PCM_CHNINFO_BLOCK | SND_PCM_CHNINFO_INTERLEAVE |
 	SND_PCM_CHNINFO_MMAP_VALID,	/* flags */
 	SND_PCM_FMT_U8,		/* formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000 |
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000 |
 	SND_PCM_RATE_11025 | SND_PCM_RATE_22050,
 	4000,			/* min. rate */
 	23000,			/* max. rate */
@@ -446,7 +452,6 @@ static snd_pcm_hardware_t snd_sb8_playback =
 	snd_sb8_playback_ioctl,
 	snd_sb8_playback_prepare,
 	snd_sb8_playback_trigger,
-	snd_pcm_playback_write,
 	snd_sb8_playback_pointer
 };
 
@@ -456,7 +461,7 @@ static snd_pcm_hardware_t snd_sb8_capture =
 	SND_PCM_CHNINFO_BLOCK | SND_PCM_CHNINFO_INTERLEAVE |
 	SND_PCM_CHNINFO_MMAP_VALID,	/* flags */
 	SND_PCM_FMT_U8,		/* formats */
-	SND_PCM_RATE_PLL | SND_PCM_RATE_8000 |
+	SND_PCM_RATE_CONTINUOUS | SND_PCM_RATE_8000 |
 	SND_PCM_RATE_11025,
 	4000,			/* min. rate */
 	13000,			/* max. rate */
@@ -470,7 +475,6 @@ static snd_pcm_hardware_t snd_sb8_capture =
 	snd_sb8_capture_ioctl,
 	snd_sb8_capture_prepare,
 	snd_sb8_capture_trigger,
-	snd_pcm_capture_read,
 	snd_sb8_capture_pointer
 };
 
@@ -490,13 +494,18 @@ int snd_sb8_playback_open(void *private_data, snd_pcm_subchn_t *subchn)
 		spin_unlock_irqrestore(&codec->open8_lock, flags);
 		return -EAGAIN;
 	}
-	codec->playback_subchn = subchn;
+	if (subchn->channel == SND_PCM_CHANNEL_PLAYBACK) {
+		codec->playback_subchn = subchn;
+	} else {
+		codec->capture_subchn = subchn;
+	}
 	codec->open8 |= SB_OPEN_PCM;
 	spin_unlock_irqrestore(&codec->open8_lock, flags);
 	if ((err = snd_pcm_dma_alloc(subchn, codec->dma8ptr, "Sound Blaster DSP")) < 0) {
 	      __error:
 		spin_lock_irqsave(&codec->open8_lock, flags);
 		codec->playback_subchn = NULL;
+		codec->capture_subchn = NULL;
 		codec->open8 &= ~SB_OPEN_PCM;
 		spin_unlock_irqrestore(&codec->open8_lock, flags);
 		return err;
@@ -507,7 +516,7 @@ int snd_sb8_playback_open(void *private_data, snd_pcm_subchn_t *subchn)
 		snd_pcm_dma_free(subchn);
 		goto __error;
 	}
-	runtime->hw_free = _snd_kfree;
+	runtime->hw_free = (snd_kfree_type)_snd_kfree;
 	snd_pcm_set_mixer(subchn, codec->kmixer->device, codec->mixer.me_playback);
 	runtime->private_data = codec;
 	if (subchn->channel == SND_PCM_CHANNEL_PLAYBACK) {
@@ -531,6 +540,7 @@ int snd_sb8_playback_close(void *private_data, snd_pcm_subchn_t *subchn)
 	unsigned long flags;
 	sbdsp_t *codec = snd_magic_cast(sbdsp_t, private_data, -ENXIO);
 
+	codec->playback_subchn = NULL;
 	codec->capture_subchn = NULL;
 	snd_pcm_dma_free(subchn);
 	spin_lock_irqsave(&codec->open8_lock, flags);
